@@ -1,15 +1,14 @@
-/* Copyright (c) 2002-2016 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
 #include "istream.h"
 #include "istream-seekable.h"
-#include "fd-set-nonblock.h"
 #include "str.h"
 #include "str-sanitize.h"
 #include "strescape.h"
 #include "safe-mkstemp.h"
-#include "abspath.h"
+#include "path-util.h"
 #include "message-address.h"
 #include "mbox-from.h"
 #include "raw-storage.h"
@@ -68,9 +67,8 @@ static int seekable_fd_callback
 	}
 
 	/* we just want the fd, unlink it */
-	if (unlink(str_c(path)) < 0) {
+	if (i_unlink(str_c(path)) < 0) {
 		/* shouldn't happen.. */
-		i_error("unlink(%s) failed: %m", str_c(path));
 		i_close_fd(&fd);
 		return -1;
 	}
@@ -91,14 +89,14 @@ static struct istream *mail_raw_create_stream
 	*mtime_r = (time_t)-1;
 	fd_set_nonblock(fd, FALSE);
 
-	input = i_stream_create_fd(fd, 4096, FALSE);
+	input = i_stream_create_fd(fd, 4096);
 	input->blocking = TRUE;
 	/* If input begins with a From-line, drop it */
-	ret = i_stream_read_data(input, &data, &size, 5);
-	if (ret > 0 && size >= 5 && memcmp(data, "From ", 5) == 0) {
+	ret = i_stream_read_bytes(input, &data, &size, 5);
+	if (ret > 0 && memcmp(data, "From ", 5) == 0) {
 		/* skip until the first LF */
 		i_stream_skip(input, 5);
-		while ( i_stream_read_data(input, &data, &size, 0) > 0 ) {
+		while ( i_stream_read_more(input, &data, &size) > 0 ) {
 			for (i = 0; i < size; i++) {
 				if (data[i] == '\n')
 					break;
@@ -154,11 +152,13 @@ static struct mail_raw *mail_raw_create
 {
 	struct mail_raw *mailr;
 	struct mailbox_header_lookup_ctx *headers_ctx;
-	const char *envelope_sender;
+	const char *envelope_sender, *error;
 	int ret;
 
 	if ( mailfile != NULL && *mailfile != '/' )
-		mailfile = t_abspath(mailfile);
+		if (t_abspath(mailfile, &mailfile, &error) < 0)
+			i_fatal("t_abspath(%s) failed: %s",
+				mailfile, error);
 
 	mailr = i_new(struct mail_raw, 1);
 
@@ -181,7 +181,7 @@ static struct mail_raw *mail_raw_create
 		}
 	}
 
-	mailr->trans = mailbox_transaction_begin(mailr->box, 0);
+	mailr->trans = mailbox_transaction_begin(mailr->box, 0, __func__);
 	headers_ctx = mailbox_header_lookup_init(mailr->box, wanted_headers);
 	mailr->mail = mail_alloc(mailr->trans, 0, headers_ctx);
 	mailbox_header_lookup_unref(&headers_ctx);
@@ -220,9 +220,7 @@ struct mail_raw *mail_raw_open_file
 	}
 
 	mailr = mail_raw_create(ruser, input, path, sender, mtime);
-
-	if ( input != NULL )
-		i_stream_unref(&input);
+	i_stream_unref(&input);
 
 	return mailr;
 }
