@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2016 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 /* Extension encoded-character
@@ -41,10 +41,8 @@ bool arg_encoded_string_validate
 		struct sieve_command *context);
 
 const struct sieve_argument_def encoded_string_argument = {
-	"@encoded-string",
-	NULL,
-	arg_encoded_string_validate,
-	NULL, NULL, NULL
+	.identifier = "@encoded-string",
+	.validate = arg_encoded_string_validate
 };
 
 /* Parsing */
@@ -122,8 +120,9 @@ static bool _decode_hex
 	return ( values > 0 );
 }
 
-static int _decode_unicode
-(const char **in, const char *inend, string_t *result, unsigned int *error_hex)
+static bool _decode_unicode
+(const char **in, const char *inend, string_t *result,
+	unsigned int *error_hex)
 {
 	int values = 0;
 	bool valid = TRUE;
@@ -135,8 +134,7 @@ static int _decode_unicode
 
 		if ( !_parse_hexint(in, inend, 0, &unicode_hex) ) break;
 
-		if ( (unicode_hex <= 0xD7FF) ||
-			(unicode_hex >= 0xE000 && unicode_hex <= 0x10FFFF)	)
+		if ( uni_is_valid_ucs4((unichar_t) unicode_hex) )
 			uni_ucs4_to_utf8_c((unichar_t) unicode_hex, result);
 		else {
 			if ( valid ) *error_hex = unicode_hex;
