@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2013 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 /* Extension envelope
@@ -45,13 +45,38 @@ const struct sieve_extension_def envelope_extension;
  */
 
 static bool ext_envelope_validator_load
-(const struct sieve_extension *ext, struct sieve_validator *valdtr);
+	(const struct sieve_extension *ext, struct sieve_validator *valdtr);
+static bool ext_envelope_interpreter_load
+	(const struct sieve_extension *ext,
+		const struct sieve_runtime_env *renv,
+		sieve_size_t *address);
+
+static bool ext_envelope_validator_validate
+	(const struct sieve_extension *ext,
+		struct sieve_validator *valdtr, void *context,
+		struct sieve_ast_argument *require_arg,
+		bool required);
+static int ext_envelope_interpreter_run
+	(const struct sieve_extension *this_ext,
+		const struct sieve_runtime_env *renv,
+		void *context, bool deferred);
 
 const struct sieve_extension_def envelope_extension = {
 	.name = "envelope",
+	.interpreter_load = ext_envelope_interpreter_load,
 	.validator_load = ext_envelope_validator_load,
 	SIEVE_EXT_DEFINE_OPERATION(envelope_operation)
 };
+const struct sieve_validator_extension
+envelope_validator_extension = {
+	.ext = &envelope_extension,
+	.validate = ext_envelope_validator_validate
+};
+const struct sieve_interpreter_extension
+envelope_interpreter_extension = {
+	.ext_def = &envelope_extension,
+	.run = ext_envelope_interpreter_run
+};
 
 static bool ext_envelope_validator_load
 (const struct sieve_extension *ext, struct sieve_validator *valdtr)
@@ -59,9 +84,59 @@ static bool ext_envelope_validator_load
 	/* Register new test */
 	sieve_validator_register_command(valdtr, ext, &envelope_test);
 
+	sieve_validator_extension_register
+		(valdtr, ext, &envelope_validator_extension, NULL);
 	return TRUE;
 }
 
+static bool ext_envelope_interpreter_load
+(const struct sieve_extension *ext,
+	const struct sieve_runtime_env *renv,
+	sieve_size_t *address ATTR_UNUSED)
+{
+	sieve_interpreter_extension_register
+		(renv->interp, ext, &envelope_interpreter_extension, NULL);
+	return TRUE;
+}
+
+static bool ext_envelope_validator_validate
+(const struct sieve_extension *ext,
+	struct sieve_validator *valdtr, void *context ATTR_UNUSED,
+	struct sieve_ast_argument *require_arg,
+	bool required)
+{
+	if (required) {
+		enum sieve_compile_flags flags =
+			sieve_validator_compile_flags(valdtr);
+
+		if ( (flags & SIEVE_COMPILE_FLAG_NO_ENVELOPE) != 0 ) {
+			sieve_argument_validate_error(valdtr, require_arg,
+				"the %s extension cannot be used in this context "
+				"(needs access to message envelope)",
+				sieve_extension_name(ext));
+			return FALSE;
+		}
+	}
+	return TRUE;
+}
+
+static int ext_envelope_interpreter_run
+(const struct sieve_extension *ext,
+	const struct sieve_runtime_env *renv,
+	void *context ATTR_UNUSED, bool deferred)
+{
+	if ( (renv->flags & SIEVE_EXECUTE_FLAG_NO_ENVELOPE) != 0 ) {
+		if ( !deferred ) {
+			sieve_runtime_error(renv, NULL,
+				"the %s extension cannot be used in this context "
+				"(needs access to message envelope)",
+				sieve_extension_name(ext));
+		}
+		return SIEVE_EXEC_FAILURE;
+	}
+	return SIEVE_EXEC_OK;
+}
+
 /*
  * Envelope test
  *
@@ -79,15 +154,15 @@ static bool tst_envelope_generate
 	(const struct sieve_codegen_env *cgenv, struct sieve_command *ctx);
 
 static const struct sieve_command_def envelope_test = {
-	"envelope",
-	SCT_TEST,
-	2, 0, FALSE, FALSE,
-	tst_envelope_registered,
-	NULL,
-	tst_envelope_validate,
-	NULL,
-	tst_envelope_generate,
-	NULL
+	.identifier = "envelope",
+	.type = SCT_TEST,
+	.positional_args= 2,
+	.subtests = 0,
+	.block_allowed = FALSE,
+	.block_required = FALSE,
+	.registered = tst_envelope_registered,
+	.validate = tst_envelope_validate,
+	.generate = tst_envelope_generate
 };
 
 /*
@@ -100,11 +175,10 @@ static int ext_envelope_operation_execute
 	(const struct sieve_runtime_env *renv, sieve_size_t *address);
 
 const struct sieve_operation_def envelope_operation = {
-	"ENVELOPE",
-	&envelope_extension,
-	0,
-	ext_envelope_operation_dump,
-	ext_envelope_operation_execute
+	.mnemonic = "ENVELOPE",
+	.ext_def = &envelope_extension,
+	.dump = ext_envelope_operation_dump,
+	.execute = ext_envelope_operation_execute
 };
 
 /*
@@ -116,17 +190,17 @@ const struct sieve_operation_def envelope_operation = {
 struct sieve_envelope_part {
 	const char *identifier;
 
-	const struct sieve_address *const *(*get_addresses)
+	const struct smtp_address *const *(*get_addresses)
 		(const struct sieve_runtime_env *renv);
 	const char * const *(*get_values)
 		(const struct sieve_runtime_env *renv);
 };
 
-static const struct sieve_address *const *_from_part_get_addresses
+static const struct smtp_address *const *_from_part_get_addresses
 	(const struct sieve_runtime_env *renv);
 static const char *const *_from_part_get_values
 	(const struct sieve_runtime_env *renv);
-static const struct sieve_address *const *_to_part_get_addresses
+static const struct smtp_address *const *_to_part_get_addresses
 	(const struct sieve_runtime_env *renv);
 static const char *const *_to_part_get_values
 	(const struct sieve_runtime_env *renv);
@@ -177,49 +251,51 @@ static const struct sieve_envelope_part *_envelope_part_find
 
 /* Envelope parts implementation */
 
-static const struct sieve_address *const *_from_part_get_addresses
+static const struct smtp_address *const *_from_part_get_addresses
 (const struct sieve_runtime_env *renv)
 {
-	ARRAY(const struct sieve_address *) envelope_values;
-	const struct sieve_address *address =
-		sieve_message_get_sender_address(renv->msgctx);
+	ARRAY(const struct smtp_address *) envelope_values;
+	const struct smtp_address *address =
+		sieve_message_get_sender(renv->msgctx);
 
-	if ( address != NULL ) {
-		t_array_init(&envelope_values, 2);
+	t_array_init(&envelope_values, 2);
 
-        array_append(&envelope_values, &address, 1);
+	if ( address == NULL )
+		address = smtp_address_create_temp(NULL, NULL);
+	array_append(&envelope_values, &address, 1);
 
-	    (void)array_append_space(&envelope_values);
-    	return array_idx(&envelope_values, 0);
-	}
-
-	return NULL;
+  (void)array_append_space(&envelope_values);
+	return array_idx(&envelope_values, 0);
 }
 
 static const char *const *_from_part_get_values
 (const struct sieve_runtime_env *renv)
 {
 	ARRAY(const char *) envelope_values;
+	const struct smtp_address *address =
+		sieve_message_get_sender(renv->msgctx);
+	const char *value;
 
 	t_array_init(&envelope_values, 2);
 
-	if ( renv->msgdata->return_path != NULL ) {
-        array_append(&envelope_values, &renv->msgdata->return_path, 1);
-	}
+	value = "";
+	if ( !smtp_address_isnull(address) )
+		value = smtp_address_encode(address);
+	array_append(&envelope_values, &value, 1);
 
 	(void)array_append_space(&envelope_values);
 
 	return array_idx(&envelope_values, 0);
 }
 
-static const struct sieve_address *const *_to_part_get_addresses
+static const struct smtp_address *const *_to_part_get_addresses
 (const struct sieve_runtime_env *renv)
 {
-	ARRAY(const struct sieve_address *) envelope_values;
-	const struct sieve_address *address =
-		sieve_message_get_orig_recipient_address(renv->msgctx);
+	ARRAY(const struct smtp_address *) envelope_values;
+	const struct smtp_address *address =
+		sieve_message_get_orig_recipient(renv->msgctx);
 
-	if ( address != NULL && address->local_part != NULL ) {
+	if ( address != NULL && address->localpart != NULL ) {
 		t_array_init(&envelope_values, 2);
 
 		array_append(&envelope_values, &address, 1);
@@ -235,11 +311,14 @@ static const char *const *_to_part_get_values
 (const struct sieve_runtime_env *renv)
 {
 	ARRAY(const char *) envelope_values;
+	const struct smtp_address *address =
+		sieve_message_get_orig_recipient(renv->msgctx);
 
 	t_array_init(&envelope_values, 2);
 
-	if ( renv->msgdata->orig_envelope_to != NULL ) {
-        array_append(&envelope_values, &renv->msgdata->orig_envelope_to, 1);
+	if ( address != NULL && address->localpart != NULL) {
+		const char *value = smtp_address_encode(address);
+		array_append(&envelope_values, &value, 1);
 	}
 
 	(void)array_append_space(&envelope_values);
@@ -255,7 +334,7 @@ static const char *const *_auth_part_get_values
 	t_array_init(&envelope_values, 2);
 
 	if ( renv->msgdata->auth_user != NULL )
-        array_append(&envelope_values, &renv->msgdata->auth_user, 1);
+		array_append(&envelope_values, &renv->msgdata->auth_user, 1);
 
 	(void)array_append_space(&envelope_values);
 
@@ -271,7 +350,7 @@ static const char *const *_auth_part_get_values
 static int sieve_envelope_address_list_next_string_item
 	(struct sieve_stringlist *_strlist, string_t **str_r);
 static int sieve_envelope_address_list_next_item
-	(struct sieve_address_list *_addrlist, struct sieve_address *addr_r,
+	(struct sieve_address_list *_addrlist, struct smtp_address *addr_r,
 		string_t **unparsed_r);
 static void sieve_envelope_address_list_reset
 	(struct sieve_stringlist *_strlist);
@@ -283,7 +362,7 @@ struct sieve_envelope_address_list {
 
 	struct sieve_stringlist *env_parts;
 
-	const struct sieve_address *const *cur_addresses;
+	const struct smtp_address *const *cur_addresses;
 	const char * const *cur_values;
 
 	int value_index;
@@ -307,14 +386,15 @@ static struct sieve_address_list *sieve_envelope_address_list_create
 }
 
 static int sieve_envelope_address_list_next_item
-(struct sieve_address_list *_addrlist, struct sieve_address *addr_r,
+(struct sieve_address_list *_addrlist, struct smtp_address *addr_r,
 	string_t **unparsed_r)
 {
 	struct sieve_envelope_address_list *addrlist =
 		(struct sieve_envelope_address_list *) _addrlist;
 	const struct sieve_runtime_env *renv = _addrlist->strlist.runenv;
 
-	if ( addr_r != NULL ) addr_r->local_part = NULL;
+	if ( addr_r != NULL )
+		smtp_address_init(addr_r, NULL, NULL);
 	if ( unparsed_r != NULL ) *unparsed_r = NULL;
 
 	while ( addrlist->cur_addresses == NULL && addrlist->cur_values == NULL ) {
@@ -359,10 +439,10 @@ static int sieve_envelope_address_list_next_item
 
 	/* Return next item */
 	if ( addrlist->cur_addresses != NULL ) {
-		const struct sieve_address *addr =
+		const struct smtp_address *addr =
 			addrlist->cur_addresses[addrlist->value_index];
 
-		if ( addr->local_part == NULL ) {
+		if ( addr->localpart == NULL ) {
 			/* Null path <> */
 			if ( unparsed_r != NULL )
 				*unparsed_r = t_str_new_const("", 0);
@@ -398,17 +478,19 @@ static int sieve_envelope_address_list_next_item
 static int sieve_envelope_address_list_next_string_item
 (struct sieve_stringlist *_strlist, string_t **str_r)
 {
-	struct sieve_address_list *addrlist = (struct sieve_address_list *)_strlist;
-	struct sieve_address addr;
+	struct sieve_address_list *addrlist =
+		(struct sieve_address_list *)_strlist;
+	struct smtp_address addr;
 	int ret;
 
 	if ( (ret=sieve_envelope_address_list_next_item
 		(addrlist, &addr, str_r)) <= 0 )
 		return ret;
 
-	if ( addr.local_part != NULL ) {
-		const char *addr_str = sieve_address_to_string(&addr);
-		*str_r = t_str_new_const(addr_str, strlen(addr_str));
+	if ( addr.localpart != NULL ) {
+		const char *addr_str = smtp_address_encode(&addr);
+		if (str_r != NULL)
+			*str_r = t_str_new_const(addr_str, strlen(addr_str));
 	}
 
 	return 1;
@@ -435,10 +517,9 @@ static bool tst_envelope_registered
 	struct sieve_command_registration *cmd_reg)
 {
 	/* The order of these is not significant */
-	sieve_comparators_link_tag(valdtr, cmd_reg, SIEVE_AM_OPT_COMPARATOR);
+	sieve_comparators_link_tag(valdtr, cmd_reg, SIEVE_MATCH_OPT_COMPARATOR);
+	sieve_match_types_link_tags(valdtr, cmd_reg, SIEVE_MATCH_OPT_MATCH_TYPE);
 	sieve_address_parts_link_tags(valdtr, cmd_reg, SIEVE_AM_OPT_ADDRESS_PART);
-	sieve_match_types_link_tags(valdtr, cmd_reg, SIEVE_AM_OPT_MATCH_TYPE);
-
 	return TRUE;
 }
 
@@ -461,13 +542,13 @@ static int _envelope_part_is_supported
 					*not_address = epart;
 			}
 
-			return TRUE;
+			return 1;
 		}
 
-		return FALSE;
+		return 0;
 	}
 
-	return TRUE; /* Can't check at compile time */
+	return 1; /* Can't check at compile time */
 }
 
 static bool tst_envelope_validate
@@ -493,9 +574,9 @@ static bool tst_envelope_validate
 	 *   FIXME: verify dynamic envelope parts at runtime
 	 */
 	epart = arg;
-	if ( !sieve_ast_stringlist_map(&epart, (void *) &not_address,
-		_envelope_part_is_supported) ) {
-
+	if ( sieve_ast_stringlist_map(&epart, (void *) &not_address,
+		_envelope_part_is_supported) <= 0 ) {
+		i_assert(epart != NULL);
 		sieve_argument_validate_error(valdtr, epart,
 			"specified envelope part '%s' is not supported by the envelope test",
 				str_sanitize(sieve_ast_strlist_strc(epart), 64));
