@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2016 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 /* Extension reject
@@ -50,16 +50,40 @@ static const struct sieve_operation_def ereject_operation;
  * Extensions
  */
 
+static bool ext_reject_validator_validate
+	(const struct sieve_extension *ext,
+		struct sieve_validator *valdtr, void *context,
+		struct sieve_ast_argument *require_arg,
+		bool required);
+static int ext_reject_interpreter_run
+	(const struct sieve_extension *this_ext,
+		const struct sieve_runtime_env *renv,
+		void *context, bool deferred);
+
 /* Reject */
 
 static bool ext_reject_validator_load
-(const struct sieve_extension *ext, struct sieve_validator *valdtr);
+	(const struct sieve_extension *ext, struct sieve_validator *valdtr);
+static bool ext_reject_interpreter_load
+	(const struct sieve_extension *ext,
+		const struct sieve_runtime_env *renv, sieve_size_t *address);
 
 const struct sieve_extension_def reject_extension = {
 	.name = "reject",
 	.validator_load =	ext_reject_validator_load,
+	.interpreter_load = ext_reject_interpreter_load,
 	SIEVE_EXT_DEFINE_OPERATION(reject_operation)
 };
+const struct sieve_validator_extension
+reject_validator_extension = {
+	.ext = &reject_extension,
+	.validate = ext_reject_validator_validate
+};
+const struct sieve_interpreter_extension
+reject_interpreter_extension = {
+	.ext_def = &reject_extension,
+	.run = ext_reject_interpreter_run
+};
 
 static bool ext_reject_validator_load
 (const struct sieve_extension *ext, struct sieve_validator *valdtr)
@@ -67,19 +91,45 @@ static bool ext_reject_validator_load
 	/* Register new command */
 	sieve_validator_register_command(valdtr, ext, &reject_command);
 
+	sieve_validator_extension_register
+		(valdtr, ext, &reject_validator_extension, NULL);
+	return TRUE;
+}
+
+static bool ext_reject_interpreter_load
+(const struct sieve_extension *ext,
+	const struct sieve_runtime_env *renv,
+	sieve_size_t *address ATTR_UNUSED)
+{
+	sieve_interpreter_extension_register
+		(renv->interp, ext, &reject_interpreter_extension, NULL);
 	return TRUE;
 }
 
 /* EReject */
 
 static bool ext_ereject_validator_load
-(const struct sieve_extension *ext, struct sieve_validator *valdtr);
+	(const struct sieve_extension *ext, struct sieve_validator *valdtr);
+static bool ext_ereject_interpreter_load
+	(const struct sieve_extension *ext,
+		const struct sieve_runtime_env *renv, sieve_size_t *address);
 
 const struct sieve_extension_def ereject_extension = {
 	.name = "ereject",
 	.validator_load = ext_ereject_validator_load,
+	.interpreter_load = ext_ereject_interpreter_load,
 	SIEVE_EXT_DEFINE_OPERATION(ereject_operation)
 };
+const struct sieve_validator_extension
+ereject_validator_extension = {
+	.ext = &ereject_extension,
+	.validate = ext_reject_validator_validate
+};
+const struct sieve_interpreter_extension
+ereject_interpreter_extension = {
+	.ext_def = &ereject_extension,
+	.run = ext_reject_interpreter_run
+};
 
 static bool ext_ereject_validator_load
 (const struct sieve_extension *ext, struct sieve_validator *valdtr)
@@ -87,9 +137,61 @@ static bool ext_ereject_validator_load
 	/* Register new command */
 	sieve_validator_register_command(valdtr, ext, &ereject_command);
 
+	sieve_validator_extension_register
+		(valdtr, ext, &ereject_validator_extension, NULL);
 	return TRUE;
 }
 
+static bool ext_ereject_interpreter_load
+(const struct sieve_extension *ext,
+	const struct sieve_runtime_env *renv,
+	sieve_size_t *address ATTR_UNUSED)
+{
+	sieve_interpreter_extension_register
+		(renv->interp, ext, &ereject_interpreter_extension, NULL);
+	return TRUE;
+}
+
+/* Environment checking */
+
+static bool ext_reject_validator_validate
+(const struct sieve_extension *ext,
+	struct sieve_validator *valdtr, void *context ATTR_UNUSED,
+	struct sieve_ast_argument *require_arg,
+	bool required)
+{
+	if (required) {
+		enum sieve_compile_flags flags =
+			sieve_validator_compile_flags(valdtr);
+
+		if ( (flags & SIEVE_COMPILE_FLAG_NO_ENVELOPE) != 0 ) {
+			sieve_argument_validate_error(valdtr, require_arg,
+				"the %s extension cannot be used in this context "
+				"(needs access to message envelope)",
+				sieve_extension_name(ext));
+			return FALSE;
+		}
+	}
+	return TRUE;
+}
+
+static int ext_reject_interpreter_run
+(const struct sieve_extension *ext,
+	const struct sieve_runtime_env *renv,
+	void *context ATTR_UNUSED, bool deferred)
+{
+	if ( (renv->flags & SIEVE_EXECUTE_FLAG_NO_ENVELOPE) != 0 ) {
+		if ( !deferred ) {
+			sieve_runtime_error(renv, NULL,
+				"the %s extension cannot be used in this context "
+				"(needs access to message envelope)",
+				sieve_extension_name(ext));
+		}
+		return SIEVE_EXEC_FAILURE;
+	}
+	return SIEVE_EXEC_OK;
+}
+
 /*
  * Commands
  */
@@ -108,14 +210,14 @@ static bool cmd_reject_generate
  */
 
 static const struct sieve_command_def reject_command = {
-	"reject",
-	SCT_COMMAND,
-	1, 0, FALSE, FALSE,
-	NULL, NULL,
-	cmd_reject_validate,
-	NULL,
-	cmd_reject_generate,
-	NULL
+	.identifier = "reject",
+	.type = SCT_COMMAND,
+	.positional_args = 1,
+	.subtests = 0,
+	.block_allowed = FALSE,
+	.block_required = FALSE,
+	.validate = cmd_reject_validate,
+	.generate = cmd_reject_generate
 };
 
 /* EReject command
@@ -125,14 +227,14 @@ static const struct sieve_command_def reject_command = {
  */
 
 static const struct sieve_command_def ereject_command = {
-	"ereject",
-	SCT_COMMAND,
-	1, 0, FALSE, FALSE,
-	NULL, NULL,
-	cmd_reject_validate,
-	NULL,
-	cmd_reject_generate,
-	NULL
+	.identifier = "ereject",
+	.type = SCT_COMMAND,
+	.positional_args = 1,
+	.subtests = 0,
+	.block_allowed = FALSE,
+	.block_required = FALSE,
+	.validate = cmd_reject_validate,
+	.generate = cmd_reject_generate,
 };
 
 /*
@@ -149,21 +251,19 @@ static int ext_reject_operation_execute
 /* Reject operation */
 
 static const struct sieve_operation_def reject_operation = {
-	"REJECT",
-	&reject_extension,
-	0,
-	ext_reject_operation_dump,
-	ext_reject_operation_execute
+	.mnemonic = "REJECT",
+	.ext_def = &reject_extension,
+	.dump = ext_reject_operation_dump,
+	.execute = ext_reject_operation_execute
 };
 
 /* EReject operation */
 
 static const struct sieve_operation_def ereject_operation = {
-	"EREJECT",
-	&ereject_extension,
-	0,
-	ext_reject_operation_dump,
-	ext_reject_operation_execute
+	.mnemonic = "EREJECT",
+	.ext_def = &ereject_extension,
+	.dump = ext_reject_operation_dump,
+	.execute = ext_reject_operation_execute
 };
 
 /*
@@ -382,10 +482,19 @@ static int act_reject_commit
 {
 	struct act_reject_context *rj_ctx =
 		(struct act_reject_context *) action->context;
-	const char *sender = sieve_message_get_sender(aenv->msgctx);
-	const char *recipient = sieve_message_get_final_recipient(aenv->msgctx);
+	const struct smtp_address *sender, *recipient;
+	int ret;
 
-	if ( recipient == NULL ) {
+	sender = sieve_message_get_sender(aenv->msgctx);
+	recipient = sieve_message_get_orig_recipient(aenv->msgctx);
+
+	if ((aenv->flags & SIEVE_EXECUTE_FLAG_SKIP_RESPONSES) != 0) {
+		sieve_result_global_log(aenv,
+			"not sending reject message (skipped)");
+		return SIEVE_EXEC_OK;
+	}
+
+	if ( smtp_address_isnull(recipient) ) {
 		sieve_result_global_warning(aenv,
 			"reject action aborted: envelope recipient is <>");
 		return SIEVE_EXEC_OK;
@@ -399,23 +508,24 @@ static int act_reject_commit
 		return SIEVE_EXEC_OK;
 	}
 
-	if ( sender == NULL ) {
+	if ( smtp_address_isnull(sender) ) {
 		sieve_result_global_log(aenv, "not sending reject message to <>");
 
 		*keep = FALSE;
 		return SIEVE_EXEC_OK;
 	}
 
-	if ( sieve_action_reject_mail(aenv, sender, recipient, rj_ctx->reason) ) {
-		sieve_result_global_log(aenv,
-			"rejected message from <%s> (%s)", str_sanitize(sender, 80),
-			( rj_ctx->ereject ? "ereject" : "reject" ));
+	if ( (ret=sieve_action_reject_mail
+		(aenv, recipient, rj_ctx->reason)) <= 0 )
+		return ret;
 
-		*keep = FALSE;
-		return SIEVE_EXEC_OK;
-	}
+	sieve_result_global_log(aenv,
+		"rejected message from <%s> (%s)",
+		smtp_address_encode(sender),
+		( rj_ctx->ereject ? "ereject" : "reject" ));
 
-	return SIEVE_EXEC_FAILURE;
+	*keep = FALSE;
+	return SIEVE_EXEC_OK;
 }
 
 
