@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2016 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 /* Match-type ':is':
@@ -26,10 +26,9 @@ static int mcht_is_match_key
  */
 
 const struct sieve_match_type_def is_match_type = {
-	SIEVE_OBJECT("is", &match_type_operand, SIEVE_MATCH_TYPE_IS),
-	NULL, NULL, NULL, NULL, NULL,
-	mcht_is_match_key,
-	NULL
+	SIEVE_OBJECT("is",
+		&match_type_operand, SIEVE_MATCH_TYPE_IS),
+	.match_key = mcht_is_match_key
 };
 
 /*
@@ -42,12 +41,12 @@ static int mcht_is_match_key
 	const char *key, size_t key_size)
 {
 	if ( val_size == 0 )
-		return ( key_size == 0 );
+		return ( key_size == 0 ? 1 : 0 );
 
 	if ( mctx->comparator->def != NULL && mctx->comparator->def->compare != NULL )
 		return (mctx->comparator->def->compare(mctx->comparator,
-			val, val_size, key, key_size) == 0);
+			val, val_size, key, key_size) == 0 ? 1 : 0 );
 
-	return FALSE;
+	return 0;
 }
 
