@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 /* Extension comparator-i;ascii-numeric
@@ -58,11 +58,10 @@ static const struct sieve_extension_objects ext_comparators =
 	SIEVE_EXT_DEFINE_COMPARATOR(i_ascii_numeric_comparator);
 
 static const struct sieve_operand_def my_comparator_operand = {
-	"comparator-i;ascii-numeric",
-	&comparator_i_ascii_numeric_extension,
-	0,
-	&sieve_comparator_operand_class,
-	&ext_comparators
+	.name = "comparator-i;ascii-numeric",
+	.ext_def = &comparator_i_ascii_numeric_extension,
+	.class = &sieve_comparator_operand_class,
+	.interface = &ext_comparators
 };
 
 /*
@@ -78,11 +77,12 @@ static int cmp_i_ascii_numeric_compare
 /* Comparator object */
 
 const struct sieve_comparator_def i_ascii_numeric_comparator = {
-	SIEVE_OBJECT("i;ascii-numeric", &my_comparator_operand, 0),
-	SIEVE_COMPARATOR_FLAG_ORDERING | SIEVE_COMPARATOR_FLAG_EQUALITY,
-	cmp_i_ascii_numeric_compare,
-	NULL,
-	NULL
+	SIEVE_OBJECT("i;ascii-numeric",
+		&my_comparator_operand, 0),
+	.flags =
+		SIEVE_COMPARATOR_FLAG_ORDERING |
+		SIEVE_COMPARATOR_FLAG_EQUALITY,
+	.compare = cmp_i_ascii_numeric_compare
 };
 
 /* Comparator implementation */
