@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 #ifndef __URI_MAILTO_H
@@ -15,7 +15,7 @@ struct uri_mailto_header_field {
 
 struct uri_mailto_recipient {
 	const char *full;
-	const char *normalized;
+	const struct smtp_address *address;
 	bool carbon_copy;
 };
 
@@ -27,7 +27,6 @@ struct uri_mailto {
 	ARRAY_TYPE(headers) headers;
 	const char *subject;
 	const char *body;
-	const char *from_normalized;
 };
 
 bool uri_mailto_validate
