@@ -1,7 +1,8 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
+#include "unichar.h"
 #include "str.h"
 
 #include "sieve-common.h"
@@ -15,7 +16,9 @@
  * Encodeurl modifier
  */
 
-bool mod_encodeurl_modify(string_t *in, string_t **result);
+static bool
+mod_encodeurl_modify(const struct sieve_variables_modifier *modf,
+		     string_t *in, string_t **result);
 
 const struct sieve_variables_modifier_def encodeurl_modifier = {
 	SIEVE_OBJECT("encodeurl", &encodeurl_operand, 0),
@@ -31,11 +34,10 @@ static const struct sieve_extension_objects ext_enotify_modifiers =
 	SIEVE_VARIABLES_DEFINE_MODIFIER(encodeurl_modifier);
 
 const struct sieve_operand_def encodeurl_operand = {
-	"modifier",
-	&enotify_extension,
-	0,
-	&sieve_variables_modifier_operand_class,
-	&ext_enotify_modifiers
+	.name = "modifier",
+	.ext_def = &enotify_extension,
+	.class = &sieve_variables_modifier_operand_class,
+	.interface = &ext_enotify_modifiers
 };
 
 /*
@@ -62,22 +64,55 @@ static const char _uri_reserved_lookup[256] = {
 	1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,  // F0
 };
 
-bool mod_encodeurl_modify(string_t *in, string_t **result)
+static bool
+mod_encodeurl_modify(const struct sieve_variables_modifier *modf,
+		     string_t *in, string_t **result)
 {
-	unsigned int i;
-	const unsigned char *c;
+	size_t max_var_size =
+		sieve_variables_get_max_variable_size(modf->var_ext);
+	const unsigned char *p, *poff, *pend;
+	size_t new_size;
 
-	*result = t_str_new(2*str_len(in));
-	c = str_data(in);
-
-	for ( i = 0; i < str_len(in); i++, c++ ) {
-		if ( _uri_reserved_lookup[*c] ) {
-			str_printfa(*result, "%%%02X", *c);
-		} else {
-			str_append_c(*result, *c);
-		}
+	if ( str_len(in) == 0 ) {
+		*result = in;
+		return TRUE;
 	}
 
+	/* allocate new string */
+	new_size = str_len(in) + 32;
+	if (new_size > max_var_size)
+		new_size = max_var_size;
+	*result = t_str_new(new_size + 1);
+
+	/* escape string */
+	p = str_data(in);
+	pend = p + str_len(in);
+	poff = p;
+	while (p < pend) {
+		unsigned int i, n = uni_utf8_char_bytes(*p);
+
+		if (n > 1 || (_uri_reserved_lookup[*p] & 0x01) != 0) {
+			str_append_data(*result, poff, p - poff);
+			poff = p;
+
+			if (str_len(*result) + 3 * n > max_var_size)
+				break;
+
+			str_printfa(*result, "%%%02X", *p);
+			for (i = 1; i < n && p < pend; i++) {
+				p++;
+				poff++;
+				str_printfa(*result, "%%%02X", *p);
+			}
+
+			poff++;
+		} else if ((str_len(*result) + (p - poff) + 1) > max_var_size) {
+			break;
+		}
+		p++;
+	}
+
+	str_append_data(*result, poff, p - poff);
 	return TRUE;
 }
 
