@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -29,15 +29,14 @@ static bool cmd_flag_generate
  */
 
 const struct sieve_command_def cmd_setflag = {
-	"setflag",
-	SCT_COMMAND,
-	-1, /* We check positional arguments ourselves */
-	0, FALSE, FALSE,
-	NULL, NULL,
-	ext_imap4flags_command_validate,
-	NULL,
-	cmd_flag_generate,
-	NULL
+	.identifier = "setflag",
+	.type = SCT_COMMAND,
+	.positional_args = -1, /* We check positional arguments ourselves */
+	.subtests = 0,
+	.block_allowed = FALSE,
+	.block_required = FALSE,
+	.validate = ext_imap4flags_command_validate,
+	.generate = cmd_flag_generate
 };
 
 /* Addflag command
@@ -47,15 +46,14 @@ const struct sieve_command_def cmd_setflag = {
  */
 
 const struct sieve_command_def cmd_addflag = {
-	"addflag",
-	SCT_COMMAND,
-	-1, /* We check positional arguments ourselves */
-	0, FALSE, FALSE,
-	NULL, NULL,
-	ext_imap4flags_command_validate,
-	NULL,
-	cmd_flag_generate,
-	NULL
+	.identifier = "addflag",
+	.type = SCT_COMMAND,
+	.positional_args = -1, /* We check positional arguments ourselves */
+	.subtests = 0,
+	.block_allowed = FALSE,
+	.block_required = FALSE,
+	.validate = ext_imap4flags_command_validate,
+	.generate = cmd_flag_generate
 };
 
 
@@ -66,15 +64,14 @@ const struct sieve_command_def cmd_addflag = {
  */
 
 const struct sieve_command_def cmd_removeflag = {
-	"removeflag",
-	SCT_COMMAND,
-	-1, /* We check positional arguments ourselves */
-	0, FALSE, FALSE,
-	NULL, NULL,
-	ext_imap4flags_command_validate,
-	NULL,
-	cmd_flag_generate,
-	NULL
+	.identifier = "removeflag",
+	.type = SCT_COMMAND,
+	.positional_args = -1, /* We check positional arguments ourselves */
+	.subtests = 0,
+	.block_allowed = FALSE,
+	.block_required = FALSE,
+	.validate = ext_imap4flags_command_validate,
+	.generate = cmd_flag_generate
 };
 
 /*
@@ -91,31 +88,31 @@ static int cmd_flag_operation_execute
 /* Setflag operation */
 
 const struct sieve_operation_def setflag_operation = {
-	"SETFLAG",
-	&imap4flags_extension,
-	ext_imap4flags_OPERATION_SETFLAG,
-	cmd_flag_operation_dump,
-	cmd_flag_operation_execute
+	.mnemonic = "SETFLAG",
+	.ext_def = &imap4flags_extension,
+	.code = EXT_IMAP4FLAGS_OPERATION_SETFLAG,
+	.dump = cmd_flag_operation_dump,
+	.execute = cmd_flag_operation_execute
 };
 
 /* Addflag operation */
 
 const struct sieve_operation_def addflag_operation = {
-	"ADDFLAG",
-	&imap4flags_extension,
-	ext_imap4flags_OPERATION_ADDFLAG,
-	cmd_flag_operation_dump,
-	cmd_flag_operation_execute
+	.mnemonic = "ADDFLAG",
+	.ext_def = &imap4flags_extension,
+	.code = EXT_IMAP4FLAGS_OPERATION_ADDFLAG,
+	.dump = cmd_flag_operation_dump,
+	.execute = cmd_flag_operation_execute
 };
 
 /* Removeflag operation */
 
 const struct sieve_operation_def removeflag_operation = {
-	"REMOVEFLAG",
-	&imap4flags_extension,
-	ext_imap4flags_OPERATION_REMOVEFLAG,
-	cmd_flag_operation_dump,
-	cmd_flag_operation_execute
+	.mnemonic = "REMOVEFLAG",
+	.ext_def = &imap4flags_extension,
+	.code = EXT_IMAP4FLAGS_OPERATION_REMOVEFLAG,
+	.dump = cmd_flag_operation_dump,
+	.execute = cmd_flag_operation_execute
 };
 
 /*
@@ -125,6 +122,8 @@ const struct sieve_operation_def removeflag_operation = {
 static bool cmd_flag_generate
 (const struct sieve_codegen_env *cgenv, struct sieve_command *cmd)
 {
+	struct sieve_ast_argument *arg1, *arg2;
+
 	/* Emit operation */
 	if ( sieve_command_is(cmd, cmd_setflag) )
 		sieve_operation_emit(cgenv->sblock, cmd->ext, &setflag_operation);
@@ -133,10 +132,21 @@ static bool cmd_flag_generate
 	else if ( sieve_command_is(cmd, cmd_removeflag) )
 		sieve_operation_emit(cgenv->sblock, cmd->ext, &removeflag_operation);
 
-	/* Generate arguments */
-	if ( !sieve_generate_arguments(cgenv, cmd, NULL) )
-		return FALSE;
+	arg1 = cmd->first_positional;
+	arg2 = sieve_ast_argument_next(arg1);
 
+	if ( arg2 == NULL ) {
+		/* No variable */
+		sieve_opr_omitted_emit(cgenv->sblock);
+		if ( !sieve_generate_argument(cgenv, arg1, cmd) )
+			return FALSE;
+	} else {
+		/* Full command */
+		if ( !sieve_generate_argument(cgenv, arg1, cmd) )
+			return FALSE;
+		if ( !sieve_generate_argument(cgenv, arg2, cmd) )
+			return FALSE;
+	}
 	return TRUE;
 }
 
@@ -158,14 +168,14 @@ bool cmd_flag_operation_dump
 		return FALSE;
 	}
 
-	if ( sieve_operand_is_variable(&oprnd) ) {
+	if ( !sieve_operand_is_omitted(&oprnd) ) {
 		return
 			sieve_opr_string_dump_data(denv, &oprnd, address, "variable name") &&
 			sieve_opr_stringlist_dump(denv, address, "list of flags");
 	}
 
 	return
-		sieve_opr_stringlist_dump_data(denv, &oprnd, address, "list of flags");
+		sieve_opr_stringlist_dump(denv, address, "list of flags");
 }
 
 /*
@@ -193,10 +203,10 @@ static int cmd_flag_operation_execute
 		return ret;
 
 	/* Variable operand (optional) */
-	if ( sieve_operand_is_variable(&oprnd) ) {
+	if ( !sieve_operand_is_omitted(&oprnd) ) {
 		/* Read the variable operand */
 		if ( (ret=sieve_variable_operand_read_data
-				(renv, &oprnd, address, "variable", &storage, &var_index)) <= 0 )
+			(renv, &oprnd, address, "variable", &storage, &var_index)) <= 0 )
 			return ret;
 
 		/* Read flag list */
@@ -205,21 +215,14 @@ static int cmd_flag_operation_execute
 			return ret;
 
 	/* Flag-list operand */
-	} else if ( sieve_operand_is_stringlist(&oprnd) ) {
+	} else {
 		storage = NULL;
 		var_index = 0;
 
 		/* Read flag list */
-		if ( (ret=sieve_opr_stringlist_read_data
-			(renv, &oprnd, address, "flag-list", &flag_list)) <= 0 )
+		if ( (ret=sieve_opr_stringlist_read(renv, address,
+			"flag-list", &flag_list)) <= 0 )
 			return ret;
-
-	/* Invalid */
-	} else {
-		sieve_runtime_trace_operand_error
-			(renv, &oprnd, "expected variable or string-list (flag-list) operand "
-				"but found %s", sieve_operand_name(&oprnd));
-		return SIEVE_EXEC_BIN_CORRUPT;
 	}
 
 	/*
@@ -244,5 +247,5 @@ static int cmd_flag_operation_execute
 	sieve_runtime_trace_descend(renv);
 
 	/* Perform requested operation */
-	return flag_op(renv, storage, var_index, flag_list);
+	return flag_op(renv, op->ext, storage, var_index, flag_list);
 }
