@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2016 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -97,8 +97,8 @@ bool ext_imap4flags_command_validate
 			return FALSE;
 		}
 
-		if ( !sieve_variable_argument_activate
-			(var_ext, valdtr, cmd, arg, !sieve_command_is(cmd, tst_hasflag) ) )
+		if ( !sieve_variable_argument_activate(var_ext, var_ext,
+			valdtr, cmd, arg, !sieve_command_is(cmd, tst_hasflag) ) )
 			return FALSE;
 
 		if ( sieve_ast_argument_type(arg2) != SAAT_STRING &&
@@ -309,7 +309,7 @@ bool sieve_ext_imap4flags_flag_is_valid(const char *flag)
 static void ext_imap4flags_iter_clear
 (struct ext_imap4flags_iter *iter)
 {
-	memset(iter, 0, sizeof(*iter));
+	i_zero(iter);
 }
 
 void ext_imap4flags_iter_init
@@ -611,7 +611,7 @@ struct ext_imap4flags_stringlist {
 	string_t *flags_string;
 	struct ext_imap4flags_iter flit;
 
-	unsigned int normalize:1;
+	bool normalize:1;
 };
 
 static struct sieve_stringlist *ext_imap4flags_stringlist_create
