@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2014 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -10,6 +10,7 @@
 #include "sieve-settings.h"
 #include "sieve-error.h"
 #include "sieve-script.h"
+#include "sieve-storage.h"
 #include "sieve-ast.h"
 #include "sieve-binary.h"
 #include "sieve-commands.h"
@@ -22,7 +23,6 @@
 #include "ext-include-binary.h"
 #include "ext-include-variables.h"
 
-#include <stdlib.h>
 
 /*
  * Forward declarations
@@ -76,36 +76,26 @@ bool ext_include_load
 {
 	struct sieve_instance *svinst = ext->svinst;
 	struct ext_include_context *ctx;
-	const char *sieve_dir;
+	const char *location;
 	unsigned long long int uint_setting;
 
 	if ( *context != NULL ) {
 		ext_include_unload(ext);
 	}
 
-	// FIXME: sieve_dir and sieve_global_dir settings have somewhat become
-	// misnomers; these are not necessary filesystem directories anymore.
-
 	ctx = i_new(struct ext_include_context, 1);
 
-	/* Get directory for :global scripts */
-	sieve_dir = sieve_setting_get(svinst, "sieve_global_dir");
+	/* Get location for :global scripts */
+	location = sieve_setting_get(svinst, "sieve_global");
+	if ( location == NULL )
+		location = sieve_setting_get(svinst, "sieve_global_dir");
 
-	if ( sieve_dir == NULL && svinst->debug ) {
-		sieve_sys_debug(svinst, "include: sieve_global_dir is not set; "
+	if ( location == NULL && svinst->debug ) {
+		sieve_sys_debug(svinst, "include: sieve_global is not set; "
 			"it is currently not possible to include `:global' scripts.");
 	}
 
-	ctx->global_dir = i_strdup(sieve_dir);
-
-	/* Get directory for :personal scripts */
- 	sieve_dir = sieve_setting_get(svinst, "sieve_dir");
-
-	if ( sieve_dir == NULL ) {
-		sieve_dir = "~/sieve";
-	}
-
-	ctx->personal_dir = i_strdup(sieve_dir);
+	ctx->global_location = i_strdup(location);
 
 	/* Get limits */
 	ctx->max_nesting_depth = EXT_INCLUDE_DEFAULT_MAX_NESTING_DEPTH;
@@ -135,8 +125,12 @@ void ext_include_unload
 	struct ext_include_context *ctx =
 		(struct ext_include_context *) ext->context;
 
-	i_free(ctx->personal_dir);
-	i_free(ctx->global_dir);
+	if ( ctx->global_storage != NULL )
+		sieve_storage_unref(&ctx->global_storage);
+	if ( ctx->personal_storage != NULL )
+		sieve_storage_unref(&ctx->personal_storage);
+
+	i_free(ctx->global_location);
 	i_free(ctx);
 }
 
@@ -144,44 +138,43 @@ void ext_include_unload
  * Script access
  */
 
-const char *ext_include_get_script_location
-(const struct sieve_extension *ext, enum ext_include_script_location location,
-   const char *script_name)
+struct sieve_storage *ext_include_get_script_storage
+(const struct sieve_extension *ext,
+	enum ext_include_script_location location,
+	const char *script_name, enum sieve_error *error_r)
 {
 	struct sieve_instance *svinst = ext->svinst;
 	struct ext_include_context *ctx =
 		(struct ext_include_context *) ext->context;
-	const char *sieve_dir;
 
 	switch ( location ) {
 	case EXT_INCLUDE_LOCATION_PERSONAL:
-
-		if ( ctx->personal_dir == NULL ) {
-			sieve_sys_error(svinst, "include: sieve_dir is unconfigured; "
-				"include of `:personal' script `%s' is therefore not possible",
-				str_sanitize(script_name, 80));
-			return NULL;
+		if ( ctx->personal_storage == NULL ) {
+			ctx->personal_storage = sieve_storage_create_main
+				(svinst, NULL, 0, error_r);
 		}
-
-		sieve_dir = ctx->personal_dir;
-		break;
+		return ctx->personal_storage;
 
 	case EXT_INCLUDE_LOCATION_GLOBAL:
-
-		if ( ctx->global_dir == NULL ) {
-			sieve_sys_error(svinst, "include: sieve_global_dir is unconfigured; "
+		if ( ctx->global_location == NULL ) {
+			sieve_sys_info(svinst, "include: sieve_global is unconfigured; "
 				"include of `:global' script `%s' is therefore not possible",
 				str_sanitize(script_name, 80));
+			if ( error_r != NULL )
+				*error_r = SIEVE_ERROR_NOT_FOUND;
 			return NULL;
 		}
-
-		sieve_dir = ctx->global_dir;
-		break;
+		if ( ctx->global_storage == NULL ) {
+			ctx->global_storage = sieve_storage_create
+				(svinst, ctx->global_location, 0, error_r);
+		}
+		return ctx->global_storage;
 	default:
-		i_unreached();
+		break;
 	}
 
-	return sieve_dir;
+	i_unreached();
+	return NULL;
 }
 
 /*
@@ -233,7 +226,10 @@ struct ext_include_ast_context *ext_include_create_ast_context
 
 		sieve_variable_scope_ref(actx->global_vars);
 	} else {
-		actx->global_vars = sieve_variable_scope_create(this_ext->svinst, this_ext);
+		struct ext_include_context *ectx =
+			ext_include_get_context(this_ext);
+		actx->global_vars = sieve_variable_scope_create
+			(this_ext->svinst, ectx->var_ext, this_ext);
 	}
 
 	sieve_ast_extension_register
@@ -335,36 +331,36 @@ void ext_include_register_generator_context
  * Runtime initialization
  */
 
-static void ext_include_runtime_init
-(const struct sieve_extension *this_ext, const struct sieve_runtime_env *renv,
-	void *context)
+static int ext_include_runtime_init
+(const struct sieve_extension *this_ext,
+	const struct sieve_runtime_env *renv,
+	void *context, bool deferred ATTR_UNUSED)
 {
 	struct ext_include_interpreter_context *ctx =
 		(struct ext_include_interpreter_context *) context;
 	struct ext_include_context *ectx = ext_include_get_context(this_ext);
 
-
 	if ( ctx->parent == NULL ) {
 		ctx->global = p_new(ctx->pool, struct ext_include_interpreter_global, 1);
 		p_array_init(&ctx->global->included_scripts, ctx->pool, 10);
 
-
 		ctx->global->var_scope =
 			ext_include_binary_get_global_scope(this_ext, renv->sbin);
 		ctx->global->var_storage =
-			sieve_variable_storage_create(ctx->pool, ctx->global->var_scope);
+			sieve_variable_storage_create(ectx->var_ext,
+				ctx->pool, ctx->global->var_scope);
 	} else {
 		ctx->global = ctx->parent->global;
 	}
 
 	sieve_ext_variables_runtime_set_storage
 		(ectx->var_ext, renv, this_ext, ctx->global->var_storage);
+	return SIEVE_EXEC_OK;
 }
 
 static struct sieve_interpreter_extension include_interpreter_extension = {
-	&include_extension,
-	ext_include_runtime_init,
-	NULL,
+	.ext_def = &include_extension,
+	.run = ext_include_runtime_init
 };
 
 /*
@@ -566,9 +562,9 @@ int ext_include_generate_include
 			(void)ext_include_create_ast_context(this_ext, ast, cmd->ast_node->ast);
 
 			if ( location == EXT_INCLUDE_LOCATION_GLOBAL )
-				cpflags &= ~SIEVE_RUNTIME_FLAG_NOGLOBAL;
+				cpflags &= ~SIEVE_EXECUTE_FLAG_NOGLOBAL;
 			else
-				cpflags |= SIEVE_RUNTIME_FLAG_NOGLOBAL;
+				cpflags |= SIEVE_EXECUTE_FLAG_NOGLOBAL;
 
 			/* Validate */
 			if ( !sieve_validate(ast, ehandler, cpflags, NULL) ) {
@@ -610,7 +606,7 @@ int ext_include_generate_include
  * Executing an included script during interpretation
  */
 
-static int ext_include_runtime_check_circular
+static bool ext_include_runtime_check_circular
 (struct ext_include_interpreter_context *ctx,
 	const struct ext_include_script_info *include)
 {
@@ -707,25 +703,27 @@ int ext_include_execute_include
 		/* We are the top-level interpreter instance */
 
 		if ( result == SIEVE_EXEC_OK ) {
-			enum sieve_runtime_flags rtflags = 0;
+			enum sieve_execute_flags exflags = renv->flags;
 
 			if ( included->location != EXT_INCLUDE_LOCATION_GLOBAL )
-				rtflags |= SIEVE_RUNTIME_FLAG_NOGLOBAL;
+				exflags |= SIEVE_EXECUTE_FLAG_NOGLOBAL;
+			else
+				exflags &= ~SIEVE_EXECUTE_FLAG_NOGLOBAL;
 
 			/* Create interpreter for top-level included script
 			 * (first sub-interpreter)
 			 */
 			subinterp = sieve_interpreter_create_for_block
-				(included->block, included->script, renv->msgdata, renv->scriptenv,
-					ehandler, rtflags);
+				(included->block, included->script, renv->interp,
+					renv->msgdata, renv->scriptenv, ehandler, exflags);
 
 			if ( subinterp != NULL ) {
 				curctx = ext_include_interpreter_context_init_child
 					(this_ext, subinterp, ctx, included->script, included);
 
 				/* Activate and start the top-level included script */
-				result = ( sieve_interpreter_start
-					(subinterp, renv->result, &interrupted) == 1 );
+				result = sieve_interpreter_start
+					(subinterp, renv->result, &interrupted);
 			} else
 				result = SIEVE_EXEC_BIN_CORRUPT;
 		}
@@ -762,21 +760,24 @@ int ext_include_execute_include
 					curctx->include = NULL;
 					curctx->returned = FALSE;
 
-					result = ( sieve_interpreter_continue(subinterp, &interrupted) == 1 );
+					result = sieve_interpreter_continue(subinterp, &interrupted);
 				} else {
 					if ( curctx->include != NULL ) {
 						/* Sub-include requested */
 
 						if ( result == SIEVE_EXEC_OK ) {
-							enum sieve_runtime_flags rtflags = 0;
+							enum sieve_execute_flags exflags = renv->flags;
 
 							if ( curctx->include->location != EXT_INCLUDE_LOCATION_GLOBAL )
-								rtflags |= SIEVE_RUNTIME_FLAG_NOGLOBAL;
+								exflags |= SIEVE_EXECUTE_FLAG_NOGLOBAL;
+							else
+								exflags &= ~SIEVE_EXECUTE_FLAG_NOGLOBAL;
 
 							/* Create sub-interpreter */
 							subinterp = sieve_interpreter_create_for_block
-								(curctx->include->block, curctx->include->script, renv->msgdata,
-									renv->scriptenv, ehandler, rtflags);
+								(curctx->include->block, curctx->include->script,
+									curctx->interp, renv->msgdata,
+									renv->scriptenv, ehandler, exflags);
 
 							if ( subinterp != NULL ) {
 								curctx = ext_include_interpreter_context_init_child
@@ -786,8 +787,8 @@ int ext_include_execute_include
 								/* Start the sub-include's interpreter */
 								curctx->include = NULL;
 								curctx->returned = FALSE;
-								result = ( sieve_interpreter_start(subinterp, renv->result,
-									&interrupted) == 1 );
+								result = sieve_interpreter_start
+									(subinterp, renv->result, &interrupted);
 							} else
 								result = SIEVE_EXEC_BIN_CORRUPT;
 						}
