@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -32,14 +32,14 @@ static bool tst_mailboxexists_generate
 	(const struct sieve_codegen_env *cgenv, struct sieve_command *ctx);
 
 const struct sieve_command_def mailboxexists_test = {
-	"mailboxexists",
-	SCT_TEST,
-	1, 0, FALSE, FALSE,
-	NULL, NULL,
-	tst_mailboxexists_validate,
-	NULL,
-	tst_mailboxexists_generate,
-	NULL
+	.identifier = "mailboxexists",
+	.type = SCT_TEST,
+	.positional_args = 1,
+	.subtests = 0,
+	.block_allowed = FALSE,
+	.block_required = FALSE,
+	.validate = tst_mailboxexists_validate,
+	.generate = tst_mailboxexists_generate
 };
 
 /*
@@ -52,11 +52,10 @@ static int tst_mailboxexists_operation_execute
 	(const struct sieve_runtime_env *renv, sieve_size_t *address);
 
 const struct sieve_operation_def mailboxexists_operation = {
-	"MAILBOXEXISTS",
-	&mailbox_extension,
-	0,
-	tst_mailboxexists_operation_dump,
-	tst_mailboxexists_operation_execute
+	.mnemonic = "MAILBOXEXISTS",
+	.ext_def = &mailbox_extension,
+	.dump = tst_mailboxexists_operation_dump,
+	.execute = tst_mailboxexists_operation_execute
 };
 
 /*
@@ -86,7 +85,7 @@ static int tst_mailboxexists_mailbox_validate
 		}
 	}
 
-	return TRUE;
+	return 1;
 }
 
 static bool tst_mailboxexists_validate
@@ -105,12 +104,12 @@ static bool tst_mailboxexists_validate
 		return FALSE;
 
 	aarg = arg;
-	memset(&valctx, 0, sizeof(valctx));
+	i_zero(&valctx);
 	valctx.valdtr = valdtr;
 	valctx.tst = tst;
 
-	return sieve_ast_stringlist_map
-		(&aarg, (void*)&valctx, tst_mailboxexists_mailbox_validate);
+	return ( sieve_ast_stringlist_map(&aarg,
+		(void*)&valctx, tst_mailboxexists_mailbox_validate) >= 0 );
 }
 
 /*
