@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2016 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -31,11 +31,9 @@ static bool tag_mime_generate
     struct sieve_command *context);
 
 const struct sieve_argument_def mime_tag = {
-	"mime",
-	NULL,
-	tag_mime_validate,
-	NULL, NULL,
-	tag_mime_generate
+	.identifier = "mime",
+	.validate = tag_mime_validate,
+	.generate = tag_mime_generate
 };
 
 static bool tag_mime_option_validate
@@ -43,38 +41,28 @@ static bool tag_mime_option_validate
 		struct sieve_command *cmd);
 
 const struct sieve_argument_def mime_anychild_tag = {
-	"anychild",
-	NULL,
-	tag_mime_option_validate,
-	NULL, NULL, NULL
+	.identifier = "anychild",
+	.validate = tag_mime_option_validate
 };
 
 const struct sieve_argument_def mime_type_tag = {
-	"type",
-	NULL,
-	tag_mime_option_validate,
-	NULL, NULL, NULL
+	.identifier = "type",
+	.validate = tag_mime_option_validate
 };
 
 const struct sieve_argument_def mime_subtype_tag = {
-	"subtype",
-	NULL,
-	tag_mime_option_validate,
-	NULL, NULL, NULL
+	.identifier = "subtype",
+	.validate = tag_mime_option_validate
 };
 
 const struct sieve_argument_def mime_contenttype_tag = {
-	"contenttype",
-	NULL,
-	tag_mime_option_validate,
-	NULL, NULL, NULL
+	.identifier = "contenttype",
+	.validate = tag_mime_option_validate
 };
 
 const struct sieve_argument_def mime_param_tag = {
-	"param",
-	NULL,
-	tag_mime_option_validate,
-	NULL, NULL, NULL
+	.identifier = "param",
+	.validate = tag_mime_option_validate
 };
 
 /*
@@ -109,11 +97,10 @@ static const struct sieve_extension_objects ext_header_overrides =
 	SIEVE_EXT_DEFINE_MESSAGE_OVERRIDE(mime_header_override);
 
 const struct sieve_operand_def mime_operand = {
-	"mime operand",
-	&mime_extension,
-	0,
-	&sieve_message_override_operand_class,
-	&ext_header_overrides
+	.name = "mime operand",
+	.ext_def = &mime_extension,
+	.class = &sieve_message_override_operand_class,
+	.interface = &ext_header_overrides
 };
 
 /*
@@ -123,7 +110,7 @@ const struct sieve_operand_def mime_operand = {
 struct tag_mime_data {
 	enum ext_mime_option mimeopt;
 	struct sieve_ast_argument *param_arg;
-	unsigned int anychild:1;
+	bool anychild:1;
 };
 
 /*
@@ -167,6 +154,8 @@ static bool tag_mime_option_validate
 	struct sieve_ast_argument *mime_arg;
 	struct tag_mime_data *data;
 
+	i_assert(tag != NULL);
+
 	/* Detach tag itself */
 	*arg = sieve_ast_arguments_detach(*arg,1);
 
@@ -597,7 +586,7 @@ static void content_header_stringlist_set_trace
 struct svmo_mime_context {
 	enum ext_mime_option mimeopt;
 	const char *const *params;
-	unsigned int anychild:1;
+	bool anychild:1;
 };
 
 /* Context coding */
