@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2014 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -52,15 +52,16 @@ static bool cmd_notify_generate
 		struct sieve_command *ctx);
 
 const struct sieve_command_def cmd_notify_old = {
-	"notify",
-	SCT_COMMAND,
-	0, 0, FALSE, FALSE,
-	cmd_notify_registered,
-	cmd_notify_pre_validate,
-	cmd_notify_validate,
-	NULL,
-	cmd_notify_generate,
-	NULL,
+	.identifier = "notify",
+	.type = SCT_COMMAND,
+	.positional_args = 0,
+	.subtests = 0,
+	.block_allowed = FALSE,
+	.block_required = FALSE,
+	.registered = cmd_notify_registered,
+	.pre_validate = cmd_notify_pre_validate,
+	.validate = cmd_notify_validate,
+	.generate = cmd_notify_generate
 };
 
 /*
@@ -79,31 +80,23 @@ static bool cmd_notify_validate_stringlist_tag
 /* Argument objects */
 
 static const struct sieve_argument_def notify_method_tag = {
-	"method",
-	NULL,
-	cmd_notify_validate_string_tag,
-	NULL, NULL, NULL
+	.identifier = "method",
+	.validate = cmd_notify_validate_string_tag
 };
 
 static const struct sieve_argument_def notify_options_tag = {
-	"options",
-	NULL,
-	cmd_notify_validate_stringlist_tag,
-	NULL, NULL, NULL
+	.identifier = "options",
+	.validate = cmd_notify_validate_stringlist_tag
 };
 
 static const struct sieve_argument_def notify_id_tag = {
-	"id",
-	NULL,
-	cmd_notify_validate_string_tag,
-	NULL, NULL, NULL
+	.identifier = "id",
+	.validate = cmd_notify_validate_string_tag
 };
 
 static const struct sieve_argument_def notify_message_tag = {
-	"message",
-	NULL,
-	cmd_notify_validate_string_tag,
-	NULL, NULL, NULL
+	.identifier = "message",
+	.validate = cmd_notify_validate_string_tag
 };
 
 /*
@@ -116,11 +109,11 @@ static int cmd_notify_operation_execute
 	(const struct sieve_runtime_env *renv, sieve_size_t *address);
 
 const struct sieve_operation_def notify_old_operation = {
-	"NOTIFY",
-	&notify_extension,
-	EXT_NOTIFY_OPERATION_NOTIFY,
-	cmd_notify_operation_dump,
-	cmd_notify_operation_execute
+	.mnemonic = "NOTIFY",
+	.ext_def = &notify_extension,
+	.code = EXT_NOTIFY_OPERATION_NOTIFY,
+	.dump = cmd_notify_operation_dump,
+	.execute = cmd_notify_operation_execute
 };
 
 /* Codes for optional operands */
@@ -290,12 +283,12 @@ static int cmd_notify_address_validate
 	if ( sieve_argument_is_string_literal(arg) ) {
 		string_t *address = sieve_ast_argument_str(arg);
 		const char *error;
-		bool result = FALSE;
+		int result;
 
 		T_BEGIN {
-			result = sieve_address_validate(address, &error);
+			result = ( sieve_address_validate_str(address, &error) ? 1 : -1 );
 
-			if ( !result ) {
+			if ( result <= 0 ) {
 				sieve_argument_validate_error(valdtr, arg,
 					"specified :options address '%s' is invalid for "
 					"the mailto notify method: %s",
@@ -306,7 +299,7 @@ static int cmd_notify_address_validate
 		return result;
 	}
 
-	return TRUE;
+	return 1;
 }
 
 static bool cmd_notify_validate
@@ -497,10 +490,11 @@ static int cmd_notify_operation_execute
 		raw_address = NULL;
 		while ( (ret=sieve_stringlist_next_item(options, &raw_address)) > 0 ) {
 			const char *error = NULL;
-			const char *addr_norm = sieve_address_normalize(raw_address, &error);
+			const struct smtp_address *address;
 
 			/* Add if valid address */
-			if ( addr_norm != NULL ) {
+			address = sieve_address_parse_str(raw_address, &error);
+			if ( address != NULL ) {
 				const struct ext_notify_recipient *rcpts;
 				unsigned int rcpt_count, i;
 
@@ -508,8 +502,7 @@ static int cmd_notify_operation_execute
 				rcpts = array_get(&act->recipients, &rcpt_count);
 
 				for ( i = 0; i < rcpt_count; i++ ) {
-					if ( sieve_address_compare
-						(rcpts[i].normalized, addr_norm, TRUE) == 0 )
+					if ( smtp_address_equals(rcpts[i].address, address) )
 						break;
 				}
 
@@ -532,7 +525,7 @@ static int cmd_notify_operation_execute
 					struct ext_notify_recipient recipient;
 
 					recipient.full = p_strdup(pool, str_c(raw_address));
-					recipient.normalized = p_strdup(pool, addr_norm);
+					recipient.address = smtp_address_clone(pool, address);
 
 					array_append(&act->recipients, &recipient, 1);
 				}
@@ -586,8 +579,8 @@ static int act_notify_check_duplicate
 
 	for ( i = 0; i < new_count; i++ ) {
 		for ( j = 0; j < old_count; j++ ) {
-			if ( sieve_address_compare
-				(new_rcpts[i].normalized, old_rcpts[j].normalized, TRUE) == 0 )
+			if ( smtp_address_equals
+				(new_rcpts[i].address, old_rcpts[j].address) )
 				break;
 		}
 
@@ -631,12 +624,13 @@ static void act_notify_print
 	unsigned int count, i;
 
 	sieve_result_action_printf
-		( rpenv, "send (depricated) notification with method 'mailto':");
+		( rpenv, "send (deprecated) notification with method 'mailto':");
 
 	/* Print main method parameters */
 
 	sieve_result_printf
-		( rpenv, "    => importance    : %d\n", act->importance);
+		( rpenv, "    => importance    : %llu\n",
+			(unsigned long long)act->importance);
 
 	if ( act->message != NULL )
 		sieve_result_printf
@@ -729,8 +723,8 @@ static bool act_notify_send
 		break;
 	}
 
-	rfc2822_header_printf(msg, "From",
-		"Postmaster <%s>", senv->postmaster_address);
+	rfc2822_header_write(msg, "From",
+		sieve_get_postmaster_address(senv));
 
 	rfc2822_header_write(msg, "Subject", "[SIEVE] New mail notification");
 
@@ -751,37 +745,41 @@ static bool act_notify_send
 	outmsgid = sieve_message_get_new_id(aenv->svinst);
 	rfc2822_header_write(msg, "Message-ID", outmsgid);
 
-	if ( sieve_message_get_sender(aenv->msgctx) != NULL )
-		sctx = sieve_smtp_start(senv, senv->postmaster_address);
-	else
+	if ( (aenv->flags & SIEVE_EXECUTE_FLAG_NO_ENVELOPE) == 0 &&
+		sieve_message_get_sender(aenv->msgctx) != NULL ) {
+		struct smtp_address postmaster;
+		
+		smtp_address_init_from_msg(&postmaster, 
+			sieve_get_postmaster(senv));
+		sctx = sieve_smtp_start(senv, &postmaster);
+	} else {
 		sctx = sieve_smtp_start(senv, NULL);
+	}
 
 	/* Add all recipients (and compose To header field) */
 	to = t_str_new(128);
 	all = t_str_new(256);
 	for ( i = 0; i < count; i++ ) {
-		sieve_smtp_add_rcpt(sctx, recipients[i].normalized);
+		sieve_smtp_add_rcpt(sctx, recipients[i].address);
 		if ( i > 0 )
 			str_append(to, ", ");
 		str_append(to, recipients[i].full);
 		if ( i < 3) {
 			if ( i > 0 )
 				str_append(all, ", ");
-			str_append_c(all, '<');
-			str_append(all, str_sanitize(recipients[i].normalized, 256));
-			str_append_c(all, '>');
+			str_append(all, smtp_address_encode_path(recipients[i].address));
 		} else if (i == 3) {
 			str_printfa(all, ", ... (%u total)", count);
 		}
 	}
 
-	rfc2822_header_write(msg, "To", str_c(to));
+	rfc2822_header_write_address(msg, "To", str_c(to));
 
 	/* Generate message body */
 	str_printfa(msg, "\r\n%s\r\n", act->message);
 
 	output = sieve_smtp_send(sctx);
-	o_stream_send(output, str_data(msg), str_len(msg));
+	o_stream_nsend(output, str_data(msg), str_len(msg));
 
 	if ( (ret=sieve_smtp_finish(sctx, &error)) <= 0 ) {
 		if (ret < 0) {
@@ -808,26 +806,36 @@ static int act_notify_commit
 	struct mail *mail = aenv->msgdata->mail;
 	const struct ext_notify_action *act =
 		(const struct ext_notify_action *) action->context;
-	const struct sieve_message_data *msgdata = aenv->msgdata;
 	const char *const *hdsp;
 	bool result;
+	int ret;
 
 	/* Is the message an automatic reply ? */
-	if ( mail_get_headers(mail, "auto-submitted", &hdsp) < 0 ) {
+	if ( (ret=mail_get_headers(mail, "auto-submitted", &hdsp)) < 0 ) {
 		return sieve_result_mail_error(aenv, mail,
 			"notify action: "
 			"failed to read `auto-submitted' header field");
 	}
 
 	/* Theoretically multiple headers could exist, so lets make sure */
-	while ( *hdsp != NULL ) {
-		if ( strcasecmp(*hdsp, "no") != 0 ) {
-			sieve_result_global_log(aenv,
-				"not sending notification for auto-submitted message from <%s>",
-				str_sanitize(msgdata->return_path, 128));
-			return SIEVE_EXEC_OK;
+	if (ret > 0) {
+		while ( *hdsp != NULL ) {
+			if ( strcasecmp(*hdsp, "no") != 0 ) {
+				const struct smtp_address *sender = NULL;
+				const char *from;
+
+				if ( (aenv->flags & SIEVE_EXECUTE_FLAG_NO_ENVELOPE) == 0 )
+					sender = sieve_message_get_sender(aenv->msgctx);
+				from = (sender == NULL ? "" : t_strdup_printf
+					(" from <%s>", smtp_address_encode(sender)));
+
+				sieve_result_global_log(aenv,
+					"not sending notification for auto-submitted message%s",
+					from);
+				return SIEVE_EXEC_OK;
+			}
+			hdsp++;
 		}
-		hdsp++;
 	}
 
 	T_BEGIN {
