@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 /* Match-type ':regex'
@@ -50,13 +50,10 @@ static void mcht_regex_match_deinit(struct sieve_match_context *mctx);
 
 const struct sieve_match_type_def regex_match_type = {
 	SIEVE_OBJECT("regex", &regex_match_type_operand, 0),
-	NULL,
-	mcht_regex_validate_context,
-	NULL,
-	mcht_regex_match_init,
-	mcht_regex_match_keys,
-	NULL,
-	mcht_regex_match_deinit
+	.validate_context = mcht_regex_validate_context,
+	.match_init = mcht_regex_match_init,
+	.match_keys = mcht_regex_match_keys,
+	.match_deinit = mcht_regex_match_deinit
 };
 
 /*
@@ -103,11 +100,11 @@ static int mcht_regex_validate_regexp
 			str_sanitize(regex_str, 128), _regexp_error(&regexp, ret));
 
 		regfree(&regexp);
-		return FALSE;
+		return -1;
 	}
 
 	regfree(&regexp);
-	return TRUE;
+	return 1;
 }
 
 struct _regex_key_context {
@@ -129,7 +126,7 @@ static int mcht_regex_validate_key_argument
 			(keyctx->valdtr, keyctx->mtctx, key, keyctx->cflags);
 	}
 
-	return TRUE;
+	return 1;
 }
 
 static bool mcht_regex_validate_context
@@ -161,8 +158,8 @@ static bool mcht_regex_validate_context
 	keyctx.cflags = cflags;
 
 	kitem = key_arg;
-	if ( !sieve_ast_stringlist_map(&kitem, (void *) &keyctx,
-		mcht_regex_validate_key_argument) )
+	if ( sieve_ast_stringlist_map(&kitem, (void *) &keyctx,
+		mcht_regex_validate_key_argument) <= 0 )
 		return FALSE;
 
 	return TRUE;
@@ -181,7 +178,7 @@ struct mcht_regex_context {
 	ARRAY(struct mcht_regex_key) reg_expressions;
 	regmatch_t *pmatch;
 	size_t nmatch;
-	unsigned int all_compiled:1;
+	bool all_compiled:1;
 };
 
 static void mcht_regex_match_init
