@@ -1,8 +1,5 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
- */
-
-#ifndef __EXT_VACATION_COMMON_H
-#define __EXT_VACATION_COMMON_H
+#ifndef EXT_VACATION_COMMON_H
+#define EXT_VACATION_COMMON_H
 
 #include "sieve-common.h"
 
@@ -21,6 +18,7 @@ struct ext_vacation_config {
 	bool use_original_recipient;
 	bool dont_check_recipient;
 	bool send_from_recipient;
+	bool to_header_ignore_envelope;
 };
 
 /*
@@ -55,4 +53,4 @@ extern const struct sieve_extension_def vacation_seconds_extension;
 bool ext_vacation_register_seconds_tag
 	(struct sieve_validator *valdtr, const struct sieve_extension *vacation_ext);
 
-#endif /* __EXT_VACATION_COMMON_H */
+#endif
