@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -7,6 +7,7 @@
 #include "array.h"
 
 #include "sieve-common.h"
+#include "sieve-settings.h"
 
 #include "sieve-ast.h"
 #include "sieve-binary.h"
@@ -29,9 +30,81 @@
  * Limits
  */
 
-unsigned int sieve_variables_get_max_scope_size(void)
+unsigned int
+sieve_variables_get_max_scope_size(const struct sieve_extension *var_ext)
 {
-	return EXT_VARIABLES_MAX_SCOPE_SIZE;
+	const struct ext_variables_config *config =
+		ext_variables_get_config(var_ext);
+
+	return config->max_scope_size;
+}
+
+/*
+ * Extension configuration
+ */
+
+bool ext_variables_load
+(const struct sieve_extension *ext, void **context)
+{
+	struct sieve_instance *svinst = ext->svinst;
+	struct ext_variables_config *config;
+	unsigned long long int uint_setting;
+	size_t size_setting;
+
+	if ( *context != NULL )
+		ext_variables_unload(ext);
+
+	config = i_new(struct ext_variables_config, 1);
+
+	/* Get limits */
+	config->max_scope_size = EXT_VARIABLES_DEFAULT_MAX_SCOPE_SIZE;
+	config->max_variable_size = EXT_VARIABLES_DEFAULT_MAX_VARIABLE_SIZE;
+
+	if ( sieve_setting_get_uint_value(svinst,
+		"sieve_variables_max_scope_size", &uint_setting) ) {
+		if ( uint_setting < EXT_VARIABLES_REQUIRED_MAX_SCOPE_SIZE ) {
+			sieve_sys_warning(svinst,
+				"variables: setting sieve_variables_max_scope_size "
+				"is lower than required by standards (>= %llu items)",
+				(unsigned long long)EXT_VARIABLES_REQUIRED_MAX_SCOPE_SIZE);
+		} else {
+			config->max_scope_size = (unsigned int)uint_setting;
+		}
+	}
+
+	if ( sieve_setting_get_size_value(svinst,
+		"sieve_variables_max_variable_size", &size_setting) ) {
+		if ( size_setting < EXT_VARIABLES_REQUIRED_MAX_VARIABLE_SIZE ) {
+			sieve_sys_warning(svinst,
+				"variables: setting sieve_variables_max_variable_size "
+				"is lower than required by standards (>= %"PRIuSIZE_T" bytes)",
+				(size_t)EXT_VARIABLES_REQUIRED_MAX_VARIABLE_SIZE);
+		} else {
+			config->max_variable_size = size_setting;
+		}
+	}
+
+	*context = (void *)config;
+	return TRUE;
+}
+
+void ext_variables_unload
+(const struct sieve_extension *ext)
+{
+	struct ext_variables_config *config =
+		(struct ext_variables_config *) ext->context;
+
+	i_free(config);
+}
+
+const struct ext_variables_config *ext_variables_get_config
+(const struct sieve_extension *var_ext)
+{
+	const struct ext_variables_config *config =
+		(const struct ext_variables_config *)var_ext->context;
+	i_assert(var_ext->def == &variables_extension);
+
+	return config;
 }
 
 /*
@@ -43,6 +116,7 @@ struct sieve_variable_scope {
 	int refcount;
 
 	struct sieve_instance *svinst;
+	const struct sieve_extension *var_ext;
 	const struct sieve_extension *ext;
 
 	struct sieve_variable *error_var;
@@ -65,17 +139,22 @@ struct sieve_variable_scope_iter {
 };
 
 struct sieve_variable_scope *sieve_variable_scope_create
-(struct sieve_instance *svinst, const struct sieve_extension *ext)
+(struct sieve_instance *svinst,
+	const struct sieve_extension *var_ext,
+	const struct sieve_extension *ext)
 {
 	struct sieve_variable_scope *scope;
 	pool_t pool;
 
+	i_assert(var_ext->def == &variables_extension);
+
 	pool = pool_alloconly_create("sieve_variable_scope", 4096);
 	scope = p_new(pool, struct sieve_variable_scope, 1);
 	scope->pool = pool;
 	scope->refcount = 1;
 
 	scope->svinst = svinst;
+	scope->var_ext = var_ext;
 	scope->ext = ext;
 
 	hash_table_create(&scope->variables, pool, 0, strcase_hash, strcasecmp);
@@ -112,53 +191,57 @@ pool_t sieve_variable_scope_pool(struct sieve_variable_scope *scope)
 struct sieve_variable *sieve_variable_scope_declare
 (struct sieve_variable_scope *scope, const char *identifier)
 {
-	struct sieve_variable *new_var;
+	unsigned int max_scope_size;
+	struct sieve_variable *var;
 
-	new_var = p_new(scope->pool, struct sieve_variable, 1);
-	new_var->ext = scope->ext;
+	var = hash_table_lookup(scope->variables, identifier);
+	if (var != NULL)
+		return var;
 
-	if ( array_count(&scope->variable_index) >= EXT_VARIABLES_MAX_SCOPE_SIZE ) {
+	max_scope_size = sieve_variables_get_max_scope_size(scope->var_ext);
+	if ( array_count(&scope->variable_index) >= max_scope_size ) {
 		if ( scope->error_var == NULL ) {
-			new_var->identifier = "@ERROR@";
-			new_var->index = 0;
+			var = p_new(scope->pool, struct sieve_variable, 1);
+			var->identifier = "@ERROR@";
+			var->index = 0;
 
-			scope->error_var = new_var;
+			scope->error_var = var;
 			return NULL;
 		}
 
 		return scope->error_var;
 	}
 
-	new_var->identifier = p_strdup(scope->pool, identifier);
-	new_var->index = array_count(&scope->variable_index);
+	var = p_new(scope->pool, struct sieve_variable, 1);
+	var->ext = scope->ext;
+	var->identifier = p_strdup(scope->pool, identifier);
+	var->index = array_count(&scope->variable_index);
 
-	hash_table_insert(scope->variables, new_var->identifier, new_var);
-	array_append(&scope->variable_index, &new_var, 1);
-
-	return new_var;
+	hash_table_insert(scope->variables, var->identifier, var);
+	array_append(&scope->variable_index, &var, 1);
+	return var;
 }
 
 struct sieve_variable *sieve_variable_scope_get_variable
-(struct sieve_variable_scope *scope, const char *identifier, bool declare)
+(struct sieve_variable_scope *scope, const char *identifier)
 {
-	struct sieve_variable *var;
-
-	var = hash_table_lookup(scope->variables, identifier);
-
-	if ( var == NULL && declare ) {
-		var = sieve_variable_scope_declare(scope, identifier);
-	}
-
-	return var;
+	return hash_table_lookup(scope->variables, identifier);
 }
 
 struct sieve_variable *sieve_variable_scope_import
 (struct sieve_variable_scope *scope, struct sieve_variable *var)
 {
-	struct sieve_variable *new_var;
+	struct sieve_variable *old_var, *new_var;
+
+	old_var = sieve_variable_scope_get_variable
+		(scope, var->identifier);
+	if (old_var != NULL) {
+		i_assert(memcmp(old_var, var, sizeof(*var)) == 0);
+		return old_var;
+	}
 
 	new_var = p_new(scope->pool, struct sieve_variable, 1);
-	memcpy(new_var, var, sizeof(struct sieve_variable));
+	memcpy(new_var, var, sizeof(*new_var));
 
 	hash_table_insert(scope->variables, new_var->identifier, new_var);
 
@@ -231,7 +314,9 @@ struct sieve_variable *sieve_variable_scope_get_indexed
 /* Scope binary */
 
 struct sieve_variable_scope *sieve_variable_scope_binary_dump
-(struct sieve_instance *svinst, const struct sieve_extension *ext,
+(struct sieve_instance *svinst,
+	const struct sieve_extension *var_ext,
+	const struct sieve_extension *ext,
 	const struct sieve_dumptime_env *denv, sieve_size_t *address)
 {
 	struct sieve_variable_scope *local_scope;
@@ -242,15 +327,15 @@ struct sieve_variable_scope *sieve_variable_scope_binary_dump
 	/* Read scope size */
 	sieve_code_mark(denv);
 	if ( !sieve_binary_read_unsigned(denv->sblock, address, &scope_size) )
-		return FALSE;
+		return NULL;
 
 	/* Read offset */
 	pc = *address;
 	if ( !sieve_binary_read_offset(denv->sblock, address, &end_offset) )
-		return FALSE;
+		return NULL;
 
 	/* Create scope */
-	local_scope = sieve_variable_scope_create(svinst, ext);
+	local_scope = sieve_variable_scope_create(svinst, var_ext, ext);
 
 	/* Read and dump scope itself */
 
@@ -262,7 +347,7 @@ struct sieve_variable_scope *sieve_variable_scope_binary_dump
 
 		sieve_code_mark(denv);
 		if (!sieve_binary_read_string(denv->sblock, address, &identifier) ) {
-			return FALSE;
+			return NULL;
 		}
 
 		sieve_code_dumpf(denv, "%3d: '%s'", i, str_c(identifier));
@@ -298,12 +383,14 @@ void sieve_variable_scope_binary_unref
 }
 
 struct sieve_variable_scope_binary *sieve_variable_scope_binary_read
-(struct sieve_instance *svinst, const struct sieve_extension *ext,
+(struct sieve_instance *svinst,
+	const struct sieve_extension *var_ext,
+	const struct sieve_extension *ext,
 	struct sieve_binary_block *sblock, sieve_size_t *address)
 {
 	struct sieve_variable_scope *scope;
 	struct sieve_variable_scope_binary *scpbin;
-	unsigned int scope_size;
+	unsigned int scope_size, max_scope_size;
 	const char *ext_name =
 		( ext == NULL ? "variables" : sieve_extension_name(ext) );
 	sieve_size_t pc;
@@ -317,10 +404,11 @@ struct sieve_variable_scope_binary *sieve_variable_scope_binary_read
 	}
 
 	/* Check size limit */
-	if ( scope_size > EXT_VARIABLES_MAX_SCOPE_SIZE ) {
+	max_scope_size = sieve_variables_get_max_scope_size(var_ext);
+	if ( scope_size > max_scope_size ) {
 		sieve_sys_error(svinst,
 			"%s: variable scope: size exceeds the limit (%u > %u)",
-			ext_name, scope_size, EXT_VARIABLES_MAX_SCOPE_SIZE );
+			ext_name, scope_size, max_scope_size );
 		return NULL;
 	}
 
@@ -333,7 +421,7 @@ struct sieve_variable_scope_binary *sieve_variable_scope_binary_read
 	}
 
 	/* Create scope */
-	scope = sieve_variable_scope_create(svinst, ext);
+	scope = sieve_variable_scope_create(svinst, var_ext, ext);
 
 	scpbin = sieve_variable_scope_binary_create(scope);
 	scpbin->size = scope_size;
@@ -395,6 +483,7 @@ unsigned int sieve_variable_scope_binary_get_size
 
 struct sieve_variable_storage {
 	pool_t pool;
+	const struct sieve_extension *var_ext;
 	struct sieve_variable_scope *scope;
 	struct sieve_variable_scope_binary *scope_bin;
 	unsigned int max_size;
@@ -402,12 +491,14 @@ struct sieve_variable_storage {
 };
 
 struct sieve_variable_storage *sieve_variable_storage_create
-(pool_t pool, struct sieve_variable_scope_binary *scpbin)
+(const struct sieve_extension *var_ext, pool_t pool,
+	struct sieve_variable_scope_binary *scpbin)
 {
 	struct sieve_variable_storage *storage;
 
 	storage = p_new(pool, struct sieve_variable_storage, 1);
 	storage->pool = pool;
+	storage->var_ext = var_ext;
 	storage->scope_bin = scpbin;
 	storage->scope = NULL;
 
@@ -508,6 +599,8 @@ bool sieve_variable_assign
 (struct sieve_variable_storage *storage, unsigned int index,
 	const string_t *value)
 {
+	const struct ext_variables_config *config =
+		ext_variables_get_config(storage->var_ext);
 	string_t *varval;
 
 	if ( !sieve_variable_get_modifiable(storage, index, &varval) )
@@ -517,8 +610,8 @@ bool sieve_variable_assign
 	str_append_str(varval, value);
 
 	/* Just a precaution, caller should prevent this in the first place */
-	if ( str_len(varval) > EXT_VARIABLES_MAX_VARIABLE_SIZE )
-		str_truncate(varval, EXT_VARIABLES_MAX_VARIABLE_SIZE);
+	if ( str_len(varval) > config->max_variable_size )
+		str_truncate(varval, config->max_variable_size);
 
 	return TRUE;
 }
@@ -527,6 +620,8 @@ bool sieve_variable_assign_cstr
 (struct sieve_variable_storage *storage, unsigned int index,
 	const char *value)
 {
+	const struct ext_variables_config *config =
+		ext_variables_get_config(storage->var_ext);
 	string_t *varval;
 
 	if ( !sieve_variable_get_modifiable(storage, index, &varval) )
@@ -536,8 +631,8 @@ bool sieve_variable_assign_cstr
 	str_append(varval, value);
 
 	/* Just a precaution, caller should prevent this in the first place */
-	if ( str_len(varval) > EXT_VARIABLES_MAX_VARIABLE_SIZE )
-		str_truncate(varval, EXT_VARIABLES_MAX_VARIABLE_SIZE);
+	if ( str_len(varval) > config->max_variable_size )
+		str_truncate(varval, config->max_variable_size);
 
 	return TRUE;
 }
@@ -567,7 +662,8 @@ static struct sieve_variable_scope *ext_variables_create_local_scope
 {
 	struct sieve_variable_scope *scope;
 
-	scope = sieve_variable_scope_create(this_ext->svinst, NULL);
+	scope = sieve_variable_scope_create
+		(this_ext->svinst, this_ext, NULL);
 
 	sieve_ast_extension_register
 		(ast, this_ext, &variables_ast_extension, (void *) scope);
@@ -608,8 +704,10 @@ ext_variables_validator_context_create
 struct ext_variables_validator_context *ext_variables_validator_context_get
 (const struct sieve_extension *this_ext, struct sieve_validator *valdtr)
 {
-	struct ext_variables_validator_context *ctx =
-		(struct ext_variables_validator_context *)
+	struct ext_variables_validator_context *ctx;
+
+	i_assert( sieve_extension_is(this_ext, variables_extension) );
+	ctx = (struct ext_variables_validator_context *)
 		sieve_validator_extension_get_context(valdtr, this_ext);
 
 	if ( ctx == NULL ) {
@@ -634,12 +732,22 @@ void ext_variables_validator_initialize
 
 struct sieve_variable *ext_variables_validator_get_variable
 (const struct sieve_extension *this_ext, struct sieve_validator *validator,
-	const char *variable, bool declare)
+	const char *variable)
 {
 	struct ext_variables_validator_context *ctx =
 		ext_variables_validator_context_get(this_ext, validator);
 
-	return sieve_variable_scope_get_variable(ctx->local_scope, variable, declare);
+	return sieve_variable_scope_get_variable(ctx->local_scope, variable);
+}
+
+struct sieve_variable *ext_variables_validator_declare_variable
+(const struct sieve_extension *this_ext, struct sieve_validator *validator,
+	const char *variable)
+{
+	struct ext_variables_validator_context *ctx =
+		ext_variables_validator_context_get(this_ext, validator);
+
+	return sieve_variable_scope_declare(ctx->local_scope, variable);
 }
 
 struct sieve_variable_scope *sieve_ext_variables_get_local_scope
@@ -715,10 +823,10 @@ static void ext_variables_interpreter_free
 	sieve_variable_scope_binary_unref(&ctx->local_scope_bin);
 }
 
-static struct sieve_interpreter_extension variables_interpreter_extension = {
-	&variables_extension,
-	NULL,
-	ext_variables_interpreter_free
+static struct sieve_interpreter_extension
+variables_interpreter_extension = {
+	.ext_def = &variables_extension,
+	.free = ext_variables_interpreter_free
 };
 
 static struct ext_variables_interpreter_context *
@@ -733,7 +841,8 @@ ext_variables_interpreter_context_create
 	ctx->pool = pool;
 	ctx->local_scope = NULL;
 	ctx->local_scope_bin = scpbin;
-	ctx->local_storage = sieve_variable_storage_create(pool, scpbin);
+	ctx->local_storage =
+		sieve_variable_storage_create(this_ext, pool, scpbin);
 	p_array_init(&ctx->ext_storages, pool,
 		sieve_extensions_get_count(this_ext->svinst));
 
@@ -750,7 +859,7 @@ bool ext_variables_interpreter_load
 	struct sieve_variable_scope_binary *scpbin;
 
 	scpbin = sieve_variable_scope_binary_read
-		(renv->svinst, NULL, renv->sblock, address);
+		(renv->svinst, ext, NULL, renv->sblock, address);
 	if ( scpbin == NULL )
 		return FALSE;
 
@@ -768,8 +877,13 @@ static inline struct ext_variables_interpreter_context *
 ext_variables_interpreter_context_get
 	(const struct sieve_extension *this_ext, struct sieve_interpreter *interp)
 {
-	return (struct ext_variables_interpreter_context *)
+	struct ext_variables_interpreter_context *ctx;
+
+	i_assert( sieve_extension_is(this_ext, variables_extension) );
+	ctx = (struct ext_variables_interpreter_context *)
 		sieve_interpreter_extension_get_context(interp, this_ext);
+
+	return ctx;
 }
 
 struct sieve_variable_storage *sieve_ext_variables_runtime_get_storage
