@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2016 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 #ifndef __EXT_VARIABLES_COMMON_H
@@ -13,8 +13,23 @@
  * Extension
  */
 
+struct ext_variables_config {
+	/* Maximum number of variables (in a scope) */
+	unsigned int max_scope_size;
+	/* Maximum size of variable value */
+	size_t max_variable_size;
+};
+
 extern const struct sieve_extension_def variables_extension;
 
+bool ext_variables_load
+	(const struct sieve_extension *ext, void **context);
+void ext_variables_unload
+	(const struct sieve_extension *ext);
+
+const struct ext_variables_config *ext_variables_get_config
+	(const struct sieve_extension *var_ext);
+
 /*
  * Commands
  */
@@ -66,7 +81,10 @@ struct ext_variables_validator_context *ext_variables_validator_context_get
 
 struct sieve_variable *ext_variables_validator_get_variable
 	(const struct sieve_extension *this_ext, struct sieve_validator *validator,
-		const char *variable, bool declare);
+		const char *variable);
+struct sieve_variable *ext_variables_validator_declare_variable
+	(const struct sieve_extension *this_ext, struct sieve_validator *validator,
+		const char *variable);
 
 /*
  * Code generation
