@@ -1,8 +1,5 @@
-/* Copyright (c) 2002-2016 Pigeonhole authors, see the included COPYING file
- */
-
-#ifndef __EXT_VARIABLES_MODIFIERS_H
-#define __EXT_VARIABLES_MODIFIERS_H
+#ifndef EXT_VARIABLES_MODIFIERS_H
+#define EXT_VARIABLES_MODIFIERS_H
 
 #include "sieve-common.h"
 #include "sieve-runtime-trace.h"
@@ -63,4 +60,4 @@ static inline bool ext_variables_opr_modifier_dump
 		(denv, &sieve_variables_modifier_operand_class, address, NULL);
 }
 
-#endif /* __EXT_VARIABLES_MODIFIERS_H */
+#endif
