@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -41,17 +41,19 @@ const struct sieve_opr_string_interface variable_interface = {
 };
 
 const struct sieve_operand_def variable_operand = {
-	"variable",
-	&variables_extension,
-	EXT_VARIABLES_OPERAND_VARIABLE,
-	&string_class,
-	&variable_interface
+	.name = "variable",
+	.ext_def = &variables_extension,
+	.code = EXT_VARIABLES_OPERAND_VARIABLE,
+	.class = &string_class,
+	.interface = &variable_interface
 };
 
 void sieve_variables_opr_variable_emit
 (struct sieve_binary_block *sblock, const struct sieve_extension *var_ext,
 	struct sieve_variable *var)
 {
+	i_assert( sieve_extension_is(var_ext, variables_extension) );
+
 	if ( var->ext == NULL ) {
 		/* Default variable storage */
 		(void) sieve_operand_emit(sblock, var_ext, &variable_operand);
@@ -211,17 +213,18 @@ const struct sieve_opr_string_interface match_value_interface = {
 };
 
 const struct sieve_operand_def match_value_operand = {
-	"match-value",
-	&variables_extension,
-	EXT_VARIABLES_OPERAND_MATCH_VALUE,
-	&string_class,
-	&match_value_interface
+	.name = "match-value",
+	.ext_def = &variables_extension,
+	.code = EXT_VARIABLES_OPERAND_MATCH_VALUE,
+	.class = &string_class,
+	.interface = &match_value_interface
 };
 
 void sieve_variables_opr_match_value_emit
 (struct sieve_binary_block *sblock, const struct sieve_extension *var_ext,
 	unsigned int index)
 {
+	i_assert( sieve_extension_is(var_ext, variables_extension) );
 	(void) sieve_operand_emit(sblock, var_ext, &match_value_operand);
 	(void) sieve_binary_emit_unsigned(sblock, index);
 }
@@ -249,6 +252,9 @@ static int opr_match_value_read
 (const struct sieve_runtime_env *renv, const struct sieve_operand *oprnd,
 	sieve_size_t *address, string_t **str_r)
 {
+	const struct sieve_extension *this_ext = oprnd->ext;
+	const struct ext_variables_config *config =
+		ext_variables_get_config(this_ext);
 	unsigned int index = 0;
 
 	if ( sieve_binary_read_unsigned(renv->sblock, address, &index) ) {
@@ -260,8 +266,8 @@ static int opr_match_value_read
 
 			if ( *str_r == NULL )
 				*str_r = t_str_new(0);
-			else if ( str_len(*str_r) > EXT_VARIABLES_MAX_VARIABLE_SIZE )
-				str_truncate(*str_r, EXT_VARIABLES_MAX_VARIABLE_SIZE);
+			else if ( str_len(*str_r) > config->max_variable_size )
+				str_truncate(*str_r, config->max_variable_size);
 		}
 
 		return SIEVE_EXEC_OK;
