@@ -317,7 +317,7 @@ int sieve_binary_save
 
 	/* Save binary */
 	result = 1;
-	stream = o_stream_create_fd(fd, 0, FALSE);
+	stream = o_stream_create_fd(fd, 0);
 	if ( !_sieve_binary_save(sbin, stream) ) {
 		result = -1;
 		if ( error_r != NULL )
@@ -630,7 +630,7 @@ static bool _file_lazy_read
 static const void *_file_lazy_load_data
 (struct sieve_binary_file *file, off_t *offset, size_t size)
 {
-	void *data = t_malloc(size);
+	void *data = t_malloc_no0(size);
 
 	if ( _file_lazy_read(file, offset, data, size) ) {
 		return data;
