@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -264,9 +264,9 @@ const struct sieve_operand_class omitted_class =
 	{ "OMITTED" };
 
 const struct sieve_operand_def omitted_operand = {
-	"@OMITTED",
-	NULL, SIEVE_OPERAND_OPTIONAL,
-	&omitted_class, NULL
+	.name = "@OMITTED",
+	.code = SIEVE_OPERAND_OPTIONAL,
+	.class = &omitted_class
 };
 
 /* Number */
@@ -287,10 +287,10 @@ const struct sieve_operand_class number_class =
 	{ "number" };
 
 const struct sieve_operand_def number_operand = {
-	"@number",
-	NULL, SIEVE_OPERAND_NUMBER,
-	&number_class,
-	&number_interface
+	.name = "@number",
+	.code = SIEVE_OPERAND_NUMBER,
+	.class = &number_class,
+	.interface = &number_interface
 };
 
 /* String */
@@ -311,10 +311,10 @@ const struct sieve_operand_class string_class =
 	{ "string" };
 
 const struct sieve_operand_def string_operand = {
-	"@string",
-	NULL, SIEVE_OPERAND_STRING,
-	&string_class,
-	&string_interface
+	.name = "@string",
+	.code = SIEVE_OPERAND_STRING,
+	.class = &string_class,
+	.interface = &string_interface
 };
 
 /* String List */
@@ -335,10 +335,10 @@ const struct sieve_operand_class stringlist_class =
 	{ "string-list" };
 
 const struct sieve_operand_def stringlist_operand =	{
-	"@string-list",
-	NULL, SIEVE_OPERAND_STRING_LIST,
-	&stringlist_class,
-	&stringlist_interface
+	.name = "@string-list",
+	.code = SIEVE_OPERAND_STRING_LIST,
+	.class = &stringlist_class,
+	.interface = &stringlist_interface
 };
 
 /* Catenated String */
@@ -356,10 +356,10 @@ const struct sieve_opr_string_interface catenated_string_interface = {
 };
 
 const struct sieve_operand_def catenated_string_operand = {
-	"@catenated-string",
-	NULL, SIEVE_OPERAND_CATENATED_STRING,
-	&string_class,
-	&catenated_string_interface
+	.name = "@catenated-string",
+	.code = SIEVE_OPERAND_CATENATED_STRING,
+	.class = &string_class,
+	.interface = &catenated_string_interface
 };
 
 /*
@@ -607,7 +607,7 @@ int sieve_opr_string_read_ex
 
 	if ( optional && sieve_operand_is_omitted(&operand) ) {
 		*str_r = NULL;
-		return TRUE;
+		return 1;
 	}
 
 	if ( literal_r != NULL )
@@ -848,7 +848,7 @@ int sieve_opr_stringlist_read_ex
 
 	if ( optional && sieve_operand_is_omitted(&operand) ) {
 		*strlist_r = NULL;
-		return TRUE;
+		return 1;
 	}
 
 	return sieve_opr_stringlist_read_data
@@ -1009,27 +1009,24 @@ static int opc_jmpfalse_execute
 /* Operation objects defined in this file */
 
 const struct sieve_operation_def sieve_jmp_operation = {
-	"JMP",
-	NULL,
-	SIEVE_OPERATION_JMP,
-	opc_jmp_dump,
-	opc_jmp_execute
+	.mnemonic = "JMP",
+	.code = SIEVE_OPERATION_JMP,
+	.dump = opc_jmp_dump,
+	.execute = opc_jmp_execute
 };
 
 const struct sieve_operation_def sieve_jmptrue_operation = {
-	"JMPTRUE",
-	NULL,
-	SIEVE_OPERATION_JMPTRUE,
-	opc_jmp_dump,
-	opc_jmptrue_execute
+	.mnemonic = "JMPTRUE",
+	.code = SIEVE_OPERATION_JMPTRUE,
+	.dump = opc_jmp_dump,
+	.execute = opc_jmptrue_execute
 };
 
 const struct sieve_operation_def sieve_jmpfalse_operation = {
-	"JMPFALSE",
-	NULL,
-	SIEVE_OPERATION_JMPFALSE,
-	opc_jmp_dump,
-	opc_jmpfalse_execute
+	.mnemonic = "JMPFALSE",
+	.code = SIEVE_OPERATION_JMPFALSE,
+	.dump = opc_jmp_dump,
+	.execute = opc_jmpfalse_execute
 };
 
 /* Operation objects defined in other files */
@@ -1078,15 +1075,16 @@ sieve_size_t sieve_operation_emit
 	sieve_size_t address;
 
   if ( ext != NULL ) {
+		i_assert( op_def->ext_def != NULL );
 		address = sieve_binary_emit_extension
 			(sblock, ext, sieve_operation_count);
 
 		sieve_binary_emit_extension_object
 			(sblock, &op_def->ext_def->operations, op_def->code);
-
 		return address;
   }
 
+	i_assert( op_def->ext_def == NULL );
   return sieve_binary_emit_byte(sblock, op_def->code);
 }
 
@@ -1103,7 +1101,7 @@ bool sieve_operation_read
 	if ( !sieve_binary_read_extension(sblock, address, &code, &oprtn->ext) )
 		return FALSE;
 
-	if ( !oprtn->ext ) {
+	if ( oprtn->ext == NULL ) {
 		if ( code < sieve_operation_count ) {
 			oprtn->def = sieve_operations[code];
 		}
@@ -1145,7 +1143,7 @@ static bool opc_jmp_dump
 static int opc_jmp_execute
 (const struct sieve_runtime_env *renv, sieve_size_t *address ATTR_UNUSED)
 {
-	return sieve_interpreter_program_jump(renv->interp, TRUE);
+	return sieve_interpreter_program_jump(renv->interp, TRUE, FALSE);
 }
 
 static int opc_jmptrue_execute
@@ -1156,7 +1154,7 @@ static int opc_jmptrue_execute
 	sieve_runtime_trace(renv, SIEVE_TRLVL_COMMANDS, "jump if result is true");
 	sieve_runtime_trace_descend(renv);
 
-	return sieve_interpreter_program_jump(renv->interp, result);
+	return sieve_interpreter_program_jump(renv->interp, result, FALSE);
 }
 
 static int opc_jmpfalse_execute
@@ -1167,5 +1165,5 @@ static int opc_jmpfalse_execute
 	sieve_runtime_trace(renv, SIEVE_TRLVL_COMMANDS, "jump if result is false");
 	sieve_runtime_trace_descend(renv);
 
-	return sieve_interpreter_program_jump(renv->interp, !result);
+	return sieve_interpreter_program_jump(renv->interp, !result, FALSE);
 }
