@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 #ifndef __SIEVE_COMMON_H
@@ -17,16 +17,9 @@
 
 typedef size_t sieve_size_t;
 typedef uint32_t sieve_offset_t;
-typedef uint32_t sieve_number_t;
+typedef uint64_t sieve_number_t;
 
-#define SIEVE_MAX_NUMBER ((sieve_number_t) -1)
-
-enum sieve_redirect_envelope_from {
-	SIEVE_REDIRECT_ENVELOPE_FROM_SENDER,
-	SIEVE_REDIRECT_ENVELOPE_FROM_RECIPIENT,
-	SIEVE_REDIRECT_ENVELOPE_FROM_ORIG_RECIPIENT,
-	SIEVE_REDIRECT_ENVELOPE_FROM_EXPLICIT
-};
+#define SIEVE_MAX_NUMBER ((sieve_number_t)-1)
 
 /*
  * Forward declarations
@@ -117,7 +110,6 @@ struct sieve_match_type;
 struct sieve_match_context;
 
 /* sieve-address.h */
-struct sieve_address;
 struct sieve_address_list;
 
 /* sieve-address-parts.h */
@@ -164,6 +156,8 @@ bool sieve_validate
  * Sieve engine instance
  */
 
+#include "sieve-address-source.h"
+
 struct sieve_instance {
 	/* Main engine pool */
 	pool_t pool;
@@ -206,8 +200,33 @@ struct sieve_instance {
 	size_t max_script_size;
 	unsigned int max_actions;
 	unsigned int max_redirects;
-	enum sieve_redirect_envelope_from redirect_from;
-	const char *redirect_from_explicit;
+	const struct smtp_address *user_email, *user_email_implicit;
+	struct sieve_address_source redirect_from;
+	unsigned int redirect_duplicate_period;
 };
 
+/*
+ * Script trace log
+ */
+
+void sieve_trace_log_write_line
+	(struct sieve_trace_log *trace_log, const string_t *line)
+	ATTR_NULL(2);
+
+/*
+ * User e-mail address
+ */
+
+const struct smtp_address *sieve_get_user_email
+	(struct sieve_instance *svinst);
+
+/*
+ * Postmaster address 
+ */
+
+const struct message_address *
+sieve_get_postmaster(const struct sieve_script_env *senv);
+const char *
+sieve_get_postmaster_address(const struct sieve_script_env *senv);
+
 #endif /* __SIEVE_COMMON_H */
