@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -118,11 +118,9 @@ static bool tag_comparator_generate
 /* Argument object */
 
 const struct sieve_argument_def comparator_tag = {
-	"comparator",
-	NULL,
-	tag_comparator_validate,
-	NULL, NULL,
-	tag_comparator_generate
+	.identifier = "comparator",
+	.validate = tag_comparator_validate,
+	.generate = tag_comparator_generate
 };
 
 /* Argument implementation */
@@ -239,11 +237,10 @@ static const struct sieve_extension_objects core_comparators =
 	SIEVE_EXT_DEFINE_COMPARATORS(sieve_core_comparators);
 
 const struct sieve_operand_def comparator_operand = {
-	"comparator",
-	NULL,
-	SIEVE_OPERAND_COMPARATOR,
-	&sieve_comparator_operand_class,
-	&core_comparators
+	.name = "comparator",
+	.code = SIEVE_OPERAND_COMPARATOR,
+	.class = &sieve_comparator_operand_class,
+	.interface = &core_comparators
 };
 
 /*
