@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2014 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -15,7 +15,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>
 
@@ -97,9 +96,9 @@ void sieve_direct_verror
 				}
 			}
 			va_end(args_copy);
+			if (svinst->system_ehandler == ehandler)
+				return;
 		}
-		if (svinst->system_ehandler == ehandler)
-			return;
 	}
 
 	if ( ehandler == NULL )
@@ -139,9 +138,9 @@ void sieve_direct_vwarning
 				}
 			}
 			va_end(args_copy);
+			if (svinst->system_ehandler == ehandler)
+				return;
 		}
-		if (svinst->system_ehandler == ehandler)
-			return;
 	}
 
 	if ( ehandler == NULL )
@@ -1063,7 +1062,7 @@ static void sieve_logfile_start(struct sieve_logfile_ehandler *ehandler)
 		}
 	}
 
-	ostream = o_stream_create_fd(fd, 0, FALSE);
+	ostream = o_stream_create_fd(fd, 0);
 	if ( ostream == NULL ) {
 		/* Can't we do anything else in this most awkward situation? */
 		sieve_sys_error(svinst, "failed to open log stream on open file: "
@@ -1301,13 +1300,22 @@ static const char *ATTR_FORMAT(3, 0) _expand_message
 	struct var_expand_table *table =
 		array_get_modifiable(&ehandler->table, &count);
 	string_t *str = t_str_new(256);
+	const char *error;
+	static bool expand_error_logged = FALSE;
 
 	/* Fill in substitution items */
 	table[0].value = t_strdup_vprintf(fmt, args);
 	table[1].value = location;
 
 	/* Expand variables */
-	var_expand(str, ehandler->format, table);
+	if (var_expand(str, ehandler->format, table, &error) <= 0 &&
+	    !expand_error_logged) {
+		/* Log the error only once. This also prevents recursively
+		   looping back here. */
+		expand_error_logged = TRUE;
+		sieve_sys_error(_ehandler->svinst,
+			"Failed to expand error message: %s", error);
+	}
 
 	return str_c(str);
 }
