@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2016 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -108,6 +108,7 @@ extern const struct sieve_extension_def ihave_extension;
 extern const struct sieve_extension_def duplicate_extension;
 extern const struct sieve_extension_def mime_extension;
 extern const struct sieve_extension_def foreverypart_extension;
+extern const struct sieve_extension_def extracttext_extension;
 extern const struct sieve_extension_def mboxmetadata_extension;
 extern const struct sieve_extension_def servermetadata_extension;
 
@@ -123,7 +124,8 @@ const struct sieve_extension_def *sieve_core_extensions[] = {
 	&copy_extension, &include_extension, &body_extension,
 	&variables_extension, &enotify_extension, &environment_extension,
 	&mailbox_extension, &date_extension, &index_extension, &ihave_extension,
-	&duplicate_extension, &mime_extension, &foreverypart_extension
+	&duplicate_extension, &mime_extension, &foreverypart_extension,
+	&extracttext_extension
 };
 
 const unsigned int sieve_core_extensions_count =
@@ -142,6 +144,7 @@ extern const struct sieve_extension_def editheader_extension;
 
 extern const struct sieve_extension_def vnd_debug_extension;
 extern const struct sieve_extension_def vnd_environment_extension;
+extern const struct sieve_extension_def vnd_report_extension;
 
 const struct sieve_extension_def *sieve_extra_extensions[] = {
 	&vacation_seconds_extension, &spamtest_extension, &spamtestplus_extension,
@@ -149,7 +152,7 @@ const struct sieve_extension_def *sieve_extra_extensions[] = {
 	&mboxmetadata_extension, &servermetadata_extension,
 
 	/* vnd.dovecot. */
-	&vnd_debug_extension, &vnd_environment_extension
+	&vnd_debug_extension, &vnd_environment_extension, &vnd_report_extension
 };
 
 const unsigned int sieve_extra_extensions_count =
@@ -271,13 +274,21 @@ void sieve_extensions_configure(struct sieve_instance *svinst)
 
 	/* Apply sieve_extensions configuration */
 
-	if ( (extensions=sieve_setting_get(svinst, "sieve_extensions")) != NULL )
-		sieve_extensions_set_string(svinst, extensions, FALSE);
+	if ( (extensions=sieve_setting_get
+		(svinst, "sieve_extensions")) != NULL )
+		sieve_extensions_set_string(svinst, extensions, FALSE, FALSE);
 
 	/* Apply sieve_global_extensions configuration */
 
-	if ( (extensions=sieve_setting_get(svinst, "sieve_global_extensions")) != NULL )
-		sieve_extensions_set_string(svinst, extensions, TRUE);
+	if ( (extensions=sieve_setting_get
+		(svinst, "sieve_global_extensions")) != NULL )
+		sieve_extensions_set_string(svinst, extensions, TRUE, FALSE);
+
+	/* Apply sieve_implicit_extensions configuration */
+
+	if ( (extensions=sieve_setting_get
+		(svinst, "sieve_implicit_extensions")) != NULL )
+		sieve_extensions_set_string(svinst, extensions, FALSE, TRUE);
 }
 
 void sieve_extensions_deinit(struct sieve_instance *svinst)
@@ -320,6 +331,7 @@ static void _sieve_extension_unload(struct sieve_extension *ext)
 	/* Call unload handler */
 	if ( ext->def != NULL && ext->def->unload != NULL )
 		ext->def->unload(ext);
+	ext->context = NULL;
 }
 
 static void sieve_extension_registry_init(struct sieve_instance *svinst)
@@ -371,15 +383,6 @@ static struct sieve_extension *sieve_extension_lookup
 	return 	hash_table_lookup(ext_reg->extension_index, name);
 }
 
-static void sieve_extension_insert
-(struct sieve_instance *svinst, const char *name,
-	struct sieve_extension *ext)
-{
-	struct sieve_extension_registry *ext_reg = svinst->ext_reg;
-
-	hash_table_insert(ext_reg->extension_index, name, ext);
-} 
-
 static struct sieve_extension *sieve_extension_alloc
 (struct sieve_instance *svinst,
 	const struct sieve_extension_def *extdef)
@@ -410,7 +413,8 @@ static struct sieve_extension *_sieve_extension_register
 	/* Register extension if it is not registered already */
 	if ( ext == NULL ) {
 		ext = sieve_extension_alloc(svinst, extdef);
-		sieve_extension_insert(svinst, extdef->name, ext);
+		hash_table_insert
+			(svinst->ext_reg->extension_index, extdef->name, ext);
 
 	} else if ( ext->overridden ) {
 		/* Create a dummy */
@@ -466,6 +470,18 @@ void sieve_extension_unregister(const struct sieve_extension *ext)
 	}
 }
 
+const struct sieve_extension *sieve_extension_replace
+(struct sieve_instance *svinst, const struct sieve_extension_def *extdef,
+	bool load)
+{
+	struct sieve_extension *ext;
+
+	ext = sieve_extension_lookup(svinst, extdef->name);
+	if (ext != NULL)
+		sieve_extension_unregister(ext);
+	return sieve_extension_register(svinst, extdef, load);
+}
+
 const struct sieve_extension *sieve_extension_require
 (struct sieve_instance *svinst, const struct sieve_extension_def *extdef,
 	bool load)
@@ -481,24 +497,38 @@ void sieve_extension_override
 	struct sieve_extension * const *mod_ext;
 	struct sieve_extension *old_ext;
 
+	old_ext = sieve_extension_lookup(svinst, name);
+	if (old_ext == ext)
+		return;
+	i_assert( old_ext == NULL || !old_ext->overridden );
+
 	i_assert( ext->id >= 0 &&
 		ext->id < (int) array_count(&ext_reg->extensions) );
 	mod_ext = array_idx(&ext_reg->extensions, ext->id);
 
-	old_ext = sieve_extension_lookup(svinst, name);
-	i_assert( old_ext == NULL || !old_ext->overridden );
-
-	sieve_extension_insert(svinst, name, *mod_ext);
-	(*mod_ext)->overridden = TRUE;
+	hash_table_update
+		(ext_reg->extension_index, name, *mod_ext);
+	if ( old_ext != NULL )
+		old_ext->overridden = TRUE;
 }
 
-int sieve_extensions_get_count(struct sieve_instance *svinst)
+unsigned int sieve_extensions_get_count(struct sieve_instance *svinst)
 {
 	struct sieve_extension_registry *ext_reg = svinst->ext_reg;
 
 	return array_count(&ext_reg->extensions);
 }
 
+const struct sieve_extension *const *
+sieve_extensions_get_all(struct sieve_instance *svinst,
+	unsigned int *count_r)
+{
+	struct sieve_extension_registry *ext_reg = svinst->ext_reg;
+
+	return (const struct sieve_extension *const *)
+		array_get(&ext_reg->extensions, count_r);
+}
+
 const struct sieve_extension *sieve_extension_get_by_id
 (struct sieve_instance *svinst, unsigned int ext_id)
 {
@@ -536,7 +566,7 @@ const struct sieve_extension *sieve_extension_get_by_name
 static inline bool _sieve_extension_listable(const struct sieve_extension *ext)
 {
 	return ( ext->enabled && ext->def != NULL && *(ext->def->name) != '@'
-		&& !ext->dummy && !ext->global );
+		&& !ext->dummy && !ext->global && !ext->overridden);
 }
 
 const char *sieve_extensions_get_string(struct sieve_instance *svinst)
@@ -578,7 +608,6 @@ static void sieve_extension_set_enabled
 {
 	if ( enabled ) {
 		ext->enabled = TRUE;
-		ext->global = FALSE;
 
 		if ( !ext->loaded ) {
 			(void)_sieve_extension_load(ext);
@@ -594,21 +623,27 @@ static void sieve_extension_set_global
 (struct sieve_extension *ext, bool enabled)
 {
 	if ( enabled ) {
-		ext->enabled = TRUE;
+		sieve_extension_set_enabled(ext, TRUE);
 		ext->global = TRUE;
-
-		if ( !ext->loaded ) {
-			(void)_sieve_extension_load(ext);
-		}
-
-		ext->loaded = TRUE;
 	} else {
 		ext->global = FALSE;
 	}
 }
 
+static void sieve_extension_set_implicit
+(struct sieve_extension *ext, bool enabled)
+{
+	if ( enabled ) {
+		sieve_extension_set_enabled(ext, TRUE);
+		ext->implicit = TRUE;
+	} else {
+		ext->implicit = FALSE;
+	}
+}
+
 void sieve_extensions_set_string
-(struct sieve_instance *svinst, const char *ext_string, bool global)
+(struct sieve_instance *svinst, const char *ext_string,
+	bool global, bool implicit)
 {
 	struct sieve_extension_registry *ext_reg = svinst->ext_reg;
 	ARRAY(const struct sieve_extension *) enabled_extensions;
@@ -621,7 +656,7 @@ void sieve_extensions_set_string
 	bool relative = FALSE;
 
 	if ( ext_string == NULL ) {
-		if ( global ) return;
+		if ( global || implicit ) return;
 
 		/* Enable all */
 		exts = array_get_modifiable(&ext_reg->extensions, &ext_count);
@@ -694,6 +729,8 @@ void sieve_extensions_set_string
 			if ( relative ) {
 				if ( global )
 					enabled = exts[i]->global;
+				else if ( implicit )
+					enabled = exts[i]->implicit;
 				else
 					enabled = exts[i]->enabled;
 
@@ -720,6 +757,8 @@ void sieve_extensions_set_string
 			/* Perform actual activation/deactivation */
 			if ( global ) {
 				sieve_extension_set_global(exts[i], enabled);
+			} else if ( implicit ) {
+				sieve_extension_set_implicit(exts[i], enabled);
 			} else {
 				sieve_extension_set_enabled(exts[i], enabled);
 			}
@@ -782,9 +821,15 @@ void sieve_extension_capabilities_register
 {
 	struct sieve_instance *svinst = ext->svinst;
 	struct sieve_extension_registry *ext_reg = svinst->ext_reg;
-	struct sieve_capability_registration *reg =
-		p_new(svinst->pool, struct sieve_capability_registration, 1);
+	struct sieve_capability_registration *reg;
 
+	reg = hash_table_lookup(ext_reg->capabilities_index, cap->name);
+	if (reg != NULL) {
+		/* Already registered */
+		return;
+	}
+
+	reg = p_new(svinst->pool, struct sieve_capability_registration, 1);
 	reg->ext = ext;
 	reg->capabilities = cap;
 
