@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -36,6 +36,9 @@ struct sieve_interpreter_extension_reg {
 	const struct sieve_extension *ext;
 
 	void *context;
+
+	bool deferred:1;
+	bool started:1;
 };
 
 /*
@@ -94,7 +97,7 @@ static struct sieve_interpreter *_sieve_interpreter_create
 	const struct sieve_message_data *msgdata,
 	const struct sieve_script_env *senv,
 	struct sieve_error_handler *ehandler,
-	enum sieve_runtime_flags flags)
+	enum sieve_execute_flags flags)
 	ATTR_NULL(3, 4)
 {
 	unsigned int i, ext_count;
@@ -127,8 +130,8 @@ static struct sieve_interpreter *_sieve_interpreter_create
 	interp->runenv.msgdata = msgdata;
 	interp->runenv.scriptenv = senv;
 
-	if ( senv->trace_stream != NULL ) {
-		interp->trace.stream = senv->trace_stream;
+	if ( senv->trace_log != NULL ) {
+		interp->trace.log = senv->trace_log;
 		interp->trace.config = senv->trace_config;
 		interp->trace.indent = 0;
 		interp->runenv.trace = &interp->trace;
@@ -186,16 +189,26 @@ static struct sieve_interpreter *_sieve_interpreter_create
 		sieve_binary_read_unsigned(sblock, address, &ext_count) ) {
 
 		for ( i = 0; i < ext_count; i++ ) {
-			unsigned int code = 0;
+			unsigned int code = 0, deferred;
+			struct sieve_interpreter_extension_reg *reg;
 			const struct sieve_extension *ext;
 
-			if ( !sieve_binary_read_extension(sblock, address, &code, &ext) ) {
+			if ( !sieve_binary_read_extension
+					(sblock, address, &code, &ext) ||
+				!sieve_binary_read_byte
+					(sblock, address, &deferred) ) {
 				success = FALSE;
 				break;
 			}
 
+			if ( deferred > 0 && ext->id >= 0 ) {
+				reg = array_idx_modifiable
+					(&interp->extensions, (unsigned int)ext->id);
+				reg->deferred = TRUE;
+			}
+
 			if ( ext->def != NULL ) {
-				if ( ext->global && (flags & SIEVE_RUNTIME_FLAG_NOGLOBAL) != 0 ) {
+				if ( ext->global && (flags & SIEVE_EXECUTE_FLAG_NOGLOBAL) != 0 ) {
 					sieve_runtime_error(&interp->runenv, NULL,
 						"failed to enable extension `%s': "
 						"its use is restricted to global scripts",
@@ -230,7 +243,7 @@ struct sieve_interpreter *sieve_interpreter_create
 	const struct sieve_message_data *msgdata,
 	const struct sieve_script_env *senv,
 	struct sieve_error_handler *ehandler,
-	enum sieve_runtime_flags flags)
+	enum sieve_execute_flags flags)
 {
 	struct sieve_binary_block *sblock;
 
@@ -249,7 +262,7 @@ struct sieve_interpreter *sieve_interpreter_create_for_block
 	const struct sieve_message_data *msgdata,
 	const struct sieve_script_env *senv,
 	struct sieve_error_handler *ehandler,
-	enum sieve_runtime_flags flags)
+	enum sieve_execute_flags flags)
 {
 	if ( sblock == NULL ) return NULL;
 
@@ -506,6 +519,34 @@ void *sieve_interpreter_extension_get_context
 	return reg->context;
 }
 
+int sieve_interpreter_extension_start
+(struct sieve_interpreter *interp,
+	const struct sieve_extension *ext)
+{
+	struct sieve_interpreter_extension_reg *reg;
+	int ret;
+
+	i_assert( ext->id >= 0 );
+
+	if ( ext->id >= (int) array_count(&interp->extensions) )
+		return SIEVE_EXEC_OK;
+
+	reg = array_idx_modifiable
+		(&interp->extensions, (unsigned int)ext->id);
+
+	if (!reg->deferred)
+		return SIEVE_EXEC_OK;
+	reg->deferred = FALSE;
+	reg->started = TRUE;
+
+	if ( reg->intext != NULL &&
+		reg->intext->run != NULL &&
+		(ret=reg->intext->run(ext,
+			&interp->runenv, reg->context, TRUE)) <= 0 )
+		return ret;
+	return SIEVE_EXEC_OK;
+}
+
 /*
  * Loop handling
  */
@@ -923,17 +964,25 @@ int sieve_interpreter_continue
 int sieve_interpreter_start
 (struct sieve_interpreter *interp, struct sieve_result *result, bool *interrupted)
 {
-	const struct sieve_interpreter_extension_reg *eregs;
+	struct sieve_interpreter_extension_reg *eregs;
 	unsigned int ext_count, i;
+	int ret;
 
 	interp->runenv.result = result;
 	interp->runenv.msgctx = sieve_result_get_message_context(result);
 
 	/* Signal registered extensions that the interpreter is being run */
-	eregs = array_get(&interp->extensions, &ext_count);
+	eregs = array_get_modifiable(&interp->extensions, &ext_count);
 	for ( i = 0; i < ext_count; i++ ) {
-		if ( eregs[i].intext != NULL && eregs[i].intext->run != NULL )
-			eregs[i].intext->run(eregs[i].ext, &interp->runenv, eregs[i].context);
+		if ( !eregs[i].deferred ) {
+			eregs[i].started = TRUE;
+			if (eregs[i].intext != NULL &&
+				eregs[i].intext->run != NULL &&
+				(ret=eregs[i].intext->run(eregs[i].ext,
+					&interp->runenv, eregs[i].context, FALSE)) <= 0 ) {
+				return ret;
+			}
+		}
 	}
 
 	return sieve_interpreter_continue(interp, interrupted);
