@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -16,6 +16,7 @@
 #include "mail-html2text.h"
 #include "mail-storage.h"
 #include "mail-user.h"
+#include "smtp-params.h"
 #include "master-service.h"
 #include "master-service-settings.h"
 #include "raw-storage.h"
@@ -27,12 +28,12 @@
 #include "sieve-error.h"
 #include "sieve-extensions.h"
 #include "sieve-code.h"
+#include "sieve-address.h"
 #include "sieve-address-parts.h"
 #include "sieve-runtime.h"
 #include "sieve-runtime-trace.h"
 #include "sieve-match.h"
 #include "sieve-interpreter.h"
-#include "sieve-address.h"
 
 #include "sieve-message.h"
 
@@ -73,8 +74,8 @@ struct sieve_message_part {
 	size_t decoded_body_size;
 	size_t text_body_size;
 
-	unsigned int have_body:1; /* there's the empty end-of-headers line */
-	unsigned int epilogue:1;  /* this is a multipart epilogue */
+	bool have_body:1; /* there's the empty end-of-headers line */
+	bool epilogue:1;  /* this is a multipart epilogue */
 };
 
 struct sieve_message_version {
@@ -95,14 +96,6 @@ struct sieve_message_context {
 	struct mail_user *mail_user;
 	const struct sieve_message_data *msgdata;
 
-	/* Normalized envelope addresses */
-
-	bool envelope_parsed;
-
-	const struct sieve_address *envelope_sender;
-	const struct sieve_address *envelope_orig_recipient;
-	const struct sieve_address *envelope_final_recipient;
-
 	/* Message versioning */
 
 	struct mail_user *raw_mail_user;
@@ -118,8 +111,8 @@ struct sieve_message_context {
 	ARRAY(struct sieve_message_part_data) return_body_parts;
 	buffer_t *raw_body;
 
-	unsigned int edit_snapshot:1;
-	unsigned int substitute_snapshot:1;
+	bool edit_snapshot:1;
+	bool substitute_snapshot:1;
 };
 
 /*
@@ -205,11 +198,6 @@ static void sieve_message_context_clear(struct sieve_message_context *msgctx)
 
 		pool_unref(&(msgctx->pool));
 	}
-
-	msgctx->envelope_orig_recipient = NULL;
-	msgctx->envelope_final_recipient = NULL;
-	msgctx->envelope_sender = NULL;
-	msgctx->envelope_parsed = FALSE;
 }
 
 void sieve_message_context_unref(struct sieve_message_context **msgctx)
@@ -297,107 +285,38 @@ const void *sieve_message_context_extension_get
 
 /* Envelope */
 
-static void sieve_message_envelope_parse(struct sieve_message_context *msgctx)
+const struct smtp_address *sieve_message_get_orig_recipient
+(struct sieve_message_context *msgctx)
 {
 	const struct sieve_message_data *msgdata = msgctx->msgdata;
-	struct sieve_instance *svinst = msgctx->svinst;
+	const struct smtp_address *orcpt_to = NULL;
 
-	/* FIXME: log parse problems properly; logs only 'failure' now */
-
-	msgctx->envelope_orig_recipient = sieve_address_parse_envelope_path
-		(msgctx->pool, msgdata->orig_envelope_to);
-
-	if ( msgctx->envelope_orig_recipient == NULL ) {
-		sieve_sys_warning(svinst,
-			"original envelope recipient address '%s' is unparsable",
-			msgdata->orig_envelope_to);
-	} else if ( msgctx->envelope_orig_recipient->local_part == NULL ) {
-		sieve_sys_warning(svinst,
-			"original envelope recipient address '%s' is a null path",
-			msgdata->orig_envelope_to);
+	if ( msgdata->envelope.rcpt_params != NULL ) {
+		orcpt_to = msgdata->envelope.rcpt_params->orcpt.addr;
+		if ( !smtp_address_isnull(orcpt_to) )
+			return orcpt_to;
 	}
 
-	msgctx->envelope_final_recipient = sieve_address_parse_envelope_path
-		(msgctx->pool, msgdata->final_envelope_to);
-
-	if ( msgctx->envelope_final_recipient == NULL ) {
-		if ( msgctx->envelope_orig_recipient != NULL ) {
-			sieve_sys_warning(svinst,
-				"final envelope recipient address '%s' is unparsable",
-				msgdata->final_envelope_to);
-		}
-	} else if ( msgctx->envelope_final_recipient->local_part == NULL ) {
-		if ( strcmp(msgdata->orig_envelope_to, msgdata->final_envelope_to) != 0 ) {
-			sieve_sys_warning(svinst,
-				"final envelope recipient address '%s' is a null path",
-				msgdata->final_envelope_to);
-		}
-	}
-
-	msgctx->envelope_sender = sieve_address_parse_envelope_path
-		(msgctx->pool, msgdata->return_path);
-
-	if ( msgctx->envelope_sender == NULL ) {
-		sieve_sys_warning(svinst,
-			"envelope sender address '%s' is unparsable",
-			msgdata->return_path);
-	}
-
-	msgctx->envelope_parsed = TRUE;
+	orcpt_to = msgdata->envelope.rcpt_to;
+	return ( !smtp_address_isnull(orcpt_to) ? orcpt_to : NULL );
 }
 
-const struct sieve_address *sieve_message_get_orig_recipient_address
+const struct smtp_address *sieve_message_get_final_recipient
 (struct sieve_message_context *msgctx)
 {
-	if ( !msgctx->envelope_parsed )
-		sieve_message_envelope_parse(msgctx);
+	const struct sieve_message_data *msgdata = msgctx->msgdata;
+	const struct smtp_address *rcpt_to = msgdata->envelope.rcpt_to;
 
-	return msgctx->envelope_orig_recipient;
+	return ( !smtp_address_isnull(rcpt_to) ? rcpt_to : NULL);
 }
 
-const struct sieve_address *sieve_message_get_final_recipient_address
+const struct smtp_address *sieve_message_get_sender
 (struct sieve_message_context *msgctx)
 {
-	if ( !msgctx->envelope_parsed )
-		sieve_message_envelope_parse(msgctx);
+	const struct sieve_message_data *msgdata = msgctx->msgdata;
+	const struct smtp_address *mail_from = msgdata->envelope.mail_from;
 
-	return msgctx->envelope_final_recipient;
-}
-
-const struct sieve_address *sieve_message_get_sender_address
-(struct sieve_message_context *msgctx)
-{
-	if ( !msgctx->envelope_parsed )
-		sieve_message_envelope_parse(msgctx);
-
-	return msgctx->envelope_sender;
-}
-
-const char *sieve_message_get_orig_recipient
-(struct sieve_message_context *msgctx)
-{
-	if ( !msgctx->envelope_parsed )
-		sieve_message_envelope_parse(msgctx);
-
-	return sieve_address_to_string(msgctx->envelope_orig_recipient);
-}
-
-const char *sieve_message_get_final_recipient
-(struct sieve_message_context *msgctx)
-{
-	if ( !msgctx->envelope_parsed )
-		sieve_message_envelope_parse(msgctx);
-
-	return sieve_address_to_string(msgctx->envelope_final_recipient);
-}
-
-const char *sieve_message_get_sender
-(struct sieve_message_context *msgctx)
-{
-	if ( !msgctx->envelope_parsed )
-		sieve_message_envelope_parse(msgctx);
-
-	return sieve_address_to_string(msgctx->envelope_sender);
+	return ( !smtp_address_isnull(mail_from) ? mail_from : NULL);
 }
 
 /*
@@ -414,9 +333,11 @@ int sieve_message_substitute
 	struct sieve_message_version *version;
 	struct mailbox_header_lookup_ctx *headers_ctx;
 	struct mailbox *box = NULL;
-	const char *sender;
+	const struct smtp_address *sender;
 	int ret;
 
+	i_assert(input->blocking);
+
 	if ( msgctx->raw_mail_user == NULL ) {
 		void **sets = master_service_settings_get_others(master_service);
 
@@ -428,7 +349,7 @@ int sieve_message_substitute
 	sender = sieve_message_get_sender(msgctx);
 	sender = (sender == NULL ? DEFAULT_ENVELOPE_SENDER : sender );
 	ret = raw_mailbox_alloc_stream(msgctx->raw_mail_user, input, (time_t)-1,
-		sender, &box);
+		smtp_address_encode(sender), &box);
 
 	if ( ret < 0 ) {
 		sieve_sys_error(msgctx->svinst, "can't open substituted mail as raw: %s",
@@ -444,7 +365,7 @@ int sieve_message_substitute
 	}
 
 	version->box = box;
-	version->trans = mailbox_transaction_begin(box, 0);
+	version->trans = mailbox_transaction_begin(box, 0, __func__);
 	headers_ctx = mailbox_header_lookup_init(box, wanted_headers);
 	version->mail = mail_alloc(version->trans, 0, headers_ctx);
 	mailbox_header_lookup_unref(&headers_ctx);
@@ -525,7 +446,7 @@ struct sieve_message_header_list {
 	const char *const *headers;
 	int headers_index;
 
-	unsigned int mime_decode:1;
+	bool mime_decode:1;
 };
 
 struct sieve_header_list *sieve_message_header_list_create
@@ -547,17 +468,22 @@ struct sieve_header_list *sieve_message_header_list_create
 	return &hdrlist->hdrlist;
 }
 
+// NOTE: get rid of this once we have a proper Sieve string type
 static inline string_t *_header_right_trim(const char *raw)
 {
 	string_t *result;
-	int i;
+	const char *p, *pend;
 
-	for ( i = strlen(raw)-1; i >= 0; i-- ) {
-		if ( raw[i] != ' ' && raw[i] != '\t' ) break;
+	pend = raw + strlen(raw);
+	if (raw == pend) {
+		result = t_str_new(1);
+	} else {
+		for ( p = pend-1; p >= raw; p-- ) {
+			if ( *p != ' ' && *p != '\t' ) break;
+		}
+		result = t_str_new(p - raw + 1);
+		str_append_n(result, raw, p - raw + 1);
 	}
-
-	result = t_str_new(i+1);
-	str_append_n(result, raw, i + 1);
 	return result;
 }
 
@@ -618,7 +544,7 @@ static int sieve_message_header_list_next_item
 			return -1;
 		}
 
-		if ( hdrlist->headers == NULL || hdrlist->headers[0] == NULL ) {
+		if ( ret == 0 || hdrlist->headers[0] == NULL ) {
 			/* Try next item when no headers found */
 			hdrlist->headers = NULL;
 		}
@@ -847,6 +773,80 @@ int sieve_message_get_header_fields
 	return SIEVE_EXEC_OK;
 }
 
+/*
+ * Message part
+ */
+
+struct sieve_message_part *sieve_message_part_parent
+(struct sieve_message_part *mpart)
+{
+	return mpart->parent;
+}
+
+struct sieve_message_part *sieve_message_part_next
+(struct sieve_message_part *mpart)
+{
+	return mpart->next;
+}
+
+struct sieve_message_part *sieve_message_part_children
+(struct sieve_message_part *mpart)
+{
+	return mpart->children;
+}
+
+const char *sieve_message_part_content_type
+(struct sieve_message_part *mpart)
+{
+	return mpart->content_type;
+}
+
+const char *sieve_message_part_content_disposition
+(struct sieve_message_part *mpart)
+{
+	return mpart->content_disposition;
+}
+
+int sieve_message_part_get_first_header
+(struct sieve_message_part *mpart, const char *field,
+	const char **value_r)
+{
+	const struct sieve_message_header *headers;
+	unsigned int i, count;
+
+	headers = array_get(&mpart->headers, &count);
+	for ( i = 0; i < count; i++ ) {
+		if ( strcasecmp( headers[i].name, field) == 0 ) {
+			i_assert( headers[i].value[headers[i].value_len] == '\0' );
+			*value_r = (const char *)headers[i].value;
+			return 1;
+		}
+	}
+
+	*value_r = NULL;
+	return 0;
+}
+
+void sieve_message_part_get_data
+(struct sieve_message_part *mpart,
+	struct sieve_message_part_data *data, bool text)
+{
+	i_zero(data);
+	data->content_type = mpart->content_type;
+	data->content_disposition = mpart->content_disposition;
+
+	if ( !text ) {
+		data->content = mpart->decoded_body;
+		data->size = mpart->decoded_body_size;
+	} else if ( mpart->children != NULL ) {
+		data->content = "";
+		data->size = 0;
+	} else {
+		data->content = mpart->text_body;
+		data->size = mpart->text_body_size;
+	}
+}
+
 /*
  * Message body
  */
@@ -969,12 +969,12 @@ static void sieve_message_part_save
 	char *part_data;
 	size_t part_size;
 
-	/* Add terminating NUL to the body part buffer */
-	buffer_append_c(buf, '\0');
+	/* Extract text if requested */
 	result_buf = buf;
+	if ( extract_text && body_part->children == NULL &&
+		!body_part->epilogue ) {
 
-	if ( extract_text ) {
-		if ( mail_html2text_content_type_match
+		if ( buf->used > 0 && mail_html2text_content_type_match
 			(body_part->content_type) ) {
 			struct mail_html2text *html2text;
 
@@ -984,15 +984,20 @@ static void sieve_message_part_save
 			html2text = mail_html2text_init(0);
 			mail_html2text_more(html2text, buf->data, buf->used, text_buf);
 			mail_html2text_deinit(&html2text);
-	
+
 			result_buf = text_buf;
 		}
 	}
 
+	/* Add terminating NUL to the body part buffer */
+	buffer_append_c(result_buf, '\0');
+
+	/* Make copy of the buffer */
 	part_data = p_malloc(pool, result_buf->used);
 	memcpy(part_data, result_buf->data, result_buf->used);
 	part_size = result_buf->used - 1;
 
+	/* Free text buffer if used */
 	if ( text_buf != NULL)
 		buffer_free(&text_buf);
 
@@ -1073,6 +1078,8 @@ static int sieve_message_parts_add_missing
 	struct mail *mail = sieve_message_get_mail(renv->msgctx);
 	enum message_parser_flags mparser_flags =
 		MESSAGE_PARSER_FLAG_INCLUDE_MULTIPART_BLOCKS;
+	enum message_header_parser_flags hparser_flags =
+		MESSAGE_HEADER_PARSER_FLAG_SKIP_INITIAL_LWSP;
 	ARRAY(struct sieve_message_header) headers;
 	struct sieve_message_part *body_part, *header_part, *last_part;
 	struct message_parser_ctx *parser;
@@ -1109,18 +1116,19 @@ static int sieve_message_parts_add_missing
 	if (iter_all) {
 		t_array_init(&headers, 64);
 		hdr_content = t_str_new(512);
+		hparser_flags |= MESSAGE_HEADER_PARSER_FLAG_CLEAN_ONELINE;
 	} else {
-		memset(&headers, 0, sizeof(headers));
+		i_zero(&headers);
 	}
 
 	/* Initialize body decoder */
 	decoder = message_decoder_init(NULL, 0);
 
 	// FIXME: currently not tested with edit-mail.
-		//parser = message_parser_init_from_parts(parts, input, 0,
-		//mparser_flags);
-	parser = message_parser_init
-		(pool_datastack_create(), input, 0, mparser_flags);
+		//parser = message_parser_init_from_parts(parts, input,
+		// hparser_flags, mparser_flags);
+	parser = message_parser_init(pool_datastack_create(),
+		input, hparser_flags, mparser_flags);
 	while ( (ret=message_parser_parse_next_block
 		(parser, &block)) > 0 ) {
 		struct sieve_message_part **body_part_idx;
@@ -1196,20 +1204,21 @@ static int sieve_message_parts_add_missing
 						struct sieve_message_part *child = parent->children;
 						while (child->next != NULL && child != body_part)
 							child = child->next;
-						if (child->next == NULL)
+						if (child != body_part)
 							child->next = body_part;
 					}
 				}
-				last_part = body_part;
 			}
+			last_part = body_part;
 
 			/* If this is message/rfc822 content, retain the enveloping part for
 			 * storing headers as content.
 			 */
 			if ( message_rfc822 ) {
 				i_assert(idx > 0);
-				header_part = *array_idx
+				body_part_idx = array_idx_modifiable
 					(&msgctx->cached_body_parts, idx-1);
+				header_part = *body_part_idx;
 			} else {
 				header_part = NULL;
 			}
@@ -1236,7 +1245,7 @@ static int sieve_message_parts_add_missing
 				/* Save headers for message/rfc822 part */
 				if ( header_part != NULL ) {
 					sieve_message_part_save
-						(renv, buf, header_part, extract_text);
+						(renv, buf, header_part, FALSE);
 					header_part = NULL;
 				}
 
@@ -1270,7 +1279,7 @@ static int sieve_message_parts_add_missing
 
 			if ( strcasecmp(hdr->name, "Content-Type" ) == 0 )
 				hdr_field = _HDR_CONTENT_TYPE;
-			else if ( strcasecmp(hdr->name, "Content-Disposition" ) != 0 )
+			else if ( strcasecmp(hdr->name, "Content-Disposition" ) == 0 )
 				hdr_field = _HDR_CONTENT_DISPOSITION;
 			else if ( iter_all && !array_is_created(&body_part->headers) )
 				hdr_field = _HDR_OTHER;
@@ -1288,29 +1297,37 @@ static int sieve_message_parts_add_missing
 			}
 
 			if ( iter_all && !array_is_created(&body_part->headers) ) {
+				const unsigned char *value, *vp;
+				size_t vlen;
+
 				/* Add header */
 				header = array_append_space(&headers);
 				header->name = p_strdup(pool, hdr->name);
 
-				// FIXME: trim header values
+				/* Trim end of field value (not done by parser) */
+				value = hdr->full_value;
+				vp = value + hdr->full_value_len;
+				while ( vp > value &&
+					(vp[-1] == '\t' || vp[-1] == ' ') )
+					vp--;
+				vlen = (size_t)(vp - value);
 	
 				/* Decode MIME encoded-words. */
 				str_truncate(hdr_content, 0);
 				message_header_decode_utf8
-					(hdr->full_value, hdr->full_value_len, hdr_content, NULL);
-				if ( hdr->full_value_len != str_len(hdr_content) ||
-					strncmp(str_c(hdr_content), (const char *)hdr->full_value,
-						hdr->full_value_len) != 0 ) {
+					(value, vlen, hdr_content, NULL);
+				if ( vlen != str_len(hdr_content) ||
+					strncmp(str_c(hdr_content), (const char *)value,
+						vlen) != 0 ) {
 					if ( strlen(str_c(hdr_content)) != str_len(hdr_content) ) {
 						/* replace NULs with spaces */
 						str_replace_nuls(hdr_content);
 					}
 					/* store raw */
-					data = p_malloc(pool, hdr->full_value_len + 1);
-					data[hdr->full_value_len] = '\0';
-					header->value = memcpy(data,
-						hdr->full_value, hdr->full_value_len);
-					header->value_len = hdr->full_value_len;
+					data = p_malloc(pool, vlen + 1);
+					data[vlen] = '\0';
+					header->value = memcpy(data, value, vlen);
+					header->value_len = vlen;
 					/* store decoded */
 					data = p_malloc(pool, str_len(hdr_content) + 1);
 					data[str_len(hdr_content)] = '\0';
@@ -1319,12 +1336,11 @@ static int sieve_message_parts_add_missing
 					header->utf8_value_len = str_len(hdr_content);
 				} else {
 					/* raw == decoded */
-					data = p_malloc(pool, hdr->full_value_len + 1);
-					data[hdr->full_value_len] = '\0';
+					data = p_malloc(pool, vlen + 1);
+					data[vlen] = '\0';
 					header->value = header->utf8_value =
-						memcpy(data, hdr->full_value, hdr->full_value_len);
-					header->value_len = header->utf8_value_len =
-						hdr->full_value_len;
+						memcpy(data, value, vlen);
+					header->value_len = header->utf8_value_len = vlen;
 				}
 
 				if ( hdr_field == _HDR_OTHER )
@@ -1339,6 +1355,7 @@ static int sieve_message_parts_add_missing
 				case _HDR_CONTENT_TYPE:
 					body_part->content_type =
 						p_strdup(pool, _parse_content_type(block.hdr));
+					break;
 				case _HDR_CONTENT_DISPOSITION:
 					body_part->content_disposition =
 						p_strdup(pool, _parse_content_disposition(block.hdr));
@@ -1390,7 +1407,8 @@ static int sieve_message_parts_add_missing
 	if ( input->stream_errno != 0 ) {
 		sieve_runtime_critical(renv, NULL,
 			"failed to read input message",
-			"failed to read message stream: %s",
+			"read(%s) failed: %s",
+			i_stream_get_name(input),
 			i_stream_get_error(input));
 		return SIEVE_EXEC_TEMP_FAILURE;
 	}
@@ -1485,19 +1503,24 @@ int sieve_message_body_get_raw
 		i_stream_skip(input, hdr_size.physical_size);
 
 		/* Read raw message body */
-		while ( (ret=i_stream_read_data(input, &data, &size, 0)) > 0 ) {
+		while ( (ret=i_stream_read_more(input, &data, &size)) > 0 ) {
 			buffer_append(buf, data, size);
 
 			i_stream_skip(input, size);
 		}
 
-		if ( ret == -1 && input->stream_errno != 0 ) {
+		if ( ret < 0 && input->stream_errno != 0 ) {
 			sieve_runtime_critical(renv, NULL,
 				"failed to read input message",
-				"failed to read raw message stream: %s",
+				"read(%s) failed: %s",
+				i_stream_get_name(input),
 				i_stream_get_error(input));
 			return SIEVE_EXEC_TEMP_FAILURE;
 		}
+
+		/* Add terminating NUL to the body part buffer */
+		buffer_append_c(buf, '\0');
+
 	} else {
 		buf = msgctx->raw_body;
 	}
@@ -1505,14 +1528,16 @@ int sieve_message_body_get_raw
 	/* Clear result array */
 	array_clear(&msgctx->return_body_parts);
 
-	if ( buf->used > 0  ) {
-		/* Add terminating NUL to the body part buffer */
-		buffer_append_c(buf, '\0');
+	if ( buf->used > 1  ) {
+		const char *data = (const char *)buf->data;
+		size_t size = buf->used - 1;
+
+		i_assert( data[size] == '\0' );
 
 		/* Add single item to the result */
 		return_part = array_append_space(&msgctx->return_body_parts);
-		return_part->content = buf->data;
-		return_part->size = buf->used - 1;
+		return_part->content = data;
+		return_part->size = size;
 	}
 
 	/* Return the array of body items */
@@ -1545,9 +1570,10 @@ int sieve_message_part_iter_init
 	if ( status <= 0 )
 		return status;
 
-	memset(iter, 0, sizeof(*iter));
+	i_zero(iter);
 	iter->renv = renv;
 	iter->index = 0;
+	iter->offset = 0;
 
 	parts = array_get(&msgctx->cached_body_parts, &count);
 	if (count == 0)
@@ -1558,6 +1584,24 @@ int sieve_message_part_iter_init
 	return SIEVE_EXEC_OK;
 }
 
+void sieve_message_part_iter_subtree(struct sieve_message_part_iter *iter,
+	struct sieve_message_part_iter *subtree)
+{
+	const struct sieve_runtime_env *renv = iter->renv;
+	struct sieve_message_context *msgctx = renv->msgctx;
+	struct sieve_message_part *const *parts;
+	unsigned int count;
+
+	*subtree = *iter;
+
+	parts = array_get(&msgctx->cached_body_parts, &count);
+	if ( subtree->index >= count)
+		subtree->root = NULL;
+	else
+		subtree->root = parts[subtree->index];
+	subtree->offset = subtree->index;
+}
+
 void sieve_message_part_iter_children(struct sieve_message_part_iter *iter,
 	struct sieve_message_part_iter *child)
 {
@@ -1569,10 +1613,11 @@ void sieve_message_part_iter_children(struct sieve_message_part_iter *iter,
 	*child = *iter;
 
 	parts = array_get(&msgctx->cached_body_parts, &count);	
-	if ( child->index >= count || parts[child->index]->children == NULL)
+	if ( (child->index+1) >= count || parts[child->index]->children == NULL)
 		child->root = NULL;
-	else	
+	else
 		child->root = parts[child->index++];
+	child->offset = child->index;
 }
 
 struct sieve_message_part *sieve_message_part_iter_current
@@ -1590,10 +1635,10 @@ struct sieve_message_part *sieve_message_part_iter_current
 	if ( iter->index >= count )
 		return NULL;
 	do {
-		if ( parts[iter->index] == iter->root->next ||
-			parts[iter->index] == iter->root->parent ) {
+		if ( parts[iter->index] == iter->root->next )
+			return NULL;
+		if ( parts[iter->index] == iter->root->parent )
 			return NULL;
-		}
 	} while ( parts[iter->index]->epilogue && ++iter->index < count );
 	if ( iter->index >= count )
 		return NULL;
@@ -1613,6 +1658,12 @@ struct sieve_message_part *sieve_message_part_iter_next
 	return sieve_message_part_iter_current(iter);
 }
 
+void sieve_message_part_iter_reset
+(struct sieve_message_part_iter *iter)
+{
+	iter->index = iter->offset;
+}
+
 /*
  * MIME header list
  */
@@ -1634,14 +1685,14 @@ struct sieve_mime_header_list {
 
 	struct sieve_stringlist *field_names;
 
-	struct sieve_message_part_iter *part_iter, child_iter;
+	struct sieve_message_part_iter part_iter;
 
 	const char *header_name;
 	const struct sieve_message_header *headers;
 	unsigned int headers_index, headers_count;
 
-	unsigned int mime_decode:1;
-	unsigned int children;
+	bool mime_decode:1;
+	bool children:1;
 };
 
 struct sieve_header_list *sieve_mime_header_list_create
@@ -1659,10 +1710,11 @@ struct sieve_header_list *sieve_mime_header_list_create
 	hdrlist->hdrlist.strlist.reset = sieve_mime_header_list_reset;
 	hdrlist->hdrlist.next_item = sieve_mime_header_list_next_item;
 	hdrlist->field_names = field_names;
-	hdrlist->part_iter = part_iter;
 	hdrlist->mime_decode = mime_decode;
 	hdrlist->children = children;
 
+	sieve_message_part_iter_subtree(part_iter, &hdrlist->part_iter);
+
 	return &hdrlist->hdrlist;
 }
 
@@ -1673,12 +1725,8 @@ static void sieve_mime_header_list_next_name
 {
 	struct sieve_message_part *mpart;
 
-	if ( hdrlist->children ) {
-		sieve_message_part_iter_children
-			(hdrlist->part_iter, &hdrlist->child_iter);
-	}
-
-	mpart = sieve_message_part_iter_current(hdrlist->part_iter);
+	sieve_message_part_iter_reset(&hdrlist->part_iter);
+	mpart = sieve_message_part_iter_current(&hdrlist->part_iter);
 
 	if ( mpart != NULL && array_is_created(&mpart->headers) ) {
 		hdrlist->headers = array_get
@@ -1716,7 +1764,7 @@ static int sieve_mime_header_list_next_item
 			if ( hdrlist->header_name != NULL && hdrlist->children ) {
 				struct sieve_message_part *mpart;
 
-				mpart = sieve_message_part_iter_next(&hdrlist->child_iter);
+				mpart = sieve_message_part_iter_next(&hdrlist->part_iter);
 				if ( mpart != NULL && array_is_created(&mpart->headers) ) {
 					hdrlist->headers = array_get
 						(&mpart->headers, &hdrlist->headers_count);
