@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2016 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -11,7 +11,6 @@
 #include "var-expand.h"
 #include "message-address.h"
 #include "mail-storage.h"
-#include "mail-deliver.h"
 
 #include "sieve-common.h"
 #include "sieve-limits.h"
@@ -87,8 +86,8 @@ struct sieve_result {
 	HASH_TABLE(const struct sieve_action_def *,
 			   struct sieve_result_action_context *) action_contexts;
 
-	unsigned int executed:1;
-	unsigned int executed_delivery:1;
+	bool executed:1;
+	bool executed_delivery:1;
 };
 
 struct sieve_result *sieve_result_create
@@ -729,7 +728,7 @@ void sieve_result_vprintf
 
 	str_vprintfa(outbuf, fmt, args);
 
-	o_stream_send(penv->stream, str_data(outbuf), str_len(outbuf));
+	o_stream_nsend(penv->stream, str_data(outbuf), str_len(outbuf));
 }
 
 void sieve_result_printf
@@ -754,7 +753,7 @@ void sieve_result_action_printf
 	str_append_c(outbuf, '\n');
 	va_end(args);
 
-	o_stream_send(penv->stream, str_data(outbuf), str_len(outbuf));
+	o_stream_nsend(penv->stream, str_data(outbuf), str_len(outbuf));
 }
 
 void sieve_result_seffect_printf
@@ -769,7 +768,7 @@ void sieve_result_seffect_printf
 	str_append_c(outbuf, '\n');
 	va_end(args);
 
-	o_stream_send(penv->stream, str_data(outbuf), str_len(outbuf));
+	o_stream_nsend(penv->stream, str_data(outbuf), str_len(outbuf));
 }
 
 static void sieve_result_print_side_effects
@@ -928,7 +927,7 @@ static int _sieve_result_implicit_keep
 (struct sieve_result *result, bool rollback)
 {
 	const struct sieve_action_exec_env *aenv = &result->action_env;
-	struct sieve_result_action *rac;
+	struct sieve_result_action *rac, *kac;
 	int status = SIEVE_EXEC_OK;
 	struct sieve_result_side_effect *rsef, *rsef_first = NULL;
 	void *tr_context = NULL;
@@ -937,38 +936,56 @@ static int _sieve_result_implicit_keep
 	if ( (aenv->flags & SIEVE_EXECUTE_FLAG_DEFER_KEEP) != 0 )
 		return SIEVE_EXEC_OK;
 
-	if ( rollback ) {
+	if ( rollback )
 		act_keep = result->failure_action;
-		act_keep.mail = NULL;
-	} else {
+	else
 		act_keep = result->keep_action;
-		act_keep.mail = sieve_message_get_mail(aenv->msgctx);
-	}
+	act_keep.mail = NULL;
 
 	/* If keep is a non-action, return right away */
 	if ( act_keep.def == NULL )
 		return SIEVE_EXEC_OK;
 
-	/* Scan for execution of keep-equal actions */
-	rac = result->first_action;
-	while ( rac != NULL ) {
-		if ( rac->action.def == act_keep.def && act_keep.def->equals != NULL &&
-			act_keep.def->equals(aenv->scriptenv, NULL, &rac->action) &&
-			rac->action.executed )
-			return SIEVE_EXEC_OK;
-
-		rac = rac->next;
+	/* Scan for deferred keep */
+	kac = result->last_action;
+	while ( kac != NULL && kac->action.executed ) {
+		if ( kac->keep && kac->action.def == NULL )
+			break;
+		kac = kac->prev;
 	}
 
-	/* Apply any implicit side effects if applicable */
-	if ( !rollback && hash_table_is_created(result->action_contexts) ) {
-		struct sieve_result_action_context *actctx;
+	if (kac == NULL) {
+		if ( !rollback )
+			act_keep.mail = sieve_message_get_mail(aenv->msgctx);
 
-		/* Check for implicit side effects to keep action */
-		actctx = hash_table_lookup(result->action_contexts, act_keep.def);
+		/* Scan for execution of keep-equal actions */
+		rac = result->first_action;
+		while ( rac != NULL ) {
+			if ( rac->action.def == act_keep.def && act_keep.def->equals != NULL &&
+				act_keep.def->equals(aenv->scriptenv, NULL, &rac->action) &&
+				rac->action.executed )
+				return SIEVE_EXEC_OK;
 
-		if ( actctx != NULL && actctx->seffects != NULL )
-			rsef_first = actctx->seffects->first_effect;
+			rac = rac->next;
+		}
+	} else if ( !rollback ) {
+		act_keep.location = kac->action.location;
+		act_keep.mail = kac->action.mail;
+		if ( kac->seffects != NULL )
+			rsef_first = kac->seffects->first_effect;
+	}
+
+	if (rsef_first == NULL) {
+		/* Apply any implicit side effects if applicable */
+		if ( !rollback && hash_table_is_created(result->action_contexts) ) {
+			struct sieve_result_action_context *actctx;
+
+			/* Check for implicit side effects to keep action */
+			actctx = hash_table_lookup(result->action_contexts, act_keep.def);
+
+			if ( actctx != NULL && actctx->seffects != NULL )
+				rsef_first = actctx->seffects->first_effect;
+		}
 	}
 
 	/* Start keep action */
@@ -1038,7 +1055,7 @@ static int _sieve_result_implicit_keep
 	/* Finish keep action */
 	if ( act_keep.def->finish != NULL ) {
 		act_keep.def->finish
-			(&act_keep, aenv, &rac->tr_context, status);
+			(&act_keep, aenv, tr_context, status);
 	}
 
 	return status;
@@ -1350,7 +1367,7 @@ static void sieve_result_transaction_finish
 
 		if ( act->def->finish != NULL ) {
 			act->def->finish
-				(act, &result->action_env, &rac->tr_context, status);
+				(act, &result->action_env, rac->tr_context, status);
 		}
 
 		rac = rac->next;
@@ -1416,6 +1433,7 @@ int sieve_result_execute
 					result_status = ret;
 					break;
 				}
+				/* fall through */
 			default:
 				result_status = SIEVE_EXEC_KEEP_FAILED;
 			}
