@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2016 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -72,7 +72,8 @@ struct sieve_validator_extension_reg {
 	struct sieve_ast_argument *arg;
 	void *context;
 
-	bool loaded;
+	bool loaded:1;
+	bool required:1;
 };
 
 /*
@@ -550,50 +551,107 @@ static struct sieve_tag_registration *sieve_validator_command_tag_get
  * Extension support
  */
 
+static bool sieve_validator_extensions_check_conficts
+(struct sieve_validator *valdtr,
+	struct sieve_ast_argument *ext_arg,
+	const struct sieve_extension *ext)
+{
+	struct sieve_validator_extension_reg *ext_reg;
+	struct sieve_validator_extension_reg *regs;
+	unsigned int count, i;
+
+	if ( ext->id < 0 )
+		return TRUE;
+
+	ext_reg = array_idx_modifiable
+		(&valdtr->extensions, (unsigned int) ext->id);
+
+	regs = array_get_modifiable(&valdtr->extensions, &count);
+	for ( i = 0; i < count; i++ ) {
+		bool required = ext_reg->required && regs[i].required;
+
+		if (regs[i].ext == NULL)
+			continue;
+		if (regs[i].ext == ext)
+			continue;
+		if (!regs[i].loaded)
+			continue;		
+
+		/* Check this extension vs other extension */
+		if ( ext_reg->valext != NULL &&
+			ext_reg->valext->check_conflict != NULL ) {
+			struct sieve_ast_argument *this_ext_arg =
+				(ext_arg == NULL ? regs[i].arg : ext_arg);
+
+			if ( !ext_reg->valext->check_conflict(ext,
+				valdtr, ext_reg->context, this_ext_arg,
+				regs[i].ext, required) )
+				return FALSE;
+		}
+
+		/* Check other extension vs this extension */
+		if ( regs[i].valext != NULL &&
+			regs[i].valext->check_conflict != NULL ) {
+			if ( !regs[i].valext->check_conflict(regs[i].ext,
+				valdtr, regs[i].context, regs[i].arg, ext,
+				required) )
+				return FALSE;
+		}
+	}
+	return TRUE;
+}
+
 bool sieve_validator_extension_load
 (struct sieve_validator *valdtr, struct sieve_command *cmd,
-	struct sieve_ast_argument *ext_arg, const struct sieve_extension *ext)
+	struct sieve_ast_argument *ext_arg,
+	const struct sieve_extension *ext,
+	bool required)
 {
 	const struct sieve_extension_def *extdef = ext->def;
-	struct sieve_validator_extension_reg *reg;
+	struct sieve_validator_extension_reg *reg = NULL;
 
 	if ( ext->global && (valdtr->flags & SIEVE_COMPILE_FLAG_NOGLOBAL) != 0 ) {
-		if ( cmd != NULL && ext_arg != NULL ) {
-			sieve_argument_validate_error(valdtr, ext_arg,
-				"%s %s: failed to load Sieve capability `%s': "
-				"its use is restricted to global scripts",
-				sieve_command_identifier(cmd), sieve_command_type_name(cmd),
-				sieve_extension_name(ext));
-		}
+		const char *cmd_prefix = (cmd == NULL ? "" :
+			t_strdup_printf("%s %s: ", sieve_command_identifier(cmd),
+				sieve_command_type_name(cmd)));
+		sieve_argument_validate_error(valdtr, ext_arg,
+			"%sfailed to load Sieve capability `%s': "
+			"its use is restricted to global scripts",
+			cmd_prefix, sieve_extension_name(ext));
 		return FALSE;
 	}
 
-	if ( !sieve_ast_extension_link(valdtr->ast, ext) ) {
-		/*if ( cmd != NULL && ext_arg != NULL ) {
-			sieve_argument_validate_warning(valdtr, ext_arg,
-				"%s %s: sieve capability `%s' already loaded",
-				sieve_command_identifier(cmd), sieve_command_type_name(cmd),
-				sieve_extension_name(ext));
-		}*/
-	} else {
-		if ( extdef->validator_load != NULL &&
-			!extdef->validator_load(ext, valdtr) ) {
-			if ( cmd != NULL && ext_arg != NULL ) {
-				sieve_argument_validate_error(valdtr, ext_arg,
-					"%s %s: failed to load Sieve capability `%s'",
-					sieve_command_identifier(cmd), sieve_command_type_name(cmd),
-					sieve_extension_name(ext));
-			}
-			return FALSE;
-		}
-	}
-
-	/* Register extension no matter what and store the AST argument registering it
-	 */
+	/* Register extension no matter what and store the
+	 * AST argument registering it */
 	if ( ext->id >= 0 ) {
-		reg = array_idx_modifiable(&valdtr->extensions, (unsigned int) ext->id);
+		reg = array_idx_modifiable
+			(&valdtr->extensions, (unsigned int) ext->id);
+		i_assert(reg->ext == NULL || reg->ext == ext);
+		reg->ext = ext;
+		reg->required = reg->required || required;
 		if ( reg->arg == NULL )
 			reg->arg = ext_arg;
+	}
+
+	if ( extdef->validator_load != NULL &&
+		!extdef->validator_load(ext, valdtr) ) {
+		const char *cmd_prefix = (cmd == NULL ? "" :
+			t_strdup_printf("%s %s: ", sieve_command_identifier(cmd),
+				sieve_command_type_name(cmd)));
+		sieve_argument_validate_error(valdtr, ext_arg,
+			"%sfailed to load Sieve capability `%s'",
+			cmd_prefix, sieve_extension_name(ext));
+		return FALSE;
+	}
+
+	/* Check conflicts with other extensions */
+	if ( !sieve_validator_extensions_check_conficts
+		(valdtr, ext_arg, ext) )
+		return FALSE;
+
+	/* Link extension to AST for use at code generation */
+	if ( reg != NULL ) {
+		sieve_ast_extension_link(valdtr->ast, ext, reg->required);
 		reg->loaded = TRUE;
 	}
 
@@ -638,7 +696,8 @@ const struct sieve_extension *sieve_validator_extension_load_by_name
 		return NULL;
 	}
 
-	if ( !sieve_validator_extension_load(valdtr, cmd, ext_arg, ext) )
+	if ( !sieve_validator_extension_load
+		(valdtr, cmd, ext_arg, ext, TRUE) )
 		return NULL;
 
 	return ext;
@@ -654,7 +713,8 @@ const struct sieve_extension *sieve_validator_extension_load_implicit
 	if ( ext == NULL || ext->def == NULL )
 		return NULL;
 
-	if ( !sieve_validator_extension_load(valdtr, NULL, NULL, ext) )
+	if ( !sieve_validator_extension_load
+		(valdtr, NULL, NULL, ext, TRUE) )
 		return NULL;
 
 	return ext;
@@ -669,8 +729,9 @@ void sieve_validator_extension_register
 	if ( ext->id < 0 ) return;
 
 	reg = array_idx_modifiable(&valdtr->extensions, (unsigned int) ext->id);
-	reg->valext = valext;
+	i_assert(reg->ext == NULL || reg->ext == ext);
 	reg->ext = ext;
+	reg->valext = valext;
 	reg->context = context;
 }
 
@@ -854,6 +915,8 @@ bool sieve_validate_tag_parameter
 	const char *arg_name, unsigned int arg_pos,
 	enum sieve_ast_argument_type req_type, bool constant)
 {
+	i_assert(tag != NULL);
+
 	if ( param == NULL ) {
 		const char *position = ( arg_pos == 0 ? "" :
 			t_strdup_printf(" %d (%s)", arg_pos, arg_name) );
@@ -1391,18 +1454,19 @@ static bool sieve_validate_block
 				for (i = 0; i < ext_count; i++) {
 					if ( exts[i]->implicit ) {
 						(void)sieve_validator_extension_load
-							(valdtr, NULL, NULL, exts[i]);
+							(valdtr, NULL, NULL, exts[i], TRUE);
 					}
 				}
 
 				/* Validate all 'require'd extensions */
 				extrs = array_get(&valdtr->extensions, &ext_count);
 				for ( i = 0; i < ext_count; i++ ) {
-					if ( extrs[i].valext != NULL
+					if ( extrs[i].loaded && extrs[i].valext != NULL
 						&& extrs[i].valext->validate != NULL ) {
 
-						if ( !extrs[i].valext->validate
-							(extrs[i].ext, valdtr, extrs[i].context, extrs[i].arg) ) {
+						if ( !extrs[i].valext->validate(extrs[i].ext,
+							valdtr, extrs[i].context, extrs[i].arg,
+							extrs[i].required) ) {
 							fatal = TRUE;
 							break;
 						}
