@@ -1,17 +1,23 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
 #include "str.h"
 #include "istream.h"
+#include "ostream.h"
 #include "buffer.h"
+#include "time-util.h"
 #include "eacces-error.h"
 #include "home-expand.h"
+#include "hostpid.h"
+#include "message-address.h"
+#include "mail-user.h"
 
 #include "sieve-settings.h"
 #include "sieve-extensions.h"
 #include "sieve-plugins.h"
 
+#include "sieve-address.h"
 #include "sieve-script.h"
 #include "sieve-storage-private.h"
 #include "sieve-ast.h"
@@ -32,7 +38,6 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
-#include <stdlib.h>
 #include <unistd.h>
 #include <stdio.h>
 #include <dirent.h>
@@ -133,7 +138,7 @@ void sieve_deinit(struct sieve_instance **_svinst)
 void sieve_set_extensions
 (struct sieve_instance *svinst, const char *extensions)
 {
-	sieve_extensions_set_string(svinst, extensions, FALSE);
+	sieve_extensions_set_string(svinst, extensions, FALSE, FALSE);
 }
 
 const char *sieve_get_capabilities
@@ -232,47 +237,49 @@ struct sieve_binary *sieve_compile_script
 {
 	struct sieve_ast *ast;
 	struct sieve_binary *sbin;
-	enum sieve_error error;
+	enum sieve_error error, *errorp;
 
 	if ( error_r != NULL )
-		*error_r = SIEVE_ERROR_NONE;
+		errorp = error_r;
 	else
-		error_r = &error;
+		errorp = &error;
+	*errorp = SIEVE_ERROR_NONE;
 
 	/* Parse */
-	if ( (ast = sieve_parse(script, ehandler, error_r)) == NULL ) {
-		switch ( *error_r ) {
+	if ( (ast = sieve_parse(script, ehandler, errorp)) == NULL ) {
+		switch ( *errorp ) {
 		case SIEVE_ERROR_NOT_FOUND:
-			sieve_error(ehandler, sieve_script_name(script), "script not found");
+			if (error_r == NULL) {
+				sieve_error(ehandler, sieve_script_name(script),
+					"script not found");
+			}
 			break;
 		default:
-	 		sieve_error(ehandler, sieve_script_name(script), "parse failed");
+			sieve_error(ehandler, sieve_script_name(script),
+				"parse failed");
 		}
 		return NULL;
 	}
 
 	/* Validate */
-	if ( !sieve_validate(ast, ehandler, flags, error_r) ) {
-		sieve_error(ehandler, sieve_script_name(script), "validation failed");
+	if ( !sieve_validate(ast, ehandler, flags, errorp) ) {
+		sieve_error(ehandler, sieve_script_name(script),
+			"validation failed");
 
  		sieve_ast_unref(&ast);
  		return NULL;
  	}
 
 	/* Generate */
-	if ( (sbin=sieve_generate(ast, ehandler, flags, error_r)) == NULL ) {
-		sieve_error(ehandler, sieve_script_name(script), "code generation failed");
-
+	if ( (sbin=sieve_generate(ast, ehandler, flags, errorp)) == NULL ) {
+		sieve_error(ehandler, sieve_script_name(script),
+			"code generation failed");
 		sieve_ast_unref(&ast);
 		return NULL;
 	}
 
 	/* Cleanup */
 	sieve_ast_unref(&ast);
-
-	if ( error_r != NULL )
-		*error_r = SIEVE_ERROR_NONE;
-
 	return sbin;
 }
 
@@ -318,26 +325,24 @@ struct sieve_binary *sieve_compile
 static int sieve_run
 (struct sieve_binary *sbin, struct sieve_result **result,
 	const struct sieve_message_data *msgdata, const struct sieve_script_env *senv,
-	struct sieve_error_handler *ehandler, enum sieve_runtime_flags flags)
+	struct sieve_error_handler *ehandler, enum sieve_execute_flags flags)
 {
 	struct sieve_interpreter *interp;
 	int ret = 0;
 
 	/* Create the interpreter */
-	if ( (interp=sieve_interpreter_create(sbin, msgdata, senv, ehandler, flags))
-		== NULL )
+	if ( (interp=sieve_interpreter_create
+		(sbin, NULL, msgdata, senv, ehandler, flags)) == NULL )
 		return SIEVE_EXEC_BIN_CORRUPT;
 
 	/* Reset execution status */
 	if ( senv->exec_status != NULL )
-		memset(senv->exec_status, 0, sizeof(*senv->exec_status));
+		i_zero(senv->exec_status);
 
 	/* Create result object */
-	if ( *result == NULL )
+	if ( *result == NULL ) {
 		*result = sieve_result_create
-			(sieve_binary_svinst(sbin), msgdata, senv, ehandler);
-	else {
-		sieve_result_set_error_handler(*result, ehandler);
+			(sieve_binary_svinst(sbin), msgdata, senv);
 	}
 
 	/* Run the interpreter */
@@ -509,7 +514,7 @@ void sieve_hexdump
 int sieve_test
 (struct sieve_binary *sbin, const struct sieve_message_data *msgdata,
 	const struct sieve_script_env *senv, struct sieve_error_handler *ehandler,
-	struct ostream *stream, enum sieve_runtime_flags flags, bool *keep)
+	struct ostream *stream, enum sieve_execute_flags flags, bool *keep)
 {
 	struct sieve_result *result = NULL;
 	int ret;
@@ -521,7 +526,8 @@ int sieve_test
 
 	/* Print result if successful */
 	if ( ret > 0 ) {
-		ret = sieve_result_print(result, senv, stream, keep);
+		ret = ( sieve_result_print(result, senv, stream, keep) ?
+			SIEVE_EXEC_OK : SIEVE_EXEC_FAILURE );
 	} else if ( ret == 0 ) {
 		if ( keep != NULL ) *keep = TRUE;
 	}
@@ -537,10 +543,33 @@ int sieve_test
  * Script execution
  */
 
+int sieve_script_env_init(struct sieve_script_env *senv,
+	struct mail_user *user, const char **error_r)
+{
+	const struct mail_storage_settings *mail_set =
+		mail_user_set_get_storage_set(user);
+	const struct message_address *postmaster;
+	const char *error;
+
+	if ( !mail_storage_get_postmaster_address(mail_set,
+		&postmaster, &error) ) {
+		*error_r = t_strdup_printf(
+			"Invalid postmaster_address: %s", error);
+		return -1;
+	}
+
+	i_zero(senv);
+	senv->user = user;
+	senv->postmaster_address = postmaster;
+	return 0;
+}
+
 int sieve_execute
 (struct sieve_binary *sbin, const struct sieve_message_data *msgdata,
-	const struct sieve_script_env *senv, struct sieve_error_handler *ehandler,
-	enum sieve_runtime_flags flags, bool *keep)
+	const struct sieve_script_env *senv,
+	struct sieve_error_handler *exec_ehandler,
+	struct sieve_error_handler *action_ehandler,
+	enum sieve_execute_flags flags, bool *keep)
 {
 	struct sieve_result *result = NULL;
 	int ret;
@@ -548,7 +577,7 @@ int sieve_execute
 	if ( keep != NULL ) *keep = FALSE;
 
 	/* Run the script */
-	ret = sieve_run(sbin, &result, msgdata, senv, ehandler, flags);
+	ret = sieve_run(sbin, &result, msgdata, senv, exec_ehandler, flags);
 
 	/* Evaluate status and execute the result:
 	 *   Strange situations, e.g. currupt binaries, must be handled by the caller.
@@ -557,10 +586,11 @@ int sieve_execute
 	 */
 	if ( ret > 0 ) {
 		/* Execute result */
-		ret = sieve_result_execute(result, keep);
+		ret = sieve_result_execute(result, keep, action_ehandler, flags);
 	} else if ( ret == SIEVE_EXEC_FAILURE ) {
 		/* Perform implicit keep if script failed with a normal runtime error */
-		switch ( sieve_result_implicit_keep(result) ) {
+		switch ( sieve_result_implicit_keep
+			(result, action_ehandler, flags, FALSE) ) {
 		case SIEVE_EXEC_OK:
 			if ( keep != NULL ) *keep = TRUE;
 			break;
@@ -590,10 +620,12 @@ struct sieve_multiscript {
 	const struct sieve_script_env *scriptenv;
 
 	int status;
-	bool active;
 	bool keep;
 
 	struct ostream *teststream;
+
+	bool active:1;
+	bool discard_handled:1;
 };
 
 struct sieve_multiscript *sieve_multiscript_start_execute
@@ -604,7 +636,7 @@ struct sieve_multiscript *sieve_multiscript_start_execute
 	struct sieve_result *result;
 	struct sieve_multiscript *mscript;
 
-	result = sieve_result_create(svinst, msgdata, senv, NULL);
+	result = sieve_result_create(svinst, msgdata, senv);
 	pool = sieve_result_pool(result);
 
 	sieve_result_set_keep_action(result, NULL, NULL);
@@ -634,14 +666,12 @@ struct sieve_multiscript *sieve_multiscript_start_test
 }
 
 static void sieve_multiscript_test
-(struct sieve_multiscript *mscript, struct sieve_error_handler *ehandler,
-	bool *keep)
+(struct sieve_multiscript *mscript, bool *keep)
 {
-	sieve_result_set_error_handler(mscript->result, ehandler);
-
 	if ( mscript->status > 0 ) {
-		mscript->status = sieve_result_print
-			(mscript->result, mscript->scriptenv, mscript->teststream, keep);
+		mscript->status = ( sieve_result_print(mscript->result,
+			mscript->scriptenv, mscript->teststream, keep) ?
+				SIEVE_EXEC_OK : SIEVE_EXEC_FAILURE );
 	} else {
 		if ( keep != NULL ) *keep = TRUE;
 	}
@@ -650,15 +680,16 @@ static void sieve_multiscript_test
 }
 
 static void sieve_multiscript_execute
-(struct sieve_multiscript *mscript, struct sieve_error_handler *ehandler,
-	bool *keep)
+(struct sieve_multiscript *mscript,
+	struct sieve_error_handler *ehandler,
+	enum sieve_execute_flags flags, bool *keep)
 {
-	sieve_result_set_error_handler(mscript->result, ehandler);
-
 	if ( mscript->status > 0 ) {
-		mscript->status = sieve_result_execute(mscript->result, keep);
+		mscript->status = sieve_result_execute
+			(mscript->result, keep, ehandler, flags);
 	} else {
-		if ( !sieve_result_implicit_keep(mscript->result) )
+		if ( sieve_result_implicit_keep
+			(mscript->result, ehandler, flags, FALSE) <= 0 )
 			mscript->status = SIEVE_EXEC_KEEP_FAILED;
 		else
 			if ( keep != NULL ) *keep = TRUE;
@@ -667,28 +698,27 @@ static void sieve_multiscript_execute
 
 bool sieve_multiscript_run
 (struct sieve_multiscript *mscript, struct sieve_binary *sbin,
-	struct sieve_error_handler *ehandler, enum sieve_runtime_flags flags,
-	bool final)
+	struct sieve_error_handler *exec_ehandler,
+	struct sieve_error_handler *action_ehandler,
+	enum sieve_execute_flags flags)
 {
 	if ( !mscript->active ) return FALSE;
 
-	if ( final )
-		sieve_result_set_keep_action(mscript->result, NULL, &act_store);
-
 	/* Run the script */
 	mscript->status = sieve_run(sbin, &mscript->result, mscript->msgdata,
-		mscript->scriptenv, ehandler, flags);
+		mscript->scriptenv, exec_ehandler, flags);
 
 	if ( mscript->status >= 0 ) {
 		mscript->keep = FALSE;
 
-		if ( mscript->teststream != NULL )
-			sieve_multiscript_test(mscript, ehandler, &mscript->keep);
-		else
-			sieve_multiscript_execute(mscript, ehandler, &mscript->keep);
-
+		if ( mscript->teststream != NULL ) {
+			sieve_multiscript_test(mscript, &mscript->keep);
+		} else {
+			sieve_multiscript_execute(mscript,
+				action_ehandler, flags, &mscript->keep);
+		}
 		mscript->active =
-			( mscript->active && mscript->keep && !final && mscript->status > 0 );
+			( mscript->active && mscript->keep && mscript->status > 0 );
 	}
 
 	if ( mscript->status <= 0 )
@@ -697,28 +727,70 @@ bool sieve_multiscript_run
 	return mscript->active;
 }
 
+bool sieve_multiscript_will_discard
+(struct sieve_multiscript *mscript)
+{
+	return ( !mscript->active &&
+		!sieve_result_executed_delivery(mscript->result) );
+}
+
+void sieve_multiscript_run_discard
+(struct sieve_multiscript *mscript, struct sieve_binary *sbin,
+	struct sieve_error_handler *exec_ehandler,
+	struct sieve_error_handler *action_ehandler,
+	enum sieve_execute_flags flags)
+{
+	if ( !sieve_multiscript_will_discard(mscript) )
+		return;
+	i_assert( !mscript->discard_handled );
+
+	sieve_result_set_keep_action
+		(mscript->result, NULL, &act_store);
+
+	/* Run the discard script */
+	flags |= SIEVE_EXECUTE_FLAG_DEFER_KEEP;
+	mscript->status = sieve_run(sbin, &mscript->result, mscript->msgdata,
+		mscript->scriptenv, exec_ehandler, flags);
+
+	if ( mscript->status >= 0 ) {
+		mscript->keep = FALSE;
+
+		if ( mscript->teststream != NULL ) {
+			sieve_multiscript_test(mscript, &mscript->keep);
+		} else {
+			sieve_multiscript_execute(mscript,
+				action_ehandler, flags, &mscript->keep);
+		}
+	}
+
+	mscript->discard_handled = TRUE;
+}
+
 int sieve_multiscript_status(struct sieve_multiscript *mscript)
 {
 	return mscript->status;
 }
 
-int sieve_multiscript_tempfail(struct sieve_multiscript **mscript,
-	struct sieve_error_handler *ehandler)
+int sieve_multiscript_tempfail(struct sieve_multiscript **_mscript,
+	struct sieve_error_handler *action_ehandler,
+	enum sieve_execute_flags flags)
 {
-	struct sieve_result *result = (*mscript)->result;
-	int ret = (*mscript)->status;
+	struct sieve_multiscript *mscript = *_mscript;
+	struct sieve_result *result = mscript->result;
+	int ret = mscript->status;
 
-	if ( ehandler != NULL )
-		sieve_result_set_error_handler(result, ehandler);
+	sieve_result_set_keep_action
+		(mscript->result, NULL, &act_store);
 
-	if ( (*mscript)->active ) {
+	if ( mscript->active ) {
 		ret = SIEVE_EXEC_TEMP_FAILURE;
 
-		if ( !(*mscript)->teststream && sieve_result_executed(result) ) {
+		if ( mscript->teststream == NULL && sieve_result_executed(result) ) {
 			/* Part of the result is already executed, need to fall back to
 			 * to implicit keep (FIXME)
 			 */
-			switch ( sieve_result_implicit_keep(result) ) {
+			switch ( sieve_result_implicit_keep
+				(result, action_ehandler, flags, FALSE) ) {
 			case SIEVE_EXEC_OK:
 				ret = SIEVE_EXEC_FAILURE;
 				break;
@@ -730,29 +802,30 @@ int sieve_multiscript_tempfail(struct sieve_multiscript **mscript,
 
 	/* Cleanup */
 	sieve_result_unref(&result);
-	*mscript = NULL;
+	*_mscript = NULL;
 
 	return ret;
 }
 
-int sieve_multiscript_finish(struct sieve_multiscript **mscript,
-	struct sieve_error_handler *ehandler, bool *keep)
+int sieve_multiscript_finish(struct sieve_multiscript **_mscript,
+	struct sieve_error_handler *action_ehandler,
+	enum sieve_execute_flags flags, bool *keep)
 {
-	struct sieve_result *result = (*mscript)->result;
-	int ret = (*mscript)->status;
+	struct sieve_multiscript *mscript = *_mscript;
+	struct sieve_result *result = mscript->result;
+	int ret = mscript->status;
 
-	if ( ehandler != NULL )
-		sieve_result_set_error_handler(result, ehandler);
+	sieve_result_set_keep_action
+		(mscript->result, NULL, &act_store);
 
-	if ( (*mscript)->active ) {
-		ret = SIEVE_EXEC_FAILURE;
-
-		if ( (*mscript)->teststream ) {
-			(*mscript)->keep = TRUE;
+	if ( mscript->active ) {
+		if ( mscript->teststream != NULL ) {
+			mscript->keep = TRUE;
 		} else {
-			switch ( sieve_result_implicit_keep(result) ) {
+			switch ( sieve_result_implicit_keep
+				(result, action_ehandler, flags, TRUE) ) {
 			case SIEVE_EXEC_OK:
-				(*mscript)->keep = TRUE;
+				mscript->keep = TRUE;
 				break;
 			case SIEVE_EXEC_TEMP_FAILURE:
 				if (!sieve_result_executed(result)) {
@@ -766,12 +839,11 @@ int sieve_multiscript_finish(struct sieve_multiscript **mscript,
 		}
 	}
 
-	if ( keep != NULL ) *keep = (*mscript)->keep;
+	if ( keep != NULL ) *keep = mscript->keep;
 
 	/* Cleanup */
 	sieve_result_unref(&result);
-	*mscript = NULL;
-
+	*_mscript = NULL;
 	return ret;
 }
 
@@ -794,5 +866,258 @@ size_t sieve_max_script_size(struct sieve_instance *svinst)
 	return svinst->max_script_size;
 }
 
+/*
+ * User log
+ */
 
+const char *sieve_user_get_log_path
+(struct sieve_instance *svinst,
+	struct sieve_script *user_script)
+{
+	const char *log_path = NULL;
+
+	/* Determine user log file path */
+	if ( (log_path=sieve_setting_get
+		(svinst, "sieve_user_log")) == NULL ) {
+		const char *path;
+
+		if ( user_script == NULL ||
+			(path=sieve_file_script_get_path(user_script)) == NULL ) {
+			/* Default */
+			if ( svinst->home_dir != NULL ) {
+				log_path = t_strconcat
+					(svinst->home_dir, "/.dovecot.sieve.log", NULL);
+			}
+		} else {
+			/* Use script file as a base (legacy behavior) */
+			log_path = t_strconcat(path, ".log", NULL);
+		}
+	} else if ( svinst->home_dir != NULL ) {
+		/* Expand home dir if necessary */
+		if ( log_path[0] == '~' ) {
+			log_path = home_expand_tilde(log_path, svinst->home_dir);
+		} else if ( log_path[0] != '/' ) {
+			log_path = t_strconcat(svinst->home_dir, "/", log_path, NULL);
+		}
+	}
+	return log_path;
+}
+
+/*
+ * Script trace log
+ */
+
+struct sieve_trace_log {
+	struct ostream *output;
+};
+
+int sieve_trace_log_create
+(struct sieve_instance *svinst, const char *path,
+	struct sieve_trace_log **trace_log_r)
+{
+	struct sieve_trace_log *trace_log;
+	struct ostream *output;
+	int fd;
+
+	*trace_log_r = NULL;
+
+	if ( path == NULL ) {
+		output = o_stream_create_fd(1, 0);
+	} else {
+		fd = open(path, O_CREAT | O_APPEND | O_WRONLY, 0600);
+		if ( fd == -1 ) {
+			sieve_sys_error(svinst, "trace: "
+				"creat(%s) failed: %m", path);
+			return -1;
+		}
+		output = o_stream_create_fd_autoclose(&fd, 0);
+		o_stream_set_name(output, path);
+	}
+
+	trace_log = i_new(struct sieve_trace_log, 1);
+	trace_log->output = output;
+
+	*trace_log_r = trace_log;
+	return 0;
+}
+
+int sieve_trace_log_create_dir
+(struct sieve_instance *svinst, const char *dir,
+	const char *label, struct sieve_trace_log **trace_log_r)
+{
+	static unsigned int counter = 0;
+	const char *timestamp, *prefix;
+	struct stat st;
+
+	*trace_log_r = NULL;
+
+	if (stat(dir, &st) < 0) {
+		if (errno != ENOENT && errno != EACCES) {
+			sieve_sys_error(svinst, "trace: "
+				"stat(%s) failed: %m", dir);
+		}
+		return -1;
+	}
+
+	timestamp = t_strflocaltime("%Y%m%d-%H%M%S", ioloop_time);
+
+	counter++;
+	if ( label != NULL ) {
+		prefix = t_strdup_printf("%s/%s.%s.%s.%u.trace",
+			dir, label, timestamp, my_pid, counter);
+	} else {
+		prefix = t_strdup_printf("%s/%s.%s.%u.trace",
+			dir, timestamp, my_pid, counter);
+	}
+	return sieve_trace_log_create(svinst, prefix, trace_log_r);
+}
+
+int sieve_trace_log_open
+(struct sieve_instance *svinst, const char *label,
+	struct sieve_trace_log **trace_log_r)
+{
+	const char *trace_dir =
+		sieve_setting_get(svinst, "sieve_trace_dir");
+
+	*trace_log_r = NULL;
+	if (trace_dir == NULL)
+		return -1;
+
+	if ( svinst->home_dir != NULL ) {
+		/* Expand home dir if necessary */
+		if ( trace_dir[0] == '~' ) {
+			trace_dir = home_expand_tilde(trace_dir, svinst->home_dir);
+		} else if ( trace_dir[0] != '/' ) {
+			trace_dir = t_strconcat(svinst->home_dir, "/", trace_dir, NULL);
+		}
+	}
+
+	return sieve_trace_log_create_dir
+		(svinst, trace_dir, label, trace_log_r)	;
+}
+
+void sieve_trace_log_write_line
+(struct sieve_trace_log *trace_log, const string_t *line)
+{
+	struct const_iovec iov[2];
+
+	if (line == NULL) {
+		o_stream_nsend_str(trace_log->output, "\n");
+		return;
+	}
+
+	memset(iov, 0, sizeof(iov));
+	iov[0].iov_base = str_data(line);
+	iov[0].iov_len = str_len(line);
+	iov[1].iov_base = "\n";
+	iov[1].iov_len = 1;
+	o_stream_nsendv(trace_log->output, iov, 2);
+}
+
+void sieve_trace_log_free(struct sieve_trace_log **_trace_log)
+{
+	struct sieve_trace_log *trace_log = *_trace_log;
+
+	*_trace_log = NULL;
+
+	if (o_stream_finish(trace_log->output) < 0) {
+		i_error("write(%s) failed: %s",
+			o_stream_get_name(trace_log->output),
+			o_stream_get_error(trace_log->output));
+	}
+	o_stream_destroy(&trace_log->output);
+	i_free(trace_log);
+}
+
+int sieve_trace_config_get(struct sieve_instance *svinst,
+	struct sieve_trace_config *tr_config)
+{
+	const char *tr_level =
+		sieve_setting_get(svinst, "sieve_trace_level");
+	bool tr_debug, tr_addresses;
+
+	i_zero(tr_config);
+
+	if ( tr_level == NULL || *tr_level == '\0' ||
+		strcasecmp(tr_level, "none") == 0 )
+		return -1;
+
+	if ( strcasecmp(tr_level, "actions") == 0 ) {
+		tr_config->level = SIEVE_TRLVL_ACTIONS;
+	} else if ( strcasecmp(tr_level, "commands") == 0 ) {
+		tr_config->level = SIEVE_TRLVL_COMMANDS;
+	} else if ( strcasecmp(tr_level, "tests") == 0 ) {
+		tr_config->level = SIEVE_TRLVL_TESTS;
+	} else if ( strcasecmp(tr_level, "matching") == 0 ) {
+		tr_config->level = SIEVE_TRLVL_MATCHING;
+	} else {
+		sieve_sys_error(svinst,
+			"Unknown trace level: %s", tr_level);
+		return -1;
+	}
+
+	tr_debug = FALSE;
+	(void)sieve_setting_get_bool_value
+		(svinst, "sieve_trace_debug", &tr_debug);
+	tr_addresses = FALSE;
+	(void)sieve_setting_get_bool_value
+		(svinst, "sieve_trace_addresses", &tr_addresses);
+
+	if (tr_debug)
+		tr_config->flags |= SIEVE_TRFLG_DEBUG;
+	if (tr_addresses)
+		tr_config->flags |= SIEVE_TRFLG_ADDRESSES;
+	return 0;
+}
+
+/*
+ * User e-mail address
+ */
+
+const struct smtp_address *sieve_get_user_email
+(struct sieve_instance *svinst)
+{
+	struct smtp_address *address;
+	const char *username = svinst->username;
+
+	if (svinst->user_email_implicit != NULL)
+		return svinst->user_email_implicit;
+	if (svinst->user_email != NULL)
+		return svinst->user_email;
+
+	if (smtp_address_parse_mailbox(svinst->pool, username,
+		0, &address, NULL) >= 0) {
+		svinst->user_email_implicit = address;
+		return svinst->user_email_implicit;
+	}
+
+	if ( svinst->domainname != NULL ) {
+		svinst->user_email_implicit = smtp_address_create(svinst->pool,
+			username, svinst->domainname);
+		return svinst->user_email_implicit;
+	}
+	return NULL;
+}
+
+/*
+ * Postmaster address
+ */
+
+const struct message_address *
+sieve_get_postmaster(const struct sieve_script_env *senv)
+{
+	i_assert(senv->postmaster_address != NULL);
+	return senv->postmaster_address;
+}
+
+const char *
+sieve_get_postmaster_address(const struct sieve_script_env *senv)
+{
+	const struct message_address *postmaster =
+		sieve_get_postmaster(senv);
+	string_t *addr = t_str_new(256);
+
+	message_address_write(addr, postmaster);
+	return str_c(addr);
+}
 
