@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2014 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #ifndef __SIEVE_H
@@ -158,20 +158,29 @@ void sieve_hexdump
 int sieve_test
 	(struct sieve_binary *sbin, const struct sieve_message_data *msgdata,
 		const struct sieve_script_env *senv, struct sieve_error_handler *ehandler,
-		struct ostream *stream, enum sieve_runtime_flags flags, bool *keep);
+		struct ostream *stream, enum sieve_execute_flags flags, bool *keep);
 
 /*
  * Script execution
  */
 
+/* sieve_script_env_init:
+ *
+ *   Initializes the scirpt environment from the given mail_user.
+ */
+int sieve_script_env_init(struct sieve_script_env *senv,
+	struct mail_user *user, const char **error_r);
+
 /* sieve_execute:
  *
  *   Executes the binary, including the result.
  */
 int sieve_execute
 	(struct sieve_binary *sbin, const struct sieve_message_data *msgdata,
-		const struct sieve_script_env *senv, struct sieve_error_handler *ehandler,
-		enum sieve_runtime_flags flags, bool *keep);
+		const struct sieve_script_env *senv,
+		struct sieve_error_handler *exec_ehandler,
+		struct sieve_error_handler *action_ehandler,
+		enum sieve_execute_flags flags, bool *keep);
 
 /*
  * Multiscript support
@@ -188,16 +197,28 @@ struct sieve_multiscript *sieve_multiscript_start_test
 
 bool sieve_multiscript_run
 	(struct sieve_multiscript *mscript, struct sieve_binary *sbin,
-		struct sieve_error_handler *ehandler, enum sieve_runtime_flags flags,
-		bool final);
+		struct sieve_error_handler *exec_ehandler,
+		struct sieve_error_handler *action_ehandler,
+		enum sieve_execute_flags flags);
+
+bool sieve_multiscript_will_discard
+	(struct sieve_multiscript *mscript);
+void sieve_multiscript_run_discard
+	(struct sieve_multiscript *mscript, struct sieve_binary *sbin,
+		struct sieve_error_handler *exec_ehandler,
+		struct sieve_error_handler *action_ehandler,
+		enum sieve_execute_flags flags);
 
 int sieve_multiscript_status(struct sieve_multiscript *mscript);
 
-int sieve_multiscript_tempfail(struct sieve_multiscript **mscript,
-	struct sieve_error_handler *ehandler);
+int sieve_multiscript_tempfail
+	(struct sieve_multiscript **_mscript,
+		struct sieve_error_handler *action_ehandler,
+		enum sieve_execute_flags flags);
 int sieve_multiscript_finish
-	(struct sieve_multiscript **mscript, struct sieve_error_handler *ehandler,
-		bool *keep);
+	(struct sieve_multiscript **_mscript,
+		struct sieve_error_handler *action_ehandler,
+		enum sieve_execute_flags flags, bool *keep);
 
 /*
  * Configured limits
@@ -207,4 +228,38 @@ unsigned int sieve_max_redirects(struct sieve_instance *svinst);
 unsigned int sieve_max_actions(struct sieve_instance *svinst);
 size_t sieve_max_script_size(struct sieve_instance *svinst);
 
+/*
+ * User log
+ */
+
+const char *sieve_user_get_log_path
+	(struct sieve_instance *svinst,
+		struct sieve_script *user_script)
+	ATTR_NULL(2);
+
+/*
+ * Script trace log
+ */
+
+struct sieve_trace_log;
+
+int sieve_trace_log_create
+	(struct sieve_instance *svinst, const char *path,
+		struct sieve_trace_log **trace_log_r)
+	ATTR_NULL(2);
+int sieve_trace_log_create_dir
+	(struct sieve_instance *svinst, const char *dir,
+		const char *label, struct sieve_trace_log **trace_log_r)
+	ATTR_NULL(3);
+
+int sieve_trace_log_open
+	(struct sieve_instance *svinst, const char *label,
+		struct sieve_trace_log **trace_log_r)
+	ATTR_NULL(2);
+
+void sieve_trace_log_free(struct sieve_trace_log **_trace_log);
+
+int sieve_trace_config_get(struct sieve_instance *svinst,
+	struct sieve_trace_config *tr_config);
+
 #endif
