@@ -1,9 +1,9 @@
-/* Copyright (c) 2002-2016 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
 #include "mempool.h"
-#include "abspath.h"
+#include "path-util.h"
 #include "istream.h"
 #include "time-util.h"
 #include "eacces-error.h"
@@ -55,10 +55,20 @@ static void sieve_file_script_handle_error
 	const char *name, enum sieve_error *error_r)
 {
 	struct sieve_script *script = &fscript->script;
+	const char *abspath, *error;
 
 	switch ( errno ) {
 	case ENOENT:
-		sieve_script_sys_debug(script, "File `%s' not found", t_abspath(path));
+		if (t_abspath(path, &abspath, &error) < 0) {
+			sieve_script_set_error(script,
+				SIEVE_ERROR_TEMP_FAILURE,
+				"t_abspath(%s) failed: %s",
+				path, error);
+			*error_r = SIEVE_ERROR_TEMP_FAILURE;
+			break;
+		}
+		sieve_script_sys_debug(script, "File `%s' not found",
+				       abspath);
 		sieve_script_set_error(script,
 			SIEVE_ERROR_NOT_FOUND,
 			"Sieve script `%s' not found", name);
@@ -471,18 +481,29 @@ static int sieve_file_script_binary_read_metadata
 	struct sieve_file_script *fscript = (struct sieve_file_script *)script;
 	struct sieve_instance *svinst = script->storage->svinst;
 	struct sieve_binary *sbin = sieve_binary_block_get_binary(sblock);
-	time_t bmtime = sieve_binary_mtime(sbin);
-	time_t smtime = ( fscript->st.st_mtime > fscript->lnk_st.st_mtime ?
-		fscript->st.st_mtime : fscript->lnk_st.st_mtime );
+	const struct stat *sstat, *bstat;
 
-	if ( bmtime <= smtime ) {
+	bstat = sieve_binary_stat(sbin);
+	if ( fscript->st.st_mtime > fscript->lnk_st.st_mtime ||
+		(fscript->st.st_mtime == fscript->lnk_st.st_mtime &&
+		 ST_MTIME_NSEC(fscript->st) >= ST_MTIME_NSEC(fscript->lnk_st)) ) {
+		sstat = &fscript->st;
+	} else {
+		sstat = &fscript->lnk_st;
+	}
+
+	if ( bstat->st_mtime < sstat->st_mtime ||
+		(bstat->st_mtime == sstat->st_mtime &&
+			ST_MTIME_NSEC(*bstat) <= ST_MTIME_NSEC(*sstat)) ) {
 		if ( svinst->debug ) {
 			sieve_script_sys_debug(script,
 				"Sieve binary `%s' is not newer "
-				"than the Sieve script `%s' (%s <= %s)",
+				"than the Sieve script `%s' (%s.%lu <= %s.%lu)",
 				sieve_binary_path(sbin), sieve_script_location(script),
-				t_strflocaltime("%Y-%m-%d %H:%M:%S", bmtime),
-				t_strflocaltime("%Y-%m-%d %H:%M:%S", smtime));
+				t_strflocaltime("%Y-%m-%d %H:%M:%S", bstat->st_mtime),
+				ST_MTIME_NSEC(*bstat),
+				t_strflocaltime("%Y-%m-%d %H:%M:%S", sstat->st_mtime),
+				ST_MTIME_NSEC(*sstat));
 		}
 		return 0;
 	}
