@@ -1,8 +1,8 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
-#include "abspath.h"
+#include "path-util.h"
 #include "home-expand.h"
 #include "ioloop.h"
 #include "mkdir-parents.h"
@@ -10,8 +10,6 @@
 #include "unlink-old-files.h"
 #include "mail-storage-private.h"
 
-#include "realpath.h"
-
 #include "sieve.h"
 #include "sieve-common.h"
 #include "sieve-settings.h"
@@ -54,6 +52,7 @@ static int sieve_file_storage_stat
 {
 	struct sieve_storage *storage = &fstorage->storage;
 	struct stat st;
+	const char *abspath, *error;
 
 	if ( lstat(path, &st) == 0 ) {
 		fstorage->lnk_st = st;
@@ -66,8 +65,14 @@ static int sieve_file_storage_stat
 
 	switch ( errno ) {
 	case ENOENT:
+		if (t_abspath(path, &abspath, &error) < 0) {
+			sieve_storage_set_critical(storage,
+				"t_abspath(%s) failed: %s", path, error);
+			*error_r = SIEVE_ERROR_TEMP_FAILURE;
+			break;
+		}
 		sieve_storage_sys_debug(storage,
-			"Storage path `%s' not found", t_abspath(path));
+			"Storage path `%s' not found", abspath);
 		sieve_storage_set_internal_error(storage); // should be overriden
 		*error_r = SIEVE_ERROR_NOT_FOUND;
 		break;
@@ -119,8 +124,8 @@ static const char *sieve_storage_get_relative_link_path
 	/* Add trailing '/' when link path is not empty
 	 */
 	pathlen = strlen(link_path);
-    if ( pathlen != 0 && link_path[pathlen-1] != '/')
-        return t_strconcat(link_path, "/", NULL);
+	if ( pathlen != 0 && link_path[pathlen-1] != '/')
+		return t_strconcat(link_path, "/", NULL);
 
 	return t_strdup(link_path);
 }
@@ -294,7 +299,7 @@ static int sieve_file_storage_init_common
 	ATTR_NULL(2, 3)
 {
 	struct sieve_storage *storage = &fstorage->storage;
-	const char *tmp_dir, *link_path, *active_fname, *storage_dir;
+	const char *tmp_dir, *link_path, *active_fname, *storage_dir, *error;
 	bool have_link = FALSE;
 	int ret;
 
@@ -333,11 +338,11 @@ static int sieve_file_storage_init_common
 			return -1;
 		}
 
-		if (t_realpath(active_dir, &active_dir) < 0) {
+		if (t_realpath(active_dir, &active_dir, &error) < 0) {
 			if (errno != ENOENT) {
 				sieve_storage_sys_error(storage,
-					"Failed to normalize active script directory (path=%s): %m",
-					active_dir);
+					"Failed to normalize active script directory (path=%s): %s",
+					active_dir, error);
 				*error_r = SIEVE_ERROR_TEMP_FAILURE;
 				return -1;
 			} 
@@ -455,10 +460,10 @@ static int sieve_file_storage_init_common
 		return -1;
 
 	if ( have_link ) {
-		if ( t_realpath(storage_path, &storage_path) < 0 ) {
+		if ( t_realpath(storage_path, &storage_path, &error) < 0 ) {
 			sieve_storage_sys_error(storage,
-				"Failed to normalize storage path (path=%s): %m",
-				storage_path);
+				"Failed to normalize storage path (path=%s): %s",
+				storage_path, error);
 			*error_r = SIEVE_ERROR_TEMP_FAILURE;
 			return -1;
 		}
@@ -597,18 +602,9 @@ static void sieve_file_storage_autodetect
 	 * It's $HOME/sieve or /sieve when (presumed to be) chrooted.
 	 */
 	if ( home != NULL && *home != '\0' ) {
-		if (access(home, mode) == 0) {
-			/* Use default ~/sieve */
-			sieve_storage_sys_debug(storage,
-				"Root exists (%s)", home);
-
-			*storage_path_r = t_strconcat(home, "/sieve", NULL);
-		} else {
-			/* Don't have required access on the home directory */
-
-			sieve_storage_sys_debug(storage,
-				"access(%s, rwx) failed: %m", home);
-		}
+		/* Use default ~/sieve */
+		sieve_storage_sys_debug(storage, "Use home (%s)", home);
+		*storage_path_r = t_strconcat(home, "/sieve", NULL);
 	} else {
 			sieve_storage_sys_debug(storage,
 				"HOME is not set");
