@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -322,8 +322,7 @@ static int db_ldap_connect_finish(struct ldap_connection *conn, int ret)
 		return -1;
 	}
 
-	if (conn->to != NULL)
-		timeout_remove(&conn->to);
+	timeout_remove(&conn->to);
 	conn->conn_state = LDAP_CONN_STATE_BOUND;
 	sieve_storage_sys_debug(storage, "db: "
 		"Successfully bound (dn %s)",
@@ -538,7 +537,7 @@ static void ldap_input(struct ldap_connection *conn)
 		if (conn->ld == NULL)
 			return;
 
-		memset(&timeout, 0, sizeof(timeout));
+		i_zero(&timeout);
 		ret = ldap_result(conn->ld, LDAP_RES_ANY, 0, &timeout, &msg);
 #ifdef OPENLDAP_ASYNC_WORKAROUND
 		if (ret == 0) {
@@ -653,8 +652,7 @@ static int db_ldap_bind(struct ldap_connection *conn)
 	conn->conn_state = LDAP_CONN_STATE_BINDING;
 	conn->default_bind_msgid = msgid;
 
-	if (conn->to != NULL)
-		timeout_remove(&conn->to);
+	timeout_remove(&conn->to);
 	conn->to = timeout_add(DB_LDAP_REQUEST_LOST_TIMEOUT_SECS*1000,
 			       ldap_connection_timeout, conn);
 	return 0;
@@ -683,14 +681,14 @@ static int db_ldap_get_fd(struct ldap_connection *conn)
 	return 0;
 }
 
-static int ATTR_NULL(1)
+static int
 db_ldap_set_opt(struct ldap_connection *conn, int opt, const void *value,
 		const char *optname, const char *value_str)
 {
 	struct sieve_storage *storage = &conn->lstorage->storage;
 	int ret;
 
-	ret = ldap_set_option(conn == NULL ? NULL : conn->ld, opt, value);
+	ret = ldap_set_option(conn->ld, opt, value);
 	if (ret != LDAP_SUCCESS) {
 		sieve_storage_sys_error(storage, "db: "
 			"Can't set option %s to %s: %s",
@@ -700,7 +698,7 @@ db_ldap_set_opt(struct ldap_connection *conn, int opt, const void *value,
 	return 0;
 }
 
-static int ATTR_NULL(1)
+static int
 db_ldap_set_opt_str(struct ldap_connection *conn, int opt, const char *value,
 		    const char *optname)
 {
@@ -717,23 +715,23 @@ static int db_ldap_set_tls_options(struct ldap_connection *conn)
 		return 0;
 
 #ifdef OPENLDAP_TLS_OPTIONS
-	if (db_ldap_set_opt_str(NULL, LDAP_OPT_X_TLS_CACERTFILE,
+	if (db_ldap_set_opt_str(conn, LDAP_OPT_X_TLS_CACERTFILE,
 			    set->tls_ca_cert_file, "tls_ca_cert_file") < 0)
 		return -1;
-	if (db_ldap_set_opt_str(NULL, LDAP_OPT_X_TLS_CACERTDIR,
+	if (db_ldap_set_opt_str(conn, LDAP_OPT_X_TLS_CACERTDIR,
 			    set->tls_ca_cert_dir, "tls_ca_cert_dir") < 0)
 		return -1;
-	if (db_ldap_set_opt_str(NULL, LDAP_OPT_X_TLS_CERTFILE,
+	if (db_ldap_set_opt_str(conn, LDAP_OPT_X_TLS_CERTFILE,
 			    set->tls_cert_file, "tls_cert_file") < 0)
 		return -1;
-	if (db_ldap_set_opt_str(NULL, LDAP_OPT_X_TLS_KEYFILE,
+	if (db_ldap_set_opt_str(conn, LDAP_OPT_X_TLS_KEYFILE,
 			    set->tls_key_file, "tls_key_file") < 0)
 		return -1;
-	if (db_ldap_set_opt_str(NULL, LDAP_OPT_X_TLS_CIPHER_SUITE,
+	if (db_ldap_set_opt_str(conn, LDAP_OPT_X_TLS_CIPHER_SUITE,
 			    set->tls_cipher_suite, "tls_cipher_suite") < 0)
 		return -1;
 	if (set->tls_require_cert != NULL) {
-		if (db_ldap_set_opt(NULL, LDAP_OPT_X_TLS_REQUIRE_CERT,
+		if (db_ldap_set_opt(conn, LDAP_OPT_X_TLS_REQUIRE_CERT,
 				&set->ldap_tls_require_cert,
 				"tls_require_cert", set->tls_require_cert) < 0)
 			return -1;
@@ -764,7 +762,7 @@ static int db_ldap_set_options(struct ldap_connection *conn)
 		return -1;
 #ifdef LDAP_OPT_DEBUG_LEVEL
 	if (str_to_int(set->debug_level, &value) >= 0 && value != 0) {
-		if (db_ldap_set_opt(NULL, LDAP_OPT_DEBUG_LEVEL, &value,
+		if (db_ldap_set_opt(conn, LDAP_OPT_DEBUG_LEVEL, &value,
 				"debug_level", set->debug_level) < 0)
 			return -1;
 	}
@@ -797,8 +795,11 @@ int sieve_ldap_db_connect(struct ldap_connection *conn)
 	const struct sieve_ldap_storage_settings *set = &conn->lstorage->set;
 	struct sieve_storage *storage = &conn->lstorage->storage;
 	struct timeval start, end;
-	int debug_level, ret;
+	int debug_level;
 	bool debug;
+#if defined(HAVE_LDAP_SASL) || defined(LDAP_HAVE_START_TLS_S)
+	int ret;
+#endif
 
 	if (conn->conn_state != LDAP_CONN_STATE_DISCONNECTED)
 		return 0;
@@ -809,7 +810,7 @@ int sieve_ldap_db_connect(struct ldap_connection *conn)
 
 	if (debug) {
 		if (gettimeofday(&start, NULL) < 0)
-			memset(&start, 0, sizeof(start));
+			i_zero(&start);
 	}
 	i_assert(conn->pending_count == 0);
 	if (conn->ld == NULL) {
@@ -861,7 +862,7 @@ int sieve_ldap_db_connect(struct ldap_connection *conn)
 #ifdef HAVE_LDAP_SASL
 		struct db_ldap_sasl_bind_context context;
 
-		memset(&context, 0, sizeof(context));
+		i_zero(&context);
 		context.authcid = set->dn;
 		context.passwd = set->dnpass;
 		context.realm = set->sasl_realm;
@@ -902,8 +903,7 @@ int sieve_ldap_db_connect(struct ldap_connection *conn)
 void db_ldap_enable_input(struct ldap_connection *conn, bool enable)
 {
 	if (!enable) {
-		if (conn->io != NULL)
-			io_remove(&conn->io);
+		io_remove(&conn->io);
 	} else {
 		if (conn->io == NULL && conn->fd != -1) {
 			conn->io = io_add(conn->fd, IO_READ, ldap_input, conn);
@@ -932,8 +932,7 @@ static void db_ldap_conn_close(struct ldap_connection *conn)
 	conn->conn_state = LDAP_CONN_STATE_DISCONNECTED;
 	conn->default_bind_msgid = -1;
 
-	if (conn->to != NULL)
-		timeout_remove(&conn->to);
+	timeout_remove(&conn->to);
 
 	if (conn->pending_count != 0) {
 		requests = array_idx(&conn->request_array, 0);
@@ -952,11 +951,9 @@ static void db_ldap_conn_close(struct ldap_connection *conn)
 	}
 	conn->fd = -1;
 
-	if (conn->io != NULL) {
-		/* the fd may have already been closed before ldap_unbind(),
-		   so we'll have to use io_remove_closed(). */
-		io_remove_closed(&conn->io);
-	}
+	/* the fd may have already been closed before ldap_unbind(),
+	   so we'll have to use io_remove_closed(). */
+	io_remove_closed(&conn->io);
 
 	if (aqueue_count(conn->request_queue) > 0) {
 		conn->to = timeout_add(DB_LDAP_REQUEST_DISCONNECT_TIMEOUT_SECS *
@@ -1203,7 +1200,7 @@ db_ldap_get_var_expand_table(struct ldap_connection *conn,
 
 	/* keep the extra fields at the beginning. the last static_tab field
 	   contains the ending NULL-fields. */
-	tab = t_malloc((auth_count) * sizeof(*tab));
+	tab = t_malloc_no0((auth_count) * sizeof(*tab));
 
 	memcpy(tab, auth_request_var_expand_static_tab,
 	       auth_count * sizeof(*tab));
@@ -1263,8 +1260,9 @@ int sieve_ldap_db_lookup_script(struct ldap_connection *conn,
 	struct sieve_storage *storage = &lstorage->storage;
 	const struct sieve_ldap_storage_settings *set = &lstorage->set;
 	struct sieve_ldap_script_lookup_request *request;
-	const struct var_expand_table *vars;
+	const struct var_expand_table *tab;
 	char **attr_names;
+	const char *error;
 	string_t *str;
 
 	pool_t pool = pool_alloconly_create
@@ -1272,17 +1270,28 @@ int sieve_ldap_db_lookup_script(struct ldap_connection *conn,
 	request = p_new(pool, struct sieve_ldap_script_lookup_request, 1);
 	request->request.pool = pool;
 
-	vars = db_ldap_get_var_expand_table(conn, name);
+	tab = db_ldap_get_var_expand_table(conn, name);
 
 	str = t_str_new(512);
-	var_expand(str, set->base, vars);
+	if (var_expand(str, set->base, tab, &error) <= 0) {
+		sieve_storage_sys_error(storage, "db: "
+			"Failed to expand base=%s: %s",
+			set->base, error);
+		return -1;
+	}
 	request->request.base = p_strdup(pool, str_c(str));
 
 	attr_names = p_new(pool, char *, 3);
 	attr_names[0] = p_strdup(pool, set->sieve_ldap_mod_attr);
 
 	str_truncate(str, 0);
-	var_expand(str, set->sieve_ldap_filter, vars);
+	if (var_expand(str, set->sieve_ldap_filter, tab, &error) <= 0) {
+		sieve_storage_sys_error(storage, "db: "
+			"Failed to expand sieve_ldap_filter=%s: %s",
+			set->sieve_ldap_filter, error);
+		return -1;
+	}
+
 	request->request.scope = lstorage->set.ldap_scope;
 	request->request.filter = p_strdup(pool, str_c(str));
 	request->request.attributes = attr_names;
@@ -1290,7 +1299,7 @@ int sieve_ldap_db_lookup_script(struct ldap_connection *conn,
 	sieve_storage_sys_debug(storage,
 			       "base=%s scope=%s filter=%s fields=%s",
 			       request->request.base, lstorage->set.scope,
-			       request->request.filter, attr_names == NULL ? "(all)" :
+			       request->request.filter,
 			       t_strarray_join((const char **)attr_names, ","));
 
 	request->request.callback = sieve_ldap_lookup_script_callback;
@@ -1330,8 +1339,7 @@ sieve_ldap_read_script_callback(struct ldap_connection *conn,
 		} else {
 			sieve_storage_sys_error(storage, "db: "
 				"Search returned more than one entry for Sieve script DN");
-			if (srequest->result != NULL)
-				i_stream_unref(&srequest->result);
+			i_stream_unref(&srequest->result);
 		}
 
 	} else {
@@ -1365,7 +1373,6 @@ int sieve_ldap_db_read_script(struct ldap_connection *conn,
 	sieve_storage_sys_debug(storage,
 			       "base=%s scope=base filter=%s fields=%s",
 			       request->request.base, request->request.filter,
-			       attr_names == NULL ? "(all)" :
 			       t_strarray_join((const char **)attr_names, ","));
 
 	request->request.callback = sieve_ldap_read_script_callback;
