@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -33,15 +33,15 @@ static bool tst_header_generate
 	(const struct sieve_codegen_env *cgenv, struct sieve_command *tst);
 
 const struct sieve_command_def tst_header = {
-	"header",
-	SCT_TEST,
-	2, 0, FALSE, FALSE,
-	tst_header_registered,
-	NULL,
-	tst_header_validate,
-	NULL,
-	tst_header_generate,
-	NULL
+	.identifier = "header",
+	.type = SCT_TEST,
+	.positional_args = 2,
+	.subtests = 0,
+	.block_allowed = FALSE,
+	.block_required = FALSE,
+	.registered = tst_header_registered,
+	.validate = tst_header_validate,
+	.generate = tst_header_generate
 };
 
 /*
@@ -54,11 +54,10 @@ static int tst_header_operation_execute
 	(const struct sieve_runtime_env *renv, sieve_size_t *address);
 
 const struct sieve_operation_def tst_header_operation = {
-	"HEADER",
-	NULL,
-	SIEVE_OPERATION_HEADER,
-	tst_header_operation_dump,
-	tst_header_operation_execute
+	.mnemonic = "HEADER",
+	.code = SIEVE_OPERATION_HEADER,
+	.dump = tst_header_operation_dump,
+	.execute = tst_header_operation_execute
 };
 
 /*
@@ -167,7 +166,7 @@ static int tst_header_operation_execute
 	 */
 
 	/* Optional operands */
-	memset(&svmos, 0, sizeof(svmos));
+	i_zero(&svmos);
 	if ( sieve_message_opr_optional_read
 		(renv, address, NULL, &ret, NULL, &mcht, &cmp, &svmos) < 0 )
 		return ret;
@@ -191,7 +190,7 @@ static int tst_header_operation_execute
 	/* Get header */
 	sieve_runtime_trace_descend(renv);
 	if ( (ret=sieve_message_get_header_fields
-		(renv, hdr_list, &svmos, &value_list)) <= 0 )
+		(renv, hdr_list, &svmos, TRUE, &value_list)) <= 0 )
 		return ret;
 	sieve_runtime_trace_ascend(renv);
 
