@@ -1,8 +1,5 @@
-/* Copyright (c) 2002-2014 Pigeonhole authors, see the included COPYING file
- */
-
-#ifndef __EDIT_MAIL_H
-#define __EDIT_MAIL_H
+#ifndef EDIT_MAIL_H
+#define EDIT_MAIL_H
 
 struct edit_mail;
 
@@ -53,4 +50,4 @@ bool edit_mail_headers_iterate_replace
 
 
 
-#endif /* __edit_mail_H */
+#endif
