@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "login-common.h"
@@ -12,7 +12,6 @@
 #include "master-service.h"
 #include "master-auth.h"
 #include "auth-client.h"
-#include "ssl-proxy.h"
 
 #include "managesieve-parser.h"
 #include "managesieve-quote.h"
@@ -74,7 +73,7 @@ static void client_send_capabilities(struct client *client)
 			(client, t_strconcat("\"SASL\" \"", saslcap, "\"\r\n", NULL));
 
 		/* STARTTLS */
-		if (ssl_initialized && !client->tls)
+		if (login_ssl_initialized && !client->tls)
 			client_send_raw(client, "\"STARTTLS\"\r\n" );
 
 		/* Protocol version */
@@ -301,8 +300,10 @@ static bool managesieve_client_input_next_cmd(struct client *_client)
 			if ( args[0].type != MANAGESIEVE_ARG_EOL )
 				ret = -1;
 		}
-		if (ret > 0)
-			ret = client->cmd->func(client, args);
+	}
+	if (ret > 0) {
+		i_assert(client->cmd != NULL);
+		ret = client->cmd->func(client, args);
 	}
 
 	if (ret != 0)
@@ -340,8 +341,7 @@ static void managesieve_client_input(struct client *client)
 			/* FIXME: Can't do untagged responses with managesieve. Any other ways?
 			client_send_ok(client, AUTH_SERVER_WAITING_MSG);
 			*/
-			if (client->to_auth_waiting != NULL)
-				timeout_remove(&client->to_auth_waiting);
+			timeout_remove(&client->to_auth_waiting);
 
 			client->input_blocked = TRUE;
 			break;
@@ -502,25 +502,24 @@ static void managesieve_login_deinit(void)
 }
 
 static struct client_vfuncs managesieve_client_vfuncs = {
-	managesieve_client_alloc,
-	managesieve_client_create,
-	managesieve_client_destroy,
-	managesieve_client_notify_auth_ready,
-	managesieve_client_notify_disconnect,
-	NULL,
-	managesieve_client_notify_starttls,
-	managesieve_client_starttls,
-	managesieve_client_input,
-	managesieve_client_auth_send_challenge,
-	managesieve_client_auth_parse_response,
-	managesieve_client_auth_result,
-	managesieve_proxy_reset,
-	managesieve_proxy_parse_line,
-	managesieve_proxy_error,
-	managesieve_proxy_get_state,
-	client_common_send_raw_data,
-	managesieve_client_input_next_cmd,
-	client_common_default_free,
+	.alloc = managesieve_client_alloc,
+	.create = managesieve_client_create,
+	.destroy = managesieve_client_destroy,
+	.notify_auth_ready = managesieve_client_notify_auth_ready,
+	.notify_disconnect = managesieve_client_notify_disconnect,
+	.notify_starttls = managesieve_client_notify_starttls,
+	.starttls = managesieve_client_starttls,
+	.input = managesieve_client_input,
+	.auth_send_challenge = managesieve_client_auth_send_challenge,
+	.auth_parse_response = managesieve_client_auth_parse_response,
+	.auth_result = managesieve_client_auth_result,
+	.proxy_reset = managesieve_proxy_reset,
+	.proxy_parse_line = managesieve_proxy_parse_line,
+	.proxy_error = managesieve_proxy_error,
+	.proxy_get_state = managesieve_proxy_get_state,
+	.send_raw_data = client_common_send_raw_data,
+	.input_next_cmd = managesieve_client_input_next_cmd,
+	.free = client_common_default_free,
 };
 
 static const struct login_binary managesieve_login_binary = {
@@ -531,7 +530,9 @@ static const struct login_binary managesieve_login_binary = {
 	.client_vfuncs = &managesieve_client_vfuncs,
 	.preinit = managesieve_login_preinit,
 	.init = managesieve_login_init,
-	.deinit = managesieve_login_deinit
+	.deinit = managesieve_login_deinit,
+
+	.anonymous_login_acceptable = FALSE,
 };
 
 int main(int argc, char *argv[])
