@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -19,20 +19,32 @@ bool cmd_renamescript(struct client_command_context *cmd)
 	struct sieve_script *script;
 
 	/* <oldname> <newname> */
-	if (!client_read_string_args(cmd, 2, TRUE, &scriptname, &newname))
+	if (!client_read_string_args(cmd, TRUE, 2, &scriptname, &newname))
 		return FALSE;
 
-	script = sieve_storage_open_script
-		(storage, scriptname, NULL);
+	event_add_str(cmd->event, "old_script_name", scriptname);
+	event_add_str(cmd->event, "new_script_name", newname);
+
+	script = sieve_storage_open_script(storage, scriptname, NULL);
 	if (script == NULL) {
-		client_send_storage_error(client, storage);
+		client_command_storage_error(
+			cmd, "Failed to open script `%s' for rename to `%s'",
+			scriptname, newname);
 		return TRUE;
 	}
 
 	if (sieve_script_rename(script, newname) < 0) {
-		client_send_storage_error(client, storage);
+		client_command_storage_error(
+			cmd, "Failed to rename script `%s' to `%s'",
+			scriptname, newname);
 	} else {
 		client->renamed_count++;
+
+		struct event_passthrough *e =
+			client_command_create_finish_event(cmd);
+		e_debug(e->event(), "Renamed script `%s' to `%s'",
+			scriptname, newname);
+
 		client_send_ok(client, "Renamescript completed.");
 	}
 
