@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -10,7 +10,9 @@
 #include "istream.h"
 #include "ostream.h"
 #include "iostream.h"
+#include "iostream-rawlog.h"
 #include "var-expand.h"
+#include "time-util.h"
 #include "master-service.h"
 #include "mail-storage-service.h"
 #include "mail-namespace.h"
@@ -31,12 +33,12 @@ struct managesieve_module_register managesieve_module_register = { 0 };
 struct client *managesieve_clients = NULL;
 unsigned int managesieve_client_count = 0;
 
-static const char *managesieve_sieve_get_setting
-(void *context, const char *identifier)
+static const char *
+managesieve_sieve_get_setting(void *context, const char *identifier)
 {
 	struct mail_user *mail_user = (struct mail_user *) context;
 
-	if ( mail_user == NULL )
+	if (mail_user == NULL)
 		return NULL;
 
 	return mail_user_plugin_getenv(mail_user, identifier);
@@ -58,44 +60,52 @@ static void client_idle_timeout(struct client *client)
 	}
 }
 
-static struct sieve_storage *client_get_storage
-(struct sieve_instance *svinst, struct mail_user *user)
+static struct sieve_storage *
+client_get_storage(struct sieve_instance *svinst, struct mail_user *user,
+		   int fd_out)
 {
 	struct sieve_storage *storage;
 	enum sieve_error error;
+	const char *errormsg, *byemsg;
 
-	storage = sieve_storage_create_main
-		(svinst, user, SIEVE_STORAGE_FLAG_READWRITE, &error);
+	/* Open personal script storage */
+
+	storage = sieve_storage_create_main(svinst, user,
+					    SIEVE_STORAGE_FLAG_READWRITE,
+					    &error);
 	if (storage == NULL) {
 		switch (error) {
+		case SIEVE_ERROR_NOT_POSSIBLE:
+			byemsg = "BYE \"Sieve processing is disabled for this user.\"\r\n";
+			errormsg = "Failed to open Sieve storage: "
+				"Sieve is disabled for this user";
+			break;
 		case SIEVE_ERROR_NOT_FOUND:
-			printf("BYE \"Sieve filtering is disabled for this user.\"\n");
-
-			i_fatal("Failed to open Sieve storage: Sieve disabled for user");
+			byemsg = "BYE \"This user cannot manage personal Sieve scripts.\"\r\n";
+			errormsg = "Failed to open Sieve storage: "
+				"Personal script storage disabled or not found.";
 			break;
 		default:
-			{ 
-				struct tm *tm;
-				char str[256];
-
-				tm = localtime(&ioloop_time);
-
-				printf("BYE \"%s\"\n",
-					strftime(str, sizeof(str), CRITICAL_MSG_STAMP, tm) > 0 ?
-						i_strdup(str) : i_strdup(CRITICAL_MSG));
-
-				i_fatal("Failed to open Sieve storage.");
-			}
+			byemsg = t_strflocaltime(
+				"BYE \""CRITICAL_MSG_STAMP"\"\r\n", ioloop_time);
+			errormsg = "Failed to open Sieve storage.";
 		}
+
+		if (write(fd_out, byemsg, strlen(byemsg)) < 0) {
+			if (errno != EAGAIN && errno != EPIPE)
+				i_error("write(client) failed: %m");
+		}
+		i_fatal("%s", errormsg);
 	}
 
 	return storage;
 }
 
-struct client *client_create
-(int fd_in, int fd_out, const char *session_id, struct mail_user *user,
-	struct mail_storage_service_user *service_user,
-	const struct managesieve_settings *set)
+struct client *
+client_create(int fd_in, int fd_out, const char *session_id,
+	      struct event *event, struct mail_user *user,
+	      struct mail_storage_service_user *service_user,
+	      const struct managesieve_settings *set)
 {
 	struct client *client;
 	const char *ident;
@@ -110,14 +120,15 @@ struct client *client_create
 	svenv.username = user->username;
 	(void)mail_user_get_home(user, &svenv.home_dir);
 	svenv.base_dir = user->set->base_dir;
+	svenv.event_parent = event;
 	svenv.flags = SIEVE_FLAG_HOME_RELATIVE;
 
-	svinst = sieve_init
-		(&svenv, &managesieve_sieve_callbacks, (void *) user, set->mail_debug);
+	svinst = sieve_init(&svenv, &managesieve_sieve_callbacks,
+			    (void *) user, set->mail_debug);
 
 	/* Get Sieve storage */
 
-	storage = client_get_storage(svinst, user);
+	storage = client_get_storage(svinst, user, fd_out);
 
 	/* always use nonblocking I/O */
 	net_set_nonblock(fd_in, TRUE);
@@ -126,14 +137,16 @@ struct client *client_create
 	pool = pool_alloconly_create("managesieve client", 1024);
 	client = p_new(pool, struct client, 1);
 	client->pool = pool;
+	client->event = event;
+	event_ref(client->event);
 	client->set = set;
 	client->service_user = service_user;
 	client->session_id = p_strdup(pool, session_id);
 	client->fd_in = fd_in;
 	client->fd_out = fd_out;
-	client->input = i_stream_create_fd
-		(fd_in, set->managesieve_max_line_length, FALSE);
-	client->output = o_stream_create_fd(fd_out, (size_t)-1, FALSE);
+	client->input = i_stream_create_fd(
+		fd_in, set->managesieve_max_line_length);
+	client->output = o_stream_create_fd(fd_out, (size_t)-1);
 
 	o_stream_set_no_error_handling(client->output, TRUE);
 	i_stream_set_name(client->input, "<managesieve client>");
@@ -143,23 +156,32 @@ struct client *client_create
 
 	client->io = io_add_istream(client->input, client_input, client);
 	client->last_input = ioloop_time;
-	client->parser = managesieve_parser_create
-		(client->input, set->managesieve_max_line_length);
-	client->to_idle = timeout_add
-		(CLIENT_IDLE_TIMEOUT_MSECS, client_idle_timeout, client);
+	client->to_idle = timeout_add(CLIENT_IDLE_TIMEOUT_MSECS,
+				      client_idle_timeout, client);
 
-	client->cmd.pool =
-		pool_alloconly_create(MEMPOOL_GROWING"client command", 1024*12);
+	client->cmd.pool = pool_alloconly_create(
+		MEMPOOL_GROWING"client command", 1024*12);
 	client->cmd.client = client;
+	client->cmd.event = event_create(client->event);
 	client->user = user;
 
+	if (set->rawlog_dir[0] != '\0') {
+		(void)iostream_rawlog_create(set->rawlog_dir, &client->input,
+					     &client->output);
+	}
+
+	client->parser = managesieve_parser_create(
+		client->input, set->managesieve_max_line_length);
+
 	client->svinst = svinst;
 	client->storage = storage;
 
 	ident = mail_user_get_anvil_userip_ident(client->user);
 	if (ident != NULL) {
-		master_service_anvil_send(master_service, t_strconcat(
-			"CONNECT\t", my_pid, "\tsieve/", ident, "\n", NULL));
+		master_service_anvil_send(
+			master_service, t_strconcat("CONNECT\t", my_pid,
+						    "\tsieve/", ident,
+						    "\n", NULL));
 		client->anvil_sent = TRUE;
 	}
 
@@ -174,59 +196,47 @@ struct client *client_create
 
 static const char *client_stats(struct client *client)
 {
-	static struct var_expand_table static_tab[] = {
-		{ 't', NULL, "put_bytes" },
-		{ 'p', NULL, "put_count" },
-		{ 'b', NULL, "get_bytes" },
-		{ 'g', NULL, "get_count" },
-		{ 'v', NULL, "check_bytes" },
-		{ 'c', NULL, "check_count" },
-		{ 'd', NULL, "deleted_count" },
-		{ 'r', NULL, "renamed_count" },
-		{ 'i', NULL, "input" },
-		{ 'o', NULL, "output" },
-		{ '\0', NULL, "session" },
+	const struct var_expand_table logout_tab[] = {
+		{ 'i', dec2str(i_stream_get_absolute_offset(client->input)),
+		  "input" },
+		{ 'o', dec2str(client->output->offset), "output" },
+		{ '\0', dec2str(client->put_bytes), "put_bytes" },
+		{ '\0', dec2str(client->put_count), "put_count" },
+		{ '\0', dec2str(client->get_bytes), "get_bytes" },
+		{ '\0', dec2str(client->get_count), "get_count" },
+		{ '\0', dec2str(client->check_bytes), "check_bytes" },
+		{ '\0', dec2str(client->check_count), "check_count" },
+		{ '\0', dec2str(client->deleted_count), "deleted_count" },
+		{ '\0', dec2str(client->renamed_count), "renamed_count" },
+		{ '\0', client->session_id, "session" },
 		{ '\0', NULL, NULL }
 	};
-	struct var_expand_table *tab;
+	const struct var_expand_table *user_tab =
+		mail_user_var_expand_table(client->user);
+	const struct var_expand_table *tab =
+		t_var_expand_merge_tables(logout_tab, user_tab);
 	string_t *str;
-
-	tab = t_malloc(sizeof(static_tab));
-	memcpy(tab, static_tab, sizeof(static_tab));
-
-	tab[0].value = dec2str(client->put_bytes);
-	tab[1].value = dec2str(client->put_count);
-	tab[2].value = dec2str(client->get_bytes);
-	tab[3].value = dec2str(client->get_count);
-	tab[4].value = dec2str(client->check_bytes);
-	tab[5].value = dec2str(client->check_count);
-	tab[6].value = dec2str(client->deleted_count);
-	tab[7].value = dec2str(client->renamed_count);
-	tab[8].value = dec2str(i_stream_get_absolute_offset(client->input));
-	tab[9].value = dec2str(client->output->offset);
-	tab[10].value = client->session_id;
+	const char *error;
 
 	str = t_str_new(128);
-	var_expand(str, client->set->managesieve_logout_format, tab);
+	if (var_expand_with_funcs(str, client->set->managesieve_logout_format,
+				  tab, mail_user_var_expand_func_table,
+				  client->user, &error) < 0) {
+		i_error("Failed to expand managesieve_logout_format=%s: %s",
+			client->set->managesieve_logout_format, error);
+	}
 	return str_c(str);
 }
 
 void client_destroy(struct client *client, const char *reason)
 {
-	int ret;
+	bool ret;
 
  	i_assert(!client->handling_input);
 	i_assert(!client->destroyed);
 	client->destroyed = TRUE;
 
-	if (!client->disconnected) {
-		client->disconnected = TRUE;
-		if (reason == NULL) {
-			reason = io_stream_get_disconnect_reason(client->input,
-								 client->output);
-		}
-		i_info("%s %s", reason, client_stats(client));
-	}
+	client_disconnect(client, reason);
 
 	if (client->command_pending) {
 		/* try to deinitialize the command */
@@ -242,35 +252,42 @@ void client_destroy(struct client *client, const char *reason)
 	}
 
 	if (client->anvil_sent) {
-		master_service_anvil_send(master_service, t_strconcat(
-			"DISCONNECT\t", my_pid, "\tsieve/",
-			mail_user_get_anvil_userip_ident(client->user),
-			"\n", NULL));
+		master_service_anvil_send(
+			master_service, t_strconcat(
+				"DISCONNECT\t", my_pid, "\tsieve/",
+				mail_user_get_anvil_userip_ident(client->user),
+				"\n", NULL));
 	}
-	mail_user_unref(&client->user);
 
 	managesieve_parser_destroy(&client->parser);
-	if (client->io != NULL)
-		io_remove(&client->io);
-	if (client->to_idle_output != NULL)
-		timeout_remove(&client->to_idle_output);
+	io_remove(&client->io);
+	timeout_remove(&client->to_idle_output);
 	timeout_remove(&client->to_idle);
 
-	i_stream_destroy(&client->input);
-	o_stream_destroy(&client->output);
-
+	/* i/ostreams are already closed at this stage, so fd can be closed */
 	net_disconnect(client->fd_in);
 	if (client->fd_in != client->fd_out)
 		net_disconnect(client->fd_out);
 
+	/* Free the user after client is already disconnected. It may start
+	   some background work like autoexpunging. */
+	mail_user_unref(&client->user);
+
+	/* free the i/ostreams after mail_user_unref(), which could trigger
+	   mail_storage_callbacks notifications that write to the ostream. */
+	i_stream_destroy(&client->input);
+	o_stream_destroy(&client->output);
+
 	sieve_storage_unref(&client->storage);
 	sieve_deinit(&client->svinst);
 
+	event_unref(&client->cmd.event);
 	pool_unref(&client->cmd.pool);
-	mail_storage_service_user_free(&client->service_user);
+	mail_storage_service_user_unref(&client->service_user);
 
 	managesieve_client_count--;
 	DLLIST_REMOVE(&managesieve_clients, client);
+	event_unref(&client->event);
 	pool_unref(&client->pool);
 
 	master_service_client_connection_destroyed(master_service);
@@ -284,12 +301,16 @@ static void client_destroy_timeout(struct client *client)
 
 void client_disconnect(struct client *client, const char *reason)
 {
-	i_assert(reason != NULL);
-
 	if (client->disconnected)
 		return;
 
-	i_info("Disconnected: %s %s", reason, client_stats(client));
+	if (reason == NULL) {
+		reason = io_stream_get_disconnect_reason(client->input,
+							 client->output);
+		i_info("%s %s", reason, client_stats(client));
+	} else {
+		i_info("Disconnected: %s %s", reason, client_stats(client));
+	}
 	client->disconnected = TRUE;
 	o_stream_flush(client->output);
 	o_stream_uncork(client->output);
@@ -297,9 +318,9 @@ void client_disconnect(struct client *client, const char *reason)
 	i_stream_close(client->input);
 	o_stream_close(client->output);
 
-	if (client->to_idle != NULL)
-		timeout_remove(&client->to_idle);
-	client->to_idle = timeout_add(0, client_destroy_timeout, client);
+	timeout_remove(&client->to_idle);
+	if (!client->destroyed)
+		client->to_idle = timeout_add(0, client_destroy_timeout, client);
 }
 
 void client_disconnect_with_error(struct client *client, const char *msg)
@@ -332,21 +353,21 @@ int client_send_line(struct client *client, const char *data)
 	return 1;
 }
 
-void client_send_response
-(struct client *client, const char *oknobye, const char *resp_code, const char *msg)
+void client_send_response(struct client *client, const char *oknobye,
+			  const char *resp_code, const char *msg)
 {
 	string_t *str;
 
 	str = t_str_new(128);
 	str_append(str, oknobye);
 
-	if ( resp_code != NULL ) {
+	if (resp_code != NULL) {
 		str_append(str, " (");
 		str_append(str, resp_code);
 		str_append_c(str, ')');
 	}
 
-	if ( msg != NULL ) {
+	if (msg != NULL) {
 		str_append_c(str, ' ');
 		managesieve_quote_append_string(str, msg, TRUE);
 	}
@@ -354,8 +375,17 @@ void client_send_response
 	client_send_line(client, str_c(str));
 }
 
-void client_send_command_error
-(struct client_command_context *cmd, const char *msg)
+struct event_passthrough *
+client_command_create_finish_event(struct client_command_context *cmd)
+{
+	struct event_passthrough *e =
+		event_create_passthrough(cmd->event)->
+		set_name("managesieve_command_finished");
+	return e;
+}
+
+void client_send_command_error(struct client_command_context *cmd,
+			       const char *msg)
 {
 	struct client *client = cmd->client;
 	const char *error, *cmd_name;
@@ -369,20 +399,20 @@ void client_send_command_error
 		}
 	}
 
-	if (cmd->name == NULL)
-		error = t_strconcat
-			("Error in MANAGESIEVE command: ", msg, NULL);
-	else {
+	if (cmd->name == NULL) {
+		error = t_strconcat("Error in MANAGESIEVE command: ",
+				    msg, NULL);
+	} else {
 		cmd_name = t_str_ucase(cmd->name);
-		error = t_strconcat
-			("Error in MANAGESIEVE command ", cmd_name, ": ", msg, NULL);
+		error = t_strconcat("Error in MANAGESIEVE command ",
+				    cmd_name, ": ", msg, NULL);
 	}
 
 	client_send_no(client, error);
 
 	if (++client->bad_counter >= CLIENT_MAX_BAD_COMMANDS) {
-		client_disconnect_with_error(client,
-			"Too many invalid MANAGESIEVE commands.");
+		client_disconnect_with_error(
+			client, "Too many invalid MANAGESIEVE commands.");
 	}
 
 	/* client_read_args() failures rely on this being set, so that the
@@ -391,65 +421,92 @@ void client_send_command_error
 	cmd->param_error = TRUE;
 }
 
-void client_send_storage_error
-(struct client *client, struct sieve_storage *storage)
+#undef client_command_storage_error
+void client_command_storage_error(struct client_command_context *cmd,
+				  const char *source_filename,
+				  unsigned int source_linenum,
+				  const char *log_prefix, ...)
 {
+	struct event_log_params params = {
+		.log_type = LOG_TYPE_INFO,
+		.source_filename = source_filename,
+		.source_linenum = source_linenum,
+	};
+	struct client *client = cmd->client;
+	struct sieve_storage *storage = client->storage;
 	enum sieve_error error_code;
 	const char *error;
+	va_list args;
 
 	error = sieve_storage_get_last_error(storage, &error_code);
 
-	switch ( error_code ) {
+	switch (error_code) {
 	case SIEVE_ERROR_TEMP_FAILURE:
 		client_send_noresp(client, "TRYLATER", error);
 		break;
-
 	case SIEVE_ERROR_NO_QUOTA:
 		client_send_noresp(client, "QUOTA", error);
 		break;
-
 	case SIEVE_ERROR_NOT_FOUND:
 		client_send_noresp(client, "NONEXISTENT", error);
 		break;
-
 	case SIEVE_ERROR_ACTIVE:
 		client_send_noresp(client, "ACTIVE", error);
 		break;
-
 	case SIEVE_ERROR_EXISTS:
 		client_send_noresp(client, "ALREADYEXISTS", error);
 		break;
-
 	case SIEVE_ERROR_NOT_POSSIBLE:
 	default:
 		client_send_no(client, error);
 		break;
 	}
+
+	struct event_passthrough *e =
+		client_command_create_finish_event(cmd)->
+		add_str("error", error);
+
+	va_start(args, log_prefix);
+	event_log(e->event(), &params, "%s: %s",
+		  t_strdup_vprintf(log_prefix, args), error);
+	va_end(args);
 }
 
 bool client_read_args(struct client_command_context *cmd, unsigned int count,
-	unsigned int flags, bool no_more, const struct managesieve_arg **args_r)
+		      unsigned int flags, bool no_more,
+		      const struct managesieve_arg **args_r)
 {
 	const struct managesieve_arg *dummy_args_r = NULL;
+	string_t *str;
 	int ret;
 
-	if ( args_r == NULL ) args_r = &dummy_args_r;
+	if (args_r == NULL)
+		args_r = &dummy_args_r;
 
 	i_assert(count <= INT_MAX);
 
-	ret = managesieve_parser_read_args
-		(cmd->client->parser, ( no_more ? 0 : count ), flags, args_r);
-	if ( ret >= 0 ) {
-		if ( count > 0 || no_more ) {
-			if ( ret < (int)count ) {
-				client_send_command_error(cmd, "Missing arguments.");
+	ret = managesieve_parser_read_args(cmd->client->parser,
+					   (no_more ? 0 : count),
+					   flags, args_r);
+	if (ret >= 0) {
+		if (count > 0 || no_more) {
+			if (ret < (int)count) {
+				client_send_command_error(
+					cmd, "Missing arguments.");
 				return FALSE;
-			} else if ( no_more && ret > (int)count ) {
-				client_send_command_error(cmd, "Too many arguments.");
+			} else if (no_more && ret > (int)count) {
+				client_send_command_error(
+					cmd, "Too many arguments.");
 				return FALSE;
 			}
 		}
 
+		str = t_str_new(256);
+		managesieve_write_args(str, *args_r);
+		cmd->args = p_strdup(cmd->pool, str_c(str));
+
+		event_add_str(cmd->event, "cmd_args", cmd->args);
+
 		/* all parameters read successfully */
 		return TRUE;
 	} else if (ret == -2) {
@@ -467,7 +524,7 @@ bool client_read_args(struct client_command_context *cmd, unsigned int count,
 }
 
 bool client_read_string_args(struct client_command_context *cmd,
-			     unsigned int count, bool no_more, ...)
+			     bool no_more, unsigned int count, ...)
 {
 	const struct managesieve_arg *msieve_args;
 	va_list va;
@@ -475,20 +532,20 @@ bool client_read_string_args(struct client_command_context *cmd,
 	unsigned int i;
 	bool result = TRUE;
 
-	if ( !client_read_args(cmd, count, 0, no_more, &msieve_args) )
+	if (!client_read_args(cmd, count, 0, no_more, &msieve_args))
 		return FALSE;
 
-	va_start(va, no_more);
-	for ( i = 0; i < count; i++ ) {
+	va_start(va, count);
+	for (i = 0; i < count; i++) {
 		const char **ret = va_arg(va, const char **);
 
-		if ( MANAGESIEVE_ARG_IS_EOL(&msieve_args[i]) ) {
+		if (MANAGESIEVE_ARG_IS_EOL(&msieve_args[i])) {
 			client_send_command_error(cmd, "Missing arguments.");
 			result = FALSE;
 			break;
 		}
 
-		if ( !managesieve_arg_get_string(&msieve_args[i], &str) ) {
+		if (!managesieve_arg_get_string(&msieve_args[i], &str)) {
 			client_send_command_error(cmd, "Invalid arguments.");
 			result = FALSE;
 			break;
@@ -507,31 +564,35 @@ void _client_reset_command(struct client *client)
 	pool_t pool;
 	size_t size;
 
-	/* reset input idle time because command output might have taken a
-	   long time and we don't want to disconnect client immediately then */
+	/* reset input idle time because command output might have taken a long
+	   time and we don't want to disconnect client immediately then */
 	client->last_input = ioloop_time;
 	timeout_reset(client->to_idle);
 
 	client->command_pending = FALSE;
-    if (client->io == NULL && !client->disconnected) {
-        i_assert(i_stream_get_fd(client->input) >= 0);
-        client->io = io_add(i_stream_get_fd(client->input),
-                    IO_READ, client_input, client);
-    }
-    o_stream_set_flush_callback(client->output, client_output, client);
+	if (client->io == NULL && !client->disconnected) {
+		i_assert(i_stream_get_fd(client->input) >= 0);
+		client->io = io_add(i_stream_get_fd(client->input),
+				    IO_READ, client_input, client);
+	}
+	o_stream_set_flush_callback(client->output, client_output, client);
+
+	event_unref(&client->cmd.event);
 
 	pool = client->cmd.pool;
-	memset(&client->cmd, 0, sizeof(client->cmd));
+	i_zero(&client->cmd);
 
 	p_clear(pool);
 	client->cmd.pool = pool;
 	client->cmd.client = client;
 
+	client->cmd.event = event_create(client->event);
+
 	managesieve_parser_reset(client->parser);
 
-	/* if there's unread data in buffer, remember that there's input
-	   pending and we should get around to calling client_input() soon.
-	   This is mostly for APPEND/IDLE. */
+	/* if there's unread data in buffer, remember that there's input pending
+	   and we should get around to calling client_input() soon. This is
+	   mostly for APPEND/IDLE. */
 	(void)i_stream_get_data(client->input, &size);
 	if (size > 0 && !client->destroyed)
 		client->input_pending = TRUE;
@@ -563,8 +624,14 @@ static bool client_handle_input(struct client_command_context *cmd)
 	struct client *client = cmd->client;
 
 	if (cmd->func != NULL) {
+		bool finished;
+
+		event_push_global(cmd->event);
+		finished = cmd->func(cmd);
+		event_pop_global(cmd->event);
+
 		/* command is being executed - continue it */
-		if (cmd->func(cmd) || cmd->param_error) {
+		if (finished || cmd->param_error) {
 			/* command execution was finished */
 			if (!cmd->param_error)
 				client->bad_counter = 0;
@@ -573,7 +640,7 @@ static bool client_handle_input(struct client_command_context *cmd)
 		}
 
 		/* unfinished */
-    if (client->command_pending)
+		if (client->command_pending)
 			o_stream_set_flush_pending(client->output, TRUE);
 		return FALSE;
 	}
@@ -597,15 +664,14 @@ static bool client_handle_input(struct client_command_context *cmd)
 		managesieve_refresh_proctitle();
 	}
 
-	if (cmd->name == '\0') {
+	if (cmd->name[0] == '\0') {
 		/* command not given - cmd_func is already NULL. */
 	} else {
 		/* find the command function */
 		struct command *command = command_find(cmd->name);
 
-		if (command != NULL) {
+		if (command != NULL)
 			cmd->func = command->func;
-		}
 	}
 
 	client->input_skip_line = TRUE;
@@ -616,6 +682,7 @@ static bool client_handle_input(struct client_command_context *cmd)
 	} else {
 		i_assert(!client->disconnected);
 
+		event_add_str(cmd->event, "cmd_name", t_str_ucase(cmd->name));
 		client_handle_input(cmd);
 	}
 
@@ -625,7 +692,7 @@ static bool client_handle_input(struct client_command_context *cmd)
 void client_input(struct client *client)
 {
 	struct client_command_context *cmd = &client->cmd;
-	int ret;
+	bool ret;
 
 	if (client->command_pending) {
 		/* already processing one command. wait. */
@@ -660,8 +727,8 @@ void client_input(struct client *client)
 			ret = client_handle_input(cmd);
 		} T_END;
 	} while (ret && !client->disconnected);
-    o_stream_uncork(client->output);
-    client->handling_input = FALSE;
+	o_stream_uncork(client->output);
+	client->handling_input = FALSE;
 
 	if (client->command_pending)
 		client->input_pending = TRUE;
@@ -677,9 +744,9 @@ int client_output(struct client *client)
 	bool finished;
 
 	client->last_output = ioloop_time;
-    timeout_reset(client->to_idle);
-    if (client->to_idle_output != NULL)
-        timeout_reset(client->to_idle_output);
+	timeout_reset(client->to_idle);
+	if (client->to_idle_output != NULL)
+		timeout_reset(client->to_idle_output);
 
 	if ((ret = o_stream_flush(client->output)) < 0) {
 		client_destroy(client, NULL);
@@ -716,6 +783,7 @@ int client_output(struct client *client)
 void clients_destroy_all(void)
 {
 	while (managesieve_clients != NULL) {
+		mail_storage_service_io_activate_user(managesieve_clients->service_user);
 		client_send_bye(managesieve_clients, "Server shutting down.");
 		client_destroy(managesieve_clients, "Server shutting down.");
 	}
