@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #ifndef __MANAGESIEVE_CLIENT_H
@@ -20,7 +20,7 @@ struct client_command_context {
 	command_func_t *func;
 	void *context;
 
-	unsigned int param_error:1;
+	bool param_error:1;
 };
 
 struct managesieve_module_register {
@@ -66,14 +66,14 @@ struct client {
 	unsigned int deleted_count;
 	unsigned int renamed_count;
 
-	unsigned int disconnected:1;
-	unsigned int destroyed:1;
-	unsigned int command_pending:1;
-	unsigned int input_pending:1;
-	unsigned int output_pending:1;
-	unsigned int handling_input:1;
-	unsigned int anvil_sent:1;
-	unsigned int input_skip_line:1; /* skip all the data until we've
+	bool disconnected:1;
+	bool destroyed:1;
+	bool command_pending:1;
+	bool input_pending:1;
+	bool output_pending:1;
+	bool handling_input:1;
+	bool anvil_sent:1;
+	bool input_skip_line:1; /* skip all the data until we've
 					   found a new line */
 };
 
@@ -131,7 +131,7 @@ bool client_read_args
 /* Reads a number of string arguments. ... is a list of pointers where to
    store the arguments. */
 bool client_read_string_args
-	(struct client_command_context *cmd, unsigned int count, bool no_more, ...);
+	(struct client_command_context *cmd, bool no_more, unsigned int count, ...);
 
 static inline bool client_read_no_args
 (struct client_command_context *cmd)
