@@ -1,8 +1,5 @@
-/* Copyright (c) 2002-2016 Pigeonhole authors, see the included COPYING file
- */
-
-#ifndef __MANAGESIEVE_SETTINGS_H
-#define __MANAGESIEVE_SETTINGS_H
+#ifndef MANAGESIEVE_SETTINGS_H
+#define MANAGESIEVE_SETTINGS_H
 
 struct mail_user_settings;
 
@@ -15,6 +12,7 @@ enum managesieve_client_workarounds {
 struct managesieve_settings {
 	bool mail_debug;
 	bool verbose_proctitle;
+	const char *rawlog_dir;
 
 	/* managesieve: */
 	unsigned int managesieve_max_line_length;
@@ -28,4 +26,4 @@ struct managesieve_settings {
 
 extern const struct setting_parser_info managesieve_setting_parser_info;
 
-#endif /* __MANAGESIEVE_SETTINGS_H */
+#endif
