@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -16,9 +16,9 @@
 #include "doveadm-sieve-plugin.h"
 
 #define SIEVE_MAIL_CONTEXT(obj) \
-	MODULE_CONTEXT(obj, sieve_storage_module)
+	MODULE_CONTEXT_REQUIRE(obj, sieve_storage_module)
 #define SIEVE_USER_CONTEXT(obj) \
-	MODULE_CONTEXT(obj, sieve_user_module)
+	MODULE_CONTEXT_REQUIRE(obj, sieve_user_module)
 
 struct sieve_mail_user {
 	union mail_user_module_context module_ctx;
@@ -79,8 +79,6 @@ mail_sieve_user_init
 		SIEVE_STORAGE_FLAG_SYNCHRONIZING;
 	struct sieve_environment svenv;
 
-	i_assert( suser != NULL );
-
 	if ( suser->svinst != NULL ) {
 		*svstorage_r = suser->sieve_storage;
 		return suser->sieve_storage != NULL ? 1 : 0;
@@ -253,7 +251,7 @@ sieve_attribute_set_default(struct mail_storage *storage,
 	if (value->value != NULL) {
 		type = value->value[0];
 	} else if (value->value_stream != NULL) {
-		ret = i_stream_read_data(value->value_stream, &data, &size, 0);
+		ret = i_stream_read_more(value->value_stream, &data, &size);
 		if (ret == -1) {
 			mail_storage_set_critical(storage, "read(%s) failed: %m",
 				i_stream_get_name(value->value_stream));
@@ -294,8 +292,7 @@ sieve_attribute_set_sieve(struct mail_storage *storage,
 
 	if (strcmp(key, MAILBOX_ATTRIBUTE_SIEVE_DEFAULT) == 0)
 		return sieve_attribute_set_default(storage, svstorage, value);
-	if (strncmp(key, MAILBOX_ATTRIBUTE_PREFIX_SIEVE_FILES,
-		    strlen(MAILBOX_ATTRIBUTE_PREFIX_SIEVE_FILES)) != 0) {
+	if (!str_begins(key, MAILBOX_ATTRIBUTE_PREFIX_SIEVE_FILES)) {
 		mail_storage_set_error(storage, MAIL_ERROR_NOTFOUND,
 				       "Nonexistent sieve attribute");
 		return -1;
@@ -328,7 +325,7 @@ sieve_attribute_set_sieve(struct mail_storage *storage,
 
 	ret = 0;
 	while (input->stream_errno == 0 &&
-		!i_stream_is_eof(input)) {
+		!i_stream_read_eof(input)) {
 		if (sieve_storage_save_continue(save_ctx) < 0) {
 			mail_storage_set_critical(storage,
 				"Failed to save sieve script '%s': %s", scriptname,
@@ -373,8 +370,7 @@ sieve_attribute_set(struct mailbox_transaction_context *t,
 
 	if (t->box->storage->user->dsyncing &&
 	    type == MAIL_ATTRIBUTE_TYPE_PRIVATE &&
-	    strncmp(key, MAILBOX_ATTRIBUTE_PREFIX_SIEVE,
-		    strlen(MAILBOX_ATTRIBUTE_PREFIX_SIEVE)) == 0) {
+	    str_begins(key, MAILBOX_ATTRIBUTE_PREFIX_SIEVE)) {
 		time_t ts =
 			(value->last_change != 0 ? value->last_change : ioloop_time);
 
@@ -520,8 +516,7 @@ sieve_attribute_get_sieve(struct mail_storage *storage, const char *key,
 
 	if (strcmp(key, MAILBOX_ATTRIBUTE_SIEVE_DEFAULT) == 0)
 		return sieve_attribute_get_default(storage, svstorage, value_r);
-	if (strncmp(key, MAILBOX_ATTRIBUTE_PREFIX_SIEVE_FILES,
-		    strlen(MAILBOX_ATTRIBUTE_PREFIX_SIEVE_FILES)) != 0)
+	if (!str_begins(key, MAILBOX_ATTRIBUTE_PREFIX_SIEVE_FILES))
 		return 0;
 	if ((value_r->flags & MAIL_ATTRIBUTE_VALUE_FLAG_INT_STREAMS) == 0) {
 		mail_storage_set_error(storage, MAIL_ERROR_PARAMS,
@@ -540,20 +535,19 @@ sieve_attribute_get_sieve(struct mail_storage *storage, const char *key,
 }
 
 static int
-sieve_attribute_get(struct mailbox_transaction_context *t,
+sieve_attribute_get(struct mailbox *box,
 		    enum mail_attribute_type type, const char *key,
 		    struct mail_attribute_value *value_r)
 {
-	union mailbox_module_context *sbox = SIEVE_MAIL_CONTEXT(t->box);
-	struct mail_user *user = t->box->storage->user;
+	union mailbox_module_context *sbox = SIEVE_MAIL_CONTEXT(box);
+	struct mail_user *user = box->storage->user;
 	int ret;
 
-	if (t->box->storage->user->dsyncing &&
+	if (box->storage->user->dsyncing &&
 	    type == MAIL_ATTRIBUTE_TYPE_PRIVATE &&
-	    strncmp(key, MAILBOX_ATTRIBUTE_PREFIX_SIEVE,
-		    strlen(MAILBOX_ATTRIBUTE_PREFIX_SIEVE)) == 0) {
+	    str_begins(key, MAILBOX_ATTRIBUTE_PREFIX_SIEVE)) {
 
-		ret = sieve_attribute_get_sieve(t->box->storage, key, value_r);
+		ret = sieve_attribute_get_sieve(box->storage, key, value_r);
 		if (ret >= 0 && user->mail_debug) {
 			struct tm *tm = localtime(&value_r->last_change);
 			char str[256];
@@ -573,7 +567,7 @@ sieve_attribute_get(struct mailbox_transaction_context *t,
 		}
 		return ret;
 	}
-	return sbox->super.attribute_get(t, type, key, value_r);
+	return sbox->super.attribute_get(box, type, key, value_r);
 }
 
 static int
@@ -614,8 +608,7 @@ sieve_attribute_iter_init(struct mailbox *box, enum mail_attribute_type type,
 
 	if (box->storage->user->dsyncing &&
 	    type == MAIL_ATTRIBUTE_TYPE_PRIVATE &&
-	    strncmp(prefix, MAILBOX_ATTRIBUTE_PREFIX_SIEVE,
-		    strlen(prefix)) == 0) {
+	    str_begins(prefix, MAILBOX_ATTRIBUTE_PREFIX_SIEVE)) {
 		if (sieve_attribute_iter_script_init(siter) < 0)
 			siter->failed = TRUE;
 	}
