@@ -1,4 +1,4 @@
-/* Copyright (c) 2016 Dovecot authors, see the included COPYING file */
+/* Copyright (c) 2016-2018 Pigeonhole authors, see the included COPYING file */
 
 #include "imap-common.h"
 #include "array.h"
@@ -14,8 +14,6 @@
 #include "imap-match.h"
 #include "imap-util.h"
 
-#include "strtrim.h"
-
 #include "imap-sieve.h"
 #include "imap-sieve-storage.h"
 
@@ -59,6 +57,7 @@ struct imap_sieve_mailbox_rule {
 	const char *from;
 	const char *const *causes;
 	const char *before, *after;
+	const char *copy_source_after;
 };
 
 struct imap_sieve_user {
@@ -71,12 +70,12 @@ struct imap_sieve_user {
 	HASH_TABLE_TYPE(imap_sieve_mailbox_rule) mbox_rules;
 	ARRAY_TYPE(imap_sieve_mailbox_rule) mbox_patterns;
 
-	unsigned int sieve_active:1;
-	unsigned int user_script:1;
+	bool sieve_active:1;
+	bool user_script:1;
 };
 
 struct imap_sieve_mailbox_event {
-	uint32_t mail_uid;
+	uint32_t dest_mail_uid, src_mail_uid;
 	unsigned int save_seq;
 
 	const char *changed_flags;
@@ -86,8 +85,9 @@ struct imap_sieve_mailbox_transaction {
 	pool_t pool;
 
 	union mailbox_transaction_module_context module_ctx;
-	struct mail *tmp_mail;
+
 	struct mailbox *src_box;
+	struct mailbox_transaction_context *src_mail_trans;
 
 	ARRAY_TYPE(imap_sieve_mailbox_event) events;
 };
@@ -165,6 +165,22 @@ imap_sieve_mailbox_debug(struct mailbox *box,
 	}
 }
 
+static inline void
+imap_sieve_mailbox_warning(struct mailbox *box,
+	const char *format, ...) ATTR_FORMAT(2, 3);
+static inline void
+imap_sieve_mailbox_warning(struct mailbox *box,
+	const char *format, ...)
+{
+	va_list args;
+
+	va_start(args, format);
+	i_warning("imapsieve: mailbox %s: %s",
+		mailbox_get_vname(box),
+		t_strdup_vprintf(format, args));
+	va_end(args);
+}
+
 static inline void
 imap_sieve_mailbox_error(struct mailbox *box,
 	const char *format, ...) ATTR_FORMAT(2, 3);
@@ -186,7 +202,7 @@ imap_sieve_mailbox_error(struct mailbox *box,
  */
 
 static int imap_sieve_mailbox_get_script_real
-(struct mailbox *box, struct mailbox_transaction_context *t,
+(struct mailbox *box,
 	const char **script_name_r)
 {
 	struct mail_user *user = box->storage->user;
@@ -196,17 +212,18 @@ static int imap_sieve_mailbox_get_script_real
 	*script_name_r = NULL;
 
 	/* get the name of the Sieve script from mailbox METADATA */
-	if ((ret=mailbox_attribute_get(t, MAIL_ATTRIBUTE_TYPE_SHARED,
+	if ((ret=mailbox_attribute_get(box, MAIL_ATTRIBUTE_TYPE_SHARED,
 			MAILBOX_ATTRIBUTE_IMAPSIEVE_SCRIPT, &value)) < 0) {
-		imap_sieve_mailbox_error(t->box,
+		imap_sieve_mailbox_error(box,
 			"Failed to read /shared/"
 			MAILBOX_ATTRIBUTE_IMAPSIEVE_SCRIPT" "
-			"mailbox attribute"); // FIXME: details?
+			"mailbox attribute: %s",
+			mailbox_get_last_error(box, NULL));
 		return -1;
 	}
 
 	if (ret > 0) {
-		imap_sieve_mailbox_debug(t->box,
+		imap_sieve_mailbox_debug(box,
 			"Mailbox attribute /shared/"
 			MAILBOX_ATTRIBUTE_IMAPSIEVE_SCRIPT" "
 			"points to Sieve script `%s'", value.value);
@@ -215,44 +232,40 @@ static int imap_sieve_mailbox_get_script_real
 	   server METADATA */
 	} else {
 		struct mail_namespace *ns;
-		struct mailbox *box;
-		struct mailbox_transaction_context *ibt;
+		struct mailbox *inbox;
 
-		imap_sieve_mailbox_debug(t->box,
+		imap_sieve_mailbox_debug(box,
 			"Mailbox attribute /shared/"
 			MAILBOX_ATTRIBUTE_IMAPSIEVE_SCRIPT" "
 			"not found");
 
 		ns = mail_namespace_find_inbox(user->namespaces);
-		box = mailbox_alloc(ns->list, "INBOX",
+		inbox = mailbox_alloc(ns->list, "INBOX",
 			MAILBOX_FLAG_READONLY);
-		if ((ret=mailbox_open(box)) >= 0) {
-			ibt = mailbox_transaction_begin
-				(box, MAILBOX_TRANSACTION_FLAG_EXTERNAL);
-			ret = mailbox_attribute_get(ibt,
-				MAIL_ATTRIBUTE_TYPE_SHARED,
-				MAILBOX_ATTRIBUTE_PREFIX_DOVECOT_PVT_SERVER
-				MAILBOX_ATTRIBUTE_IMAPSIEVE_SCRIPT, &value);
-			mailbox_transaction_rollback(&ibt);
-		}
-		mailbox_free(&box);
+		ret = mailbox_attribute_get(inbox,
+			MAIL_ATTRIBUTE_TYPE_SHARED,
+			MAILBOX_ATTRIBUTE_PREFIX_DOVECOT_PVT_SERVER
+			MAILBOX_ATTRIBUTE_IMAPSIEVE_SCRIPT, &value);
 
 		if (ret <= 0) {
 			if (ret < 0) {
-				imap_sieve_mailbox_error(t->box,
+				imap_sieve_mailbox_error(box,
 					"Failed to read /shared/"
 					MAIL_SERVER_ATTRIBUTE_IMAPSIEVE_SCRIPT" "
-					"server attribute"); // FIXME: details?
+					"server attribute: %s",
+					mailbox_get_last_error(inbox, NULL));
 			} else if (ret == 0) {
-				imap_sieve_mailbox_debug(t->box,
+				imap_sieve_mailbox_debug(box,
 					"Server attribute /shared/"
 					MAIL_SERVER_ATTRIBUTE_IMAPSIEVE_SCRIPT" "
 					"not found");
 			}
+			mailbox_free(&inbox);
 			return ret;
 		}
+		mailbox_free(&inbox);
 
-		imap_sieve_mailbox_debug(t->box,
+		imap_sieve_mailbox_debug(box,
 			"Server attribute /shared/"
 			MAIL_SERVER_ATTRIBUTE_IMAPSIEVE_SCRIPT" "
 			"points to Sieve script `%s'", value.value);
@@ -265,19 +278,32 @@ static int imap_sieve_mailbox_get_script_real
 static int imap_sieve_mailbox_get_script
 (struct mailbox *box, const char **script_name_r)
 {
-	struct mailbox_transaction_context *t;
 	int ret;
 
-	t = mailbox_transaction_begin(box, 0);
 	ret = imap_sieve_mailbox_get_script_real
-		(box, t, script_name_r);
-	mailbox_transaction_rollback(&t);
+		(box, script_name_r);
 	return ret;
 }
 
+static struct imap_sieve_mailbox_event *
+imap_sieve_create_mailbox_event
+(struct mailbox_transaction_context *t, struct mail *dest_mail)
+{
+	struct imap_sieve_mailbox_transaction *ismt = IMAP_SIEVE_CONTEXT(t);
+	struct imap_sieve_mailbox_event *event;
+
+	if (!array_is_created(&ismt->events))
+		i_array_init(&ismt->events, 64);
+
+	event = array_append_space(&ismt->events);
+	event->save_seq = t->save_count;
+	event->dest_mail_uid = dest_mail->uid;
+	return event;
+}
+
 static void imap_sieve_add_mailbox_event
 (struct mailbox_transaction_context *t,
-	struct mail *mail, struct mailbox *src_box,
+	struct mail *dest_mail, struct mailbox *src_box,
 	const char *changed_flags)
 {
 	struct imap_sieve_mailbox_transaction *ismt = IMAP_SIEVE_CONTEXT(t);
@@ -286,15 +312,28 @@ static void imap_sieve_add_mailbox_event
 	i_assert(ismt->src_box == NULL || ismt->src_box == src_box);
 	ismt->src_box = src_box;
 
-	if (!array_is_created(&ismt->events))
-		i_array_init(&ismt->events, 64);
-
-	event = array_append_space(&ismt->events);
-	event->save_seq = t->save_count;
-	event->mail_uid = mail->uid;
+	event = imap_sieve_create_mailbox_event(t, dest_mail);
 	event->changed_flags = p_strdup(ismt->pool, changed_flags);
 }
 
+static void imap_sieve_add_mailbox_copy_event
+(struct mailbox_transaction_context *t,
+	struct mail *dest_mail, struct mail *src_mail)
+{
+	struct imap_sieve_mailbox_transaction *ismt = IMAP_SIEVE_CONTEXT(t);
+	struct imap_sieve_mailbox_event *event;
+
+	i_assert(ismt->src_box == NULL || ismt->src_box == src_mail->box);
+	i_assert(ismt->src_mail_trans == NULL ||
+		ismt->src_mail_trans == src_mail->transaction);
+
+	ismt->src_box = src_mail->box;
+	ismt->src_mail_trans = src_mail->transaction;
+
+	event = imap_sieve_create_mailbox_event(t, dest_mail);
+	event->src_mail_uid = src_mail->uid;
+}
+
 /*
  * Mail
  */
@@ -325,7 +364,6 @@ imap_sieve_mail_update_keywords(struct mail *_mail,
 	enum modify_type modify_type, struct mail_keywords *keywords)
 {
 	struct mail_private *mail = (struct mail_private *)_mail;
-	struct mail_user *user = _mail->box->storage->user;
 	struct imap_sieve_mail *ismail = IMAP_SIEVE_MAIL_CONTEXT(mail);
 	const char *const *old_keywords, *const *new_keywords;
 	unsigned int i, j;
@@ -337,8 +375,6 @@ imap_sieve_mail_update_keywords(struct mail *_mail,
 	if (ismail->flags == NULL)
 		ismail->flags = str_new(default_pool, 64);
 
-	imap_sieve_debug(user, "Mail set keywords");
-
 	/* Removed flags */
 	for (i = 0; old_keywords[i] != NULL; i++) {
 		for (j = 0; new_keywords[j] != NULL; j++) {
@@ -404,10 +440,13 @@ static void imap_sieve_mail_allocated(struct mail *_mail)
 	struct mail_private *mail = (struct mail_private *)_mail;
 	struct imap_sieve_mailbox_transaction *ismt =
 		IMAP_SIEVE_CONTEXT(_mail->transaction);
+	struct mail_user *user = _mail->box->storage->user;
+	struct imap_sieve_user *isuser =
+		IMAP_SIEVE_USER_CONTEXT(user);
 	struct mail_vfuncs *v = mail->vlast;
 	struct imap_sieve_mail *ismail;
 
-	if (ismt == NULL)
+	if (ismt == NULL || isuser->sieve_active)
 		return;
 
 	ismail = p_new(mail->pool, struct imap_sieve_mail, 1);
@@ -438,57 +477,22 @@ imap_sieve_mailbox_copy(struct mail_save_context *ctx, struct mail *mail)
 	struct imap_sieve_mailbox_transaction *ismt =
 		IMAP_SIEVE_CONTEXT(t);
 
-	if (ismt != NULL) {
-		if (ctx->dest_mail == NULL) {
-			/* Dest mail is required for our purposes */
-			if (ismt->tmp_mail == NULL) {
-				ismt->tmp_mail = mail_alloc(t,
-					MAIL_FETCH_STREAM_HEADER |
-				  MAIL_FETCH_STREAM_BODY, NULL);
-			}
-			ctx->dest_mail = ismt->tmp_mail;
-		}
-	}
-
 	if (lbox->super.copy(ctx, mail) < 0)
 		return -1;
 
-	if (ismt != NULL && !ctx->dest_mail->expunged &&
+	if (ismt != NULL && !isuser->sieve_active &&
+		!ctx->dest_mail->expunged &&
 		(isuser->cur_cmd == IMAP_SIEVE_CMD_COPY ||
 			isuser->cur_cmd == IMAP_SIEVE_CMD_MOVE)) {
 		imap_sieve_mailbox_debug(t->box, "%s event",
 			(isuser->cur_cmd == IMAP_SIEVE_CMD_COPY ?
 				"COPY" : "MOVE"));
-		imap_sieve_add_mailbox_event
-			(t, ctx->dest_mail, mail->box, NULL);
+		imap_sieve_add_mailbox_copy_event(t, ctx->dest_mail, mail);
 	}
 
 	return 0;
 }
 
-static int
-imap_sieve_mailbox_save_begin(struct mail_save_context *ctx,
-	struct istream *input)
-{
-	struct imap_sieve_mailbox_transaction *ismt =
-		IMAP_SIEVE_CONTEXT(ctx->transaction);
-	union mailbox_module_context *lbox =
-		IMAP_SIEVE_CONTEXT(ctx->transaction->box);
-
-	if (ismt != NULL) {
-		if (ctx->dest_mail == NULL) {
-			/* Dest mail is required for our purposes */
-			if (ismt->tmp_mail == NULL) {
-				ismt->tmp_mail = mail_alloc(ctx->transaction,
-					MAIL_FETCH_STREAM_HEADER |
-				  MAIL_FETCH_STREAM_BODY, NULL);
-			}
-			ctx->dest_mail = ismt->tmp_mail;
-		}
-	}
-	return lbox->super.save_begin(ctx, input);
-}
-
 static int
 imap_sieve_mailbox_save_finish(struct mail_save_context *ctx)
 {
@@ -503,8 +507,8 @@ imap_sieve_mailbox_save_finish(struct mail_save_context *ctx)
 	if (lbox->super.save_finish(ctx) < 0)
 		return -1;
 
-	if (ismt != NULL && dest_mail != NULL &&
-		!dest_mail->expunged &&
+	if (ismt != NULL && !isuser->sieve_active &&
+		dest_mail != NULL && !dest_mail->expunged &&
 		isuser->cur_cmd == IMAP_SIEVE_CMD_APPEND) {
 
 		imap_sieve_mailbox_debug(t->box, "APPEND event");
@@ -519,7 +523,8 @@ imap_sieve_mailbox_save_finish(struct mail_save_context *ctx)
 
 static struct mailbox_transaction_context *
 imap_sieve_mailbox_transaction_begin(struct mailbox *box,
-			 enum mailbox_transaction_flags flags)
+			 enum mailbox_transaction_flags flags,
+			 const char *reason)
 {
 	union mailbox_module_context *lbox = IMAP_SIEVE_CONTEXT(box);
 	struct mail_user *user = box->storage->user;
@@ -529,9 +534,10 @@ imap_sieve_mailbox_transaction_begin(struct mailbox *box,
 	pool_t pool;
 
 	/* commence parent transaction */
-	t = lbox->super.transaction_begin(box, flags);
+	t = lbox->super.transaction_begin(box, flags, reason);
 
-	if (isuser == NULL || isuser->sieve_active)
+	if (isuser == NULL || isuser->sieve_active ||
+		isuser->cur_cmd == IMAP_SIEVE_CMD_NONE)
 		return t;
 
 	i_assert(isuser->client != NULL);
@@ -548,16 +554,53 @@ static void
 imap_sieve_mailbox_transaction_free
 (struct imap_sieve_mailbox_transaction *ismt)
 {
-	i_assert(ismt->tmp_mail == NULL);
 	if (array_is_created(&ismt->events))
 		array_free(&ismt->events);
 	pool_unref(&ismt->pool);
 }
 
+static void
+imap_sieve_mailbox_run_copy_source(
+	struct imap_sieve_mailbox_transaction *ismt,
+	struct imap_sieve_run *isrun,
+	const struct imap_sieve_mailbox_event *mevent,
+	struct mail **src_mail)
+{
+	struct mailbox *src_box = ismt->src_box;
+	int ret;
+
+	if (isrun == NULL)
+		return;
+
+	i_assert(ismt->src_mail_trans->box == src_box);
+
+	if (*src_mail == NULL)
+		*src_mail = mail_alloc(ismt->src_mail_trans, 0, NULL);
+
+	/* Select source message */
+	if (!mail_set_uid(*src_mail, mevent->src_mail_uid)) {
+		imap_sieve_mailbox_warning(src_box,
+			"Failed to find source message for Sieve event "
+			"(UID=%llu)", (unsigned long long)mevent->src_mail_uid);
+		return;
+	}
+
+	imap_sieve_mailbox_debug(src_box,
+		"Running copy_source_after scripts.");
+
+	/* Run scripts for source mail */
+	ret = imap_sieve_run_mail
+		(isrun, *src_mail, NULL);
+	if (ret > 0) {
+		/* Discard */
+		mail_update_flags(*src_mail, MODIFY_ADD, MAIL_DELETED);
+	}
+}
+
 static int
 imap_sieve_mailbox_transaction_run(
 	struct imap_sieve_mailbox_transaction *ismt,
-	struct mailbox *box,
+	struct mailbox *dest_box,
 	struct mail_transaction_commit_changes *changes)
 {
 	static const char *wanted_headers[] = {
@@ -565,17 +608,17 @@ imap_sieve_mailbox_transaction_run(
 		NULL
 	};
 	struct mailbox *src_box = ismt->src_box;
-	struct mail_user *user = box->storage->user;
+	struct mail_user *user = dest_box->storage->user;
 	struct imap_sieve_user *isuser = 	IMAP_SIEVE_USER_CONTEXT(user);
 	const struct imap_sieve_mailbox_event *mevent;
 	struct mailbox_header_lookup_ctx *headers_ctx;
 	struct mailbox_transaction_context *st;
 	struct mailbox *sbox;
-	struct imap_sieve_run *isrun;
+	struct imap_sieve_run *isrun, *isrun_src;
 	struct seq_range_iter siter;
 	const char *cause, *script_name = NULL;
 	bool can_discard;
-	struct mail *mail;
+	struct mail *mail, *src_mail = NULL;
 	int ret;
 
 	if (ismt == NULL || !array_is_created(&ismt->events)) {
@@ -587,15 +630,13 @@ imap_sieve_mailbox_transaction_run(
 
 	/* Get user script for this mailbox */
 	if (isuser->user_script && imap_sieve_mailbox_get_script
-		(box, &script_name) < 0) {
+		(dest_box, &script_name) < 0) {
 		return 0; // FIXME: some errors may warrant -1
 	}
 
 	/* Make sure IMAPSIEVE is initialized for this user */
-	if (isuser->isieve == NULL) {
-		isuser->isieve = imap_sieve_init
-			(user, isuser->client->lda_set);
-	}
+	if (isuser->isieve == NULL)
+		isuser->isieve = imap_sieve_init(isuser->client);
 
 	can_discard = FALSE;
 	switch (isuser->cur_cmd) {
@@ -620,16 +661,18 @@ imap_sieve_mailbox_transaction_run(
 	T_BEGIN {
 		ARRAY_TYPE(imap_sieve_mailbox_rule) mbrules;
 		ARRAY_TYPE(const_string) scripts_before, scripts_after;
+		ARRAY_TYPE(const_string) scripts_copy_source;
 		struct imap_sieve_mailbox_rule *const *rule_idx;
 
 		/* Find matching rules */
 		t_array_init(&mbrules, 16);
 		imap_sieve_mailbox_rules_get
-			(user, box, src_box, cause, &mbrules);
+			(user, dest_box, src_box, cause, &mbrules);
 
 		/* Apply all matched rules */
 		t_array_init(&scripts_before, 8);
 		t_array_init(&scripts_after, 8);
+		t_array_init(&scripts_copy_source, 4);
 		array_foreach(&mbrules, rule_idx) {
 			struct imap_sieve_mailbox_rule *rule = *rule_idx;
 
@@ -637,15 +680,32 @@ imap_sieve_mailbox_transaction_run(
 				array_append(&scripts_before, &rule->before, 1);
 			if (rule->after != NULL)
 				array_append(&scripts_after, &rule->after, 1);
+			if (rule->copy_source_after != NULL)
+				array_append(&scripts_copy_source, &rule->copy_source_after, 1);
 		}
 		(void)array_append_space(&scripts_before);
 		(void)array_append_space(&scripts_after);
 
 		/* Initialize */
 		ret = imap_sieve_run_init
-			(isuser->isieve, box, cause, script_name,
+			(isuser->isieve, dest_box, src_box, cause, script_name,
 				array_idx(&scripts_before, 0),
 				array_idx(&scripts_after, 0), &isrun);
+
+		/* Initialize source script execution */
+		isrun_src = NULL;
+		if (ret > 0 && ismt->src_mail_trans != NULL &&
+			isuser->cur_cmd == IMAP_SIEVE_CMD_COPY &&
+			array_count(&scripts_copy_source) > 0) {
+			const char *no_scripts = NULL;
+
+			(void)array_append_space(&scripts_copy_source);
+			if (imap_sieve_run_init(isuser->isieve,
+					dest_box, src_box, cause, NULL,
+					&no_scripts, array_idx(&scripts_copy_source, 0),
+					&isrun_src) <= 0)
+				isrun_src = NULL;
+		}
 	} T_END;
 
 	if (ret <= 0) {
@@ -653,15 +713,18 @@ imap_sieve_mailbox_transaction_run(
 		return 0;
 	}
 
-	/* Get synchronized view on the mailbox */
-	sbox = mailbox_alloc(box->list, box->vname, 0);
+	/* Get synchronized view on the destination mailbox */
+	sbox = mailbox_alloc(dest_box->list, dest_box->vname, 0);
 	if (mailbox_sync(sbox, 0) < 0) {
 		mailbox_free(&sbox);
+		imap_sieve_run_deinit(&isrun);
+		if (isrun_src != NULL)
+			imap_sieve_run_deinit(&isrun_src);
 		return -1;
 	}
 
 	/* Create transaction for event messages */
-	st = mailbox_transaction_begin(sbox, 0);
+	st = mailbox_transaction_begin(sbox, 0, __func__);
 	headers_ctx = mailbox_header_lookup_init(sbox, wanted_headers);
 	mail = mail_alloc(st, 0, headers_ctx);
 	mailbox_header_lookup_unref(&headers_ctx);
@@ -672,9 +735,9 @@ imap_sieve_mailbox_transaction_run(
 		uint32_t uid;
 
 		/* Determine UID for saved message */
-		if (mevent->mail_uid > 0 ||
+		if (mevent->dest_mail_uid > 0 ||
 			!seq_range_array_iter_nth(&siter, mevent->save_seq, &uid))
-			uid = mevent->mail_uid;
+			uid = mevent->dest_mail_uid;
 
 		/* Select event message */
 		if (!mail_set_uid(mail, uid)) {
@@ -693,16 +756,25 @@ imap_sieve_mailbox_transaction_run(
 		/* Handle the result */
 		if (ret < 0) {
 			/* Sieve error; keep */
-		} else if (ret > 0 && can_discard) {
-			/* Discard */
-			mail_update_flags(mail, MODIFY_ADD, MAIL_DELETED);
+		} else {
+			if (ret > 0 && can_discard) {
+				/* Discard */
+				mail_update_flags(mail, MODIFY_ADD, MAIL_DELETED);
+			}
+
+			imap_sieve_mailbox_run_copy_source
+				(ismt, isrun_src, mevent, &src_mail);
 		}
 	}
 
 	/* Cleanup */
 	mail_free(&mail);
 	ret = mailbox_transaction_commit(&st);
+	if (src_mail != NULL)
+		mail_free(&src_mail);
 	imap_sieve_run_deinit(&isrun);
+	if (isrun_src != NULL)
+		imap_sieve_run_deinit(&isrun_src);
 	mailbox_free(&sbox);
 	return ret;
 }
@@ -719,9 +791,6 @@ imap_sieve_mailbox_transaction_commit(
 	struct imap_sieve_user *isuser = 	IMAP_SIEVE_USER_CONTEXT(user);
 	int ret = 0;
 
-	if (ismt != NULL && ismt->tmp_mail != NULL)
-		mail_free(&ismt->tmp_mail);
-
 	if ((lbox->super.transaction_commit(t, changes_r)) < 0)
 		ret = -1;
 	else if (ismt != NULL) {
@@ -744,9 +813,6 @@ imap_sieve_mailbox_transaction_rollback(
 	struct imap_sieve_mailbox_transaction *ismt = IMAP_SIEVE_CONTEXT(t);
 	union mailbox_module_context *lbox = IMAP_SIEVE_CONTEXT(t->box);
 
-	if (ismt != NULL && ismt->tmp_mail != NULL)
-		mail_free(&ismt->tmp_mail);
-
 	lbox->super.transaction_rollback(t);
 
 	if (ismt != NULL)
@@ -769,7 +835,6 @@ static void imap_sieve_mailbox_allocated(struct mailbox *box)
 	box->vlast = &lbox->super;
 
 	v->copy = imap_sieve_mailbox_copy;
-	v->save_begin = imap_sieve_mailbox_save_begin;
 	v->save_finish = imap_sieve_mailbox_save_finish;
 	v->transaction_begin = imap_sieve_mailbox_transaction_begin;
 	v->transaction_commit = imap_sieve_mailbox_transaction_commit;
@@ -844,17 +909,23 @@ imap_sieve_mailbox_rules_init(struct mail_user *user)
 			(user, str_c(identifier));
 		if (setval == NULL || *setval == '\0')
 			break;
+		setval = t_str_trim(setval, "\t ");
+		if (strcasecmp(setval, "INBOX") == 0)
+			setval = t_str_ucase(setval);
 
 		mbrule = p_new(user->pool,
 			struct imap_sieve_mailbox_rule, 1);
 		mbrule->index = i;
-		mbrule->mailbox = ph_p_str_trim(user->pool, setval, "\t ");
+		mbrule->mailbox = p_strdup(user->pool, setval);
 
 		str_truncate(identifier, id_len);
 		str_append(identifier, "_from");
 		setval = mail_user_plugin_getenv(user, str_c(identifier));
 		if (setval != NULL && *setval != '\0') {
-			mbrule->from = ph_p_str_trim(user->pool, setval, "\t ");
+			setval = t_str_trim(setval, "\t ");
+			if (strcasecmp(setval, "INBOX") == 0)
+				setval = t_str_ucase(setval);
+			mbrule->from = p_strdup(user->pool, setval);
 			if (strcmp(mbrule->from, "*") == 0)
 				mbrule->from = NULL;
 		}
@@ -901,17 +972,25 @@ imap_sieve_mailbox_rules_init(struct mail_user *user)
 		setval = mail_user_plugin_getenv(user, str_c(identifier));
 		mbrule->after = p_strdup_empty(user->pool, setval);
 
+		str_truncate(identifier, id_len);
+		str_append(identifier, "_copy_source_after");
+		setval = mail_user_plugin_getenv(user, str_c(identifier));
+		mbrule->copy_source_after = p_strdup_empty(user->pool, setval);
+
 		if (user->mail_debug) {
 			imap_sieve_debug(user, "Static mailbox rule [%u]: "
 				"mailbox=`%s' from=`%s' causes=(%s) => "
-				"before=%s after=%s",
+				"before=%s after=%s%s",
 				mbrule->index, mbrule->mailbox,
 				(mbrule->from == NULL ? "*" : mbrule->from),
 				t_strarray_join(mbrule->causes, " "),
 				(mbrule->before == NULL ? "(none)" :
 					t_strconcat("`", mbrule->before, "'", NULL)),
 				(mbrule->after == NULL ? "(none)" :
-					t_strconcat("`", mbrule->after, "'", NULL)));
+					t_strconcat("`", mbrule->after, "'", NULL)),
+				(mbrule->copy_source_after == NULL ? "":
+					t_strconcat(" copy_source_after=`",
+						mbrule->copy_source_after, "'", NULL)));
 		}
 
 		if ((strcmp(mbrule->mailbox, "*") == 0 ||
@@ -934,7 +1013,7 @@ imap_sieve_mailbox_rule_match_cause
 {
 	const char *const *cp;
 
-	if (rule->causes == NULL || *rule->causes == '\0')
+	if (rule->causes == NULL || *rule->causes == NULL)
 		return TRUE;
 
 	for (cp = rule->causes; *cp != NULL; cp++) {
@@ -1002,7 +1081,7 @@ imap_sieve_mailbox_rules_match(struct mail_user *user,
 	struct imap_sieve_mailbox_rule lookup_rule;
 	struct imap_sieve_mailbox_rule *rule;
 
-	memset(&lookup_rule, 0, sizeof(lookup_rule));
+	i_zero(&lookup_rule);
 	lookup_rule.mailbox = dst_box;
 	lookup_rule.from = src_box;
 	rule = hash_table_lookup(isuser->mbox_rules, &lookup_rule);
