@@ -1,10 +1,7 @@
-/* Copyright (c) 2016-2017 Pigeonhole authors, see the included COPYING file
- */
+#ifndef IMAP_SIEVE_H
+#define IMAP_SIEVE_H
 
-#ifndef __IMAP_SIEVE_H
-#define __IMAP_SIEVE_H
-
-struct lda_settings;
+struct client;
 
 /*
  * IMAP event
@@ -36,8 +33,7 @@ imap_sieve_event_cause_valid(const char *cause)
 
 struct imap_sieve;
 
-struct imap_sieve *imap_sieve_init(struct mail_user *user,
-	const struct lda_settings *lda_set);
+struct imap_sieve *imap_sieve_init(struct client *client);
 void imap_sieve_deinit(struct imap_sieve **_isieve);
 
 /*
@@ -46,17 +42,16 @@ void imap_sieve_deinit(struct imap_sieve **_isieve);
 
 struct imap_sieve_run;
 
-int imap_sieve_run_init(struct imap_sieve *isieve,
-	struct mailbox *dest_mailbox, struct mailbox *src_mailbox,
-	const char *cause, const char *script_name,
-	const char *const *scripts_before,
-	const char *const *scripts_after,
-	struct imap_sieve_run **isrun_r)
-	ATTR_NULL(4, 5, 6);
+int imap_sieve_run_init(struct imap_sieve *isieve, struct mailbox *dest_mailbox,
+			struct mailbox *src_mailbox, const char *cause,
+			const char *script_name,
+			const char *const *scripts_before,
+			const char *const *scripts_after,
+			struct imap_sieve_run **isrun_r)
+			ATTR_NULL(4, 5, 6);
 
-int imap_sieve_run_mail
-(struct imap_sieve_run *isrun, struct mail *mail,
-	const char *changed_flags);
+int imap_sieve_run_mail(struct imap_sieve_run *isrun, struct mail *mail,
+			const char *changed_flags);
 
 void imap_sieve_run_deinit(struct imap_sieve_run **_isrun);
 
