@@ -1,8 +1,5 @@
-/* Copyright (c) 2016 Pigeonhole authors, see the included COPYING file
- */
-
-#ifndef __SIEVE_IMAPSIEVE_PLUGIN_H
-#define __SIEVE_IMAPSIEVE_PLUGIN_H
+#ifndef SIEVE_IMAPSIEVE_PLUGIN_H
+#define SIEVE_IMAPSIEVE_PLUGIN_H
 
 /*
  * Plugin interface
@@ -20,4 +17,4 @@ void sieve_imapsieve_plugin_unload
 void sieve_imapsieve_plugin_init(void);
 void sieve_imapsieve_plugin_deinit(void);
 
-#endif /* __SIEVE_IMAPSIEVE_PLUGIN_H */
+#endif
