@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -12,7 +12,6 @@
 #include "mail-namespace.h"
 #include "mail-storage.h"
 #include "mail-search-build.h"
-#include "imap-utf7.h"
 
 #include "sieve.h"
 #include "sieve-extensions.h"
@@ -41,10 +40,10 @@ static void print_help(void)
 }
 
 enum sieve_filter_discard_action {
-	SIEVE_FILTER_DACT_KEEP,        /* Keep discarded messages in source folder */
-	SIEVE_FILTER_DACT_MOVE,        /* Move discarded messages to Trash folder */
-	SIEVE_FILTER_DACT_DELETE,      /* Flag discarded messages as \DELETED */
-	SIEVE_FILTER_DACT_EXPUNGE      /* Expunge discarded messages */
+	SIEVE_FILTER_DACT_KEEP,   /* Keep discarded messages in source folder */
+	SIEVE_FILTER_DACT_MOVE,   /* Move discarded messages to Trash folder */
+	SIEVE_FILTER_DACT_DELETE, /* Flag discarded messages as \DELETED */
+	SIEVE_FILTER_DACT_EXPUNGE /* Expunge discarded messages */
 };
 
 struct sieve_filter_data {
@@ -55,9 +54,9 @@ struct sieve_filter_data {
 	struct sieve_binary *main_sbin;
 	struct sieve_error_handler *ehandler;
 
-	unsigned int execute:1;
-	unsigned int source_write:1;
-	unsigned int default_move:1;
+	bool execute:1;
+	bool source_write:1;
+	bool default_move:1;
 };
 
 struct sieve_filter_context {
@@ -68,147 +67,163 @@ struct sieve_filter_context {
 	struct ostream *teststream;
 };
 
-static int filter_message
-(struct sieve_filter_context *sfctx, struct mail *mail)
+static const char *
+result_amend_log_message(const struct sieve_script_env *senv,
+			 enum log_type log_type, const char *message)
+{
+	const struct sieve_message_data *msgdata = senv->script_context;
+	string_t *str;
+
+	if (log_type == LOG_TYPE_DEBUG)
+		return message;
+
+	str = t_str_new(256);
+	str_printfa(str, "msgid=%s", (msgdata->id == NULL ?
+				      "unspecified" : msgdata->id));
+	str_append(str, ": ");
+	str_append(str, message);
+
+	return str_c(str);
+}
+
+static int filter_message(struct sieve_filter_context *sfctx, struct mail *mail)
 {
 	struct sieve_error_handler *ehandler = sfctx->data->ehandler;
+	enum sieve_execute_flags exflags = SIEVE_EXECUTE_FLAG_LOG_RESULT;
 	struct sieve_script_env *senv = sfctx->data->senv;
 	struct sieve_exec_status estatus;
 	struct sieve_binary *sbin;
 	struct sieve_message_data msgdata;
-	const char *recipient = NULL, *sender = NULL;
 	bool execute = sfctx->data->execute;
 	bool source_write = sfctx->data->source_write;
 	const char *subject, *date;
 	uoff_t size = 0;
 	int ret;
 
-	sieve_tool_get_envelope_data(mail, &recipient, &sender);
-
 	/* Initialize execution status */
-	memset(&estatus, 0, sizeof(estatus));
+	i_zero(&estatus);
 	senv->exec_status = &estatus;
 
 	/* Collect necessary message data */
-	memset(&msgdata, 0, sizeof(msgdata));
+	i_zero(&msgdata);
 	msgdata.mail = mail;
-	msgdata.return_path = sender;
-	msgdata.orig_envelope_to = recipient;
-	msgdata.final_envelope_to = recipient;
 	msgdata.auth_user = senv->user->username;
-	(void)mail_get_first_header(mail, "Message-ID", &msgdata.id);
+	(void)mail_get_message_id(mail, &msgdata.id);
+	senv->script_context = &msgdata;
 
-	if ( mail_get_virtual_size(mail, &size) < 0 ) {
-		if ( mail->expunged )
+	sieve_tool_get_envelope_data(&msgdata, mail, NULL, NULL, NULL);
+
+	if (mail_get_virtual_size(mail, &size) < 0) {
+		if (mail->expunged)
 			return 1;
 
 		sieve_error(ehandler, NULL, "failed to obtain message size; "
-			"skipping this message (id=%s)",
-			( msgdata.id == NULL ? "none" : msgdata.id ));
+			    "skipping this message (id=%s)",
+			    (msgdata.id == NULL ? "none" : msgdata.id));
 		return 0;
 	}
 
-	if ( mail_get_first_header(mail, "date", &date) <= 0 )
+	if (mail_get_first_header(mail, "date", &date) <= 0)
 		date = "";
-	if ( mail_get_first_header(mail, "subject", &subject) <= 0 )
+	if (mail_get_first_header(mail, "subject", &subject) <= 0)
 		subject = "";
 
 	/* Single script */
 	sbin = sfctx->data->main_sbin;
 
 	/* Execute script */
-	if ( execute ) {
-		struct sieve_error_handler *action_ehandler;
-
+	if (execute) {
 		sieve_info(ehandler, NULL,
-			"filtering: [%s; %"PRIuUOFF_T" bytes] `%s'", date, size,
-			str_sanitize(subject, 40));
-
-		action_ehandler = sieve_prefix_ehandler_create
-			(ehandler, NULL, t_strdup_printf("msgid=%s",
-				( msgdata.id == NULL ? "unspecified" : msgdata.id )));
-		ret = sieve_execute(sbin, &msgdata, senv,
-				ehandler, action_ehandler, 0, NULL);
-		sieve_error_handler_unref(&action_ehandler);
+			   "filtering: [%s; %"PRIuUOFF_T" bytes] `%s'",
+			   date, size, str_sanitize(subject, 40));
 
+		ret = sieve_execute(sbin, &msgdata, senv, ehandler, ehandler,
+				    exflags);
 	} else {
-		(void)o_stream_send_str(sfctx->teststream,
+		o_stream_nsend_str(
+			sfctx->teststream,
 			t_strdup_printf(">> Filtering message:\n\n"
-				"  ID:      %s\n"
-			  "  Date:    %s\n"
-        "  Size:    %"PRIuUOFF_T" bytes\n"
-				"  Subject: %s\n", ( msgdata.id == NULL ? "none" : msgdata.id ),
-				date, size, str_sanitize(subject, 40)));
+					"  ID:      %s\n"
+					"  Date:    %s\n"
+					"  Size:    %"PRIuUOFF_T" bytes\n"
+					"  Subject: %s\n",
+					(msgdata.id == NULL ?
+					 "none" : msgdata.id), date, size,
+					str_sanitize(subject, 40)));
 
-		ret = sieve_test
-			(sbin, &msgdata, senv, ehandler, sfctx->teststream, 0, NULL);
+		ret = sieve_test(sbin, &msgdata, senv, ehandler,
+				 sfctx->teststream, exflags);
 	}
 
 	/* Handle message in source folder */
-	if ( ret > 0 ) {
+	if (ret > 0) {
 		struct mailbox *move_box = sfctx->data->move_mailbox;
 		enum sieve_filter_discard_action discard_action =
 			sfctx->data->discard_action;
 
-		if ( !source_write ) {
+		if (!source_write) {
 			/* READ-ONLY; Do nothing */
-
-		} else if ( estatus.keep_original  ) {
+		} else if (estatus.keep_original) {
 			/* Explicitly `stored' in source box; just keep it there */
-			sieve_info(ehandler, NULL, "message kept in source mailbox");
-
-		} else if ( estatus.message_saved ) {
 			sieve_info(ehandler, NULL,
+				   "message kept in source mailbox");
+		} else if (estatus.message_saved) {
+			sieve_info(
+				ehandler, NULL,
 				"message expunged from source mailbox upon successful move");
 
-			if ( execute )
+			if (execute)
 				mail_expunge(mail);
-
 		} else {
-
-			switch ( discard_action ) {
+			switch (discard_action) {
 			/* Leave it there */
 			case SIEVE_FILTER_DACT_KEEP:
-				sieve_info(ehandler, NULL, "message left in source mailbox");
+				sieve_info(ehandler, NULL,
+					   "message left in source mailbox");
 				break;
 			/* Move message to indicated folder */
 			case SIEVE_FILTER_DACT_MOVE:
-				sieve_info(ehandler, NULL,
+				sieve_info(
+					ehandler, NULL,
 					"message in source mailbox moved to mailbox '%s'",
 					mailbox_get_name(move_box));
 
-				if ( execute && move_box != NULL ) {
-					struct mailbox_transaction_context *t = sfctx->move_trans;
+				if (execute && move_box != NULL) {
+					struct mailbox_transaction_context *t =
+						sfctx->move_trans;
 					struct mail_save_context *save_ctx;
 
 					save_ctx = mailbox_save_alloc(t);
 
-					if ( mailbox_copy(&save_ctx, mail) < 0 ) {
+					if (mailbox_copy(&save_ctx, mail) < 0) {
 						enum mail_error error;
 						const char *errstr;
 
-						errstr = mail_storage_get_last_error
-							(mailbox_get_storage(move_box), &error);
+						errstr = mail_storage_get_last_error(
+							mailbox_get_storage(move_box), &error);
 
-						sieve_error(ehandler, NULL,
+						sieve_error(
+							ehandler, NULL,
 							"failed to move message to mailbox %s: %s",
 							mailbox_get_name(move_box), errstr);
 						return -1;
-				    }
-
+					}
 					mail_expunge(mail);
 				}
 				break;
 			/* Flag message as \DELETED */
 			case SIEVE_FILTER_DACT_DELETE:
-				sieve_info(ehandler, NULL, "message flagged as deleted in source mailbox");
-				if ( execute )
+				sieve_info(
+					ehandler, NULL,
+					"message flagged as deleted in source mailbox");
+				if (execute)
 					mail_update_flags(mail, MODIFY_ADD, MAIL_DELETED);
 				break;
 			/* Expunge the message immediately */
 			case SIEVE_FILTER_DACT_EXPUNGE:
-				sieve_info(ehandler, NULL, "message expunged from source mailbox");
-				if ( execute )
+				sieve_info(ehandler, NULL,
+					   "message expunged from source mailbox");
+				if (execute)
 					mail_expunge(mail);
 				break;
 			/* Unknown */
@@ -219,19 +234,24 @@ static int filter_message
 		}
 	}
 
-	switch ( ret ) {
+	switch (ret) {
 	case SIEVE_EXEC_OK:
 		break;
+	case SIEVE_EXEC_RESOURCE_LIMIT:
+		sieve_error(ehandler, NULL, "sieve resource limit exceeded");
+		return -1;
 	case SIEVE_EXEC_BIN_CORRUPT:
 		sieve_error(ehandler, NULL, "sieve script binary is corrupt");
 		return -1;
 	case SIEVE_EXEC_FAILURE:
 	case SIEVE_EXEC_TEMP_FAILURE:
-		if ( source_write && execute && sfctx->data->default_move &&
-			!estatus.keep_original &&	estatus.message_saved ) {
-			/* The implicit keep action moved message to default mailbox, so
-			   the source message still needs to be expunged */
-			sieve_error(ehandler, NULL,
+		if (source_write && execute && sfctx->data->default_move &&
+		    !estatus.keep_original && estatus.message_saved) {
+			/* The implicit keep action moved message to default
+			   mailbox, so the source message still needs to be
+			   expunged */
+			sieve_error(
+				ehandler, NULL,
 				"sieve script execution failed for this message; "
 				"message moved to default mailbox");
 			mail_expunge(mail);
@@ -239,7 +259,8 @@ static int filter_message
 		}
 		/* Fall through */
 	case SIEVE_EXEC_KEEP_FAILED:
-		sieve_error(ehandler, NULL,
+		sieve_error(
+			ehandler, NULL,
 			"sieve script execution failed for this message; "
 			"message left in source mailbox");
 		return 0;
@@ -249,8 +270,9 @@ static int filter_message
 }
 
 /* FIXME: introduce this into Dovecot */
-static void mail_search_build_add_flags
-(struct mail_search_args *args, enum mail_flags flags, bool not)
+static void
+mail_search_build_add_flags(struct mail_search_args *args,
+			    enum mail_flags flags, bool not)
 {
 	struct mail_search_arg *arg;
 
@@ -263,8 +285,8 @@ static void mail_search_build_add_flags
 	args->args = arg;
 }
 
-static int filter_mailbox
-(const struct sieve_filter_data *sfdata, struct mailbox *src_box)
+static int
+filter_mailbox(const struct sieve_filter_data *sfdata, struct mailbox *src_box)
 {
 	struct sieve_filter_context sfctx;
 	struct mailbox *move_box = sfdata->move_mailbox;
@@ -277,25 +299,28 @@ static int filter_mailbox
 
 	/* Sync source mailbox */
 
-	if ( mailbox_sync(src_box, MAILBOX_SYNC_FLAG_FULL_READ) < 0 ) {
+	if (mailbox_sync(src_box, MAILBOX_SYNC_FLAG_FULL_READ) < 0) {
 		sieve_error(ehandler, NULL, "failed to sync source mailbox");
 		return -1;
 	}
 
 	/* Initialize */
 
-	memset(&sfctx, 0, sizeof(sfctx));
+	i_zero(&sfctx);
 	sfctx.data = sfdata;
 
 	/* Create test stream */
-	if ( !sfdata->execute )
-		sfctx.teststream = o_stream_create_fd(1, 0, FALSE);
+	if (!sfdata->execute) {
+		sfctx.teststream = o_stream_create_fd(1, 0);
+		o_stream_set_no_error_handling(sfctx.teststream, TRUE);
+	}
 
 	/* Start move mailbox transaction */
 
-	if ( move_box != NULL ) {
-		sfctx.move_trans = mailbox_transaction_begin
-			(move_box, MAILBOX_TRANSACTION_FLAG_EXTERNAL);
+	if (move_box != NULL) {
+		sfctx.move_trans = mailbox_transaction_begin(
+			move_box, MAILBOX_TRANSACTION_FLAG_EXTERNAL,
+			"sieve_filter_data move_box");
 	}
 
 	/* Search non-deleted messages in the source folder */
@@ -303,42 +328,40 @@ static int filter_mailbox
 	search_args = mail_search_build_init();
 	mail_search_build_add_flags(search_args, MAIL_DELETED, TRUE);
 
-	t = mailbox_transaction_begin(src_box, 0);
+	t = mailbox_transaction_begin(src_box, 0,
+				      "sieve_filter_data src_box");
 	search_ctx = mailbox_search_init(t, search_args, NULL, 0, NULL);
 	mail_search_args_unref(&search_args);
 
 	/* Iterate through all requested messages */
 
-	while ( ret >= 0 && mailbox_search_next(search_ctx, &mail) > 0 ) {
+	while (ret >= 0 && mailbox_search_next(search_ctx, &mail))
 		ret = filter_message(&sfctx, mail);
-	}
 
 	/* Cleanup */
 
-	if ( mailbox_search_deinit(&search_ctx) < 0 ) {
+	if (mailbox_search_deinit(&search_ctx) < 0)
 		ret = -1;
-	}
 
-	if ( sfctx.move_trans != NULL ) {
-		if ( mailbox_transaction_commit(&sfctx.move_trans) < 0 ) {
+	if (sfctx.move_trans != NULL) {
+		if (mailbox_transaction_commit(&sfctx.move_trans) < 0)
 			ret = -1;
-		}
 	}
 
-	if ( mailbox_transaction_commit(&t) < 0 ) {
+	if (mailbox_transaction_commit(&t) < 0)
 		ret = -1;
-	}
 
-	if ( sfctx.teststream != NULL )
+	if (sfctx.teststream != NULL)
 		o_stream_destroy(&sfctx.teststream);
 
-	if ( ret < 0 ) return ret;
+	if (ret < 0) return ret;
 
 	/* Sync mailbox */
 
-	if ( sfdata->execute ) {
-		if ( mailbox_sync(src_box, MAILBOX_SYNC_FLAG_FULL_WRITE) < 0 ) {
-			sieve_error(ehandler, NULL, "failed to sync source mailbox");
+	if (sfdata->execute) {
+		if (mailbox_sync(src_box, MAILBOX_SYNC_FLAG_FULL_WRITE) < 0) {
+			sieve_error(ehandler, NULL,
+				    "failed to sync source mailbox");
 			return -1;
 		}
 	}
@@ -346,16 +369,6 @@ static int filter_mailbox
 	return ret;
 }
 
-static const char *mailbox_name_to_mutf7(const char *mailbox_utf8)
-{
-	string_t *str = t_str_new(128);
-
-	if (imap_utf8_to_utf7(mailbox_utf8, str) < 0)
-		return mailbox_utf8;
-	else
-		return str_c(str);
-}
-
 /*
  * Tool implementation
  */
@@ -363,10 +376,11 @@ static const char *mailbox_name_to_mutf7(const char *mailbox_utf8)
 int main(int argc, char **argv)
 {
 	struct sieve_instance *svinst;
-	ARRAY_TYPE (const_string) scriptfiles;
+	ARRAY_TYPE(const_string) scriptfiles;
 	const char *scriptfile,	*src_mailbox, *dst_mailbox, *move_mailbox;
 	struct sieve_filter_data sfdata;
-	enum sieve_filter_discard_action discard_action = SIEVE_FILTER_DACT_KEEP;
+	enum sieve_filter_discard_action discard_action =
+		SIEVE_FILTER_DACT_KEEP;
 	struct mail_user *mail_user;
 	struct sieve_binary *main_sbin;
 	struct sieve_script_env scriptenv;
@@ -376,10 +390,11 @@ int main(int argc, char **argv)
 	struct mailbox *src_box = NULL, *move_box = NULL;
 	enum mailbox_flags open_flags = MAILBOX_FLAG_IGNORE_ACLS;
 	enum mail_error error;
+	const char *errstr;
 	int c;
 
 	sieve_tool = sieve_tool_init("sieve-filter", &argc, &argv,
-		"m:s:x:P:u:q:Q:DCevW", FALSE);
+				     "m:s:x:P:u:q:Q:DCevW", FALSE);
 
 	t_array_init(&scriptfiles, 16);
 
@@ -402,16 +417,19 @@ int main(int argc, char **argv)
 				array_append(&scriptfiles, &file, 1);
 
 				/* FIXME: */
-				i_fatal_status(EX_USAGE,
+				i_fatal_status(
+					EX_USAGE,
 					"The -s argument is currently NOT IMPLEMENTED");
 			}
 			break;
 		case 'q':
-			i_fatal_status(EX_USAGE,
+			i_fatal_status(
+				EX_USAGE,
 				"The -q argument is currently NOT IMPLEMENTED");
 			break;
 		case 'Q':
-			i_fatal_status(EX_USAGE,
+			i_fatal_status(
+				EX_USAGE,
 				"The -Q argument is currently NOT IMPLEMENTED");
 			break;
 		case 'e':
@@ -439,53 +457,55 @@ int main(int argc, char **argv)
 	}
 
 	/* Script file argument */
-	if ( optind < argc ) {
+	if (optind < argc)
 		scriptfile = t_strdup(argv[optind++]);
-	} else {
+	else {
 		print_help();
 		i_fatal_status(EX_USAGE, "Missing <script-file> argument");
 	}
 
 	/* Source mailbox argument */
-	if ( optind < argc ) {
+	if (optind < argc)
 		src_mailbox = t_strdup(argv[optind++]);
-	} else {
+	else {
 		print_help();
 		i_fatal_status(EX_USAGE, "Missing <source-mailbox> argument");
 	}
 
 	/* Source action argument */
-	if ( optind < argc ) {
+	if (optind < argc) {
 		const char *srcact = argv[optind++];
 
-		if ( strcmp(srcact, "keep") == 0 ) {
+		if (strcmp(srcact, "keep") == 0) {
 			discard_action = SIEVE_FILTER_DACT_KEEP;
-		} else if ( strcmp(srcact, "move") == 0 ) {
+		} else if (strcmp(srcact, "move") == 0) {
 			discard_action = SIEVE_FILTER_DACT_MOVE;
-			if ( optind < argc ) {
+			if (optind < argc)
 				move_mailbox = t_strdup(argv[optind++]);
-			} else {
+			else {
 				print_help();
-				i_fatal_status(EX_USAGE,
+				i_fatal_status(
+					EX_USAGE,
 					"Invalid <discard-action> argument: "
 					"the `move' action requires mailbox argument");
 			}
-		} else if ( strcmp(srcact, "delete") == 0 ) {
+		} else if (strcmp(srcact, "delete") == 0) {
 			discard_action = SIEVE_FILTER_DACT_DELETE;
-		} else if ( strcmp(srcact, "expunge") == 0 ) {
+		} else if (strcmp(srcact, "expunge") == 0) {
 			discard_action = SIEVE_FILTER_DACT_EXPUNGE;
 		} else {
 			print_help();
-			i_fatal_status(EX_USAGE, "Invalid <discard-action> argument");
+			i_fatal_status(EX_USAGE,
+				       "Invalid <discard-action> argument");
 		}
 	}
 
-	if ( optind != argc ) {
+	if (optind != argc) {
 		print_help();
 		i_fatal_status(EX_USAGE, "Unknown argument: %s", argv[optind]);
 	}
 
-	if ( dst_mailbox == NULL ) {
+	if (dst_mailbox == NULL) {
 		dst_mailbox = src_mailbox;
 	} else {
 		/* The (implicit) keep action will move the message */
@@ -500,14 +520,14 @@ int main(int argc, char **argv)
 
 	/* Create error handler */
 	ehandler = sieve_stderr_ehandler_create(svinst, 0);
-	sieve_system_ehandler_set(ehandler);
 	sieve_error_handler_accept_infolog(ehandler, verbose);
+	sieve_error_handler_accept_debuglog(ehandler, svinst->debug);
 
 	/* Compile main sieve script */
-	if ( force_compile ) {
+	if (force_compile) {
 		main_sbin = sieve_tool_script_compile(svinst, scriptfile, NULL);
-		if ( main_sbin != NULL )
-			(void) sieve_save(main_sbin, TRUE, NULL);
+		if (main_sbin != NULL)
+			(void)sieve_save(main_sbin, TRUE, NULL);
 	} else {
 		main_sbin = sieve_tool_script_open(svinst, scriptfile);
 	}
@@ -517,49 +537,50 @@ int main(int argc, char **argv)
 
 	/* Open the source mailbox */
 
-	src_mailbox = mailbox_name_to_mutf7(src_mailbox);
 	ns = mail_namespace_find(mail_user->namespaces, src_mailbox);
-	if ( ns == NULL )
-		i_fatal("Unknown namespace for source mailbox '%s'", src_mailbox);
+	if (ns == NULL) {
+		i_fatal("Unknown namespace for source mailbox '%s'",
+			src_mailbox);
+	}
 
-	if ( !source_write || !execute )
+	if (!source_write || !execute)
 		open_flags |= MAILBOX_FLAG_READONLY;
 
 	src_box = mailbox_alloc(ns->list, src_mailbox, open_flags);
-	if ( mailbox_open(src_box) < 0 ) {
+	if (mailbox_open(src_box) < 0) {
 		i_fatal("Couldn't open source mailbox '%s': %s",
-			src_mailbox, mailbox_get_last_error(src_box, &error));
+			src_mailbox, mailbox_get_last_internal_error(src_box, &error));
 	}
 
 	/* Open move box if necessary */
 
-	if ( execute && discard_action == SIEVE_FILTER_DACT_MOVE &&
-		move_mailbox != NULL ) {
-		move_mailbox = mailbox_name_to_mutf7(move_mailbox);
+	if (execute && discard_action == SIEVE_FILTER_DACT_MOVE &&
+	    move_mailbox != NULL) {
 		ns = mail_namespace_find(mail_user->namespaces, move_mailbox);
-		if ( ns == NULL )
-			i_fatal("Unknown namespace for mailbox '%s'", move_mailbox);
+		if (ns == NULL)
+			i_fatal("Unknown namespace for mailbox '%s'",
+				move_mailbox);
 
 		move_box = mailbox_alloc(ns->list, move_mailbox, open_flags);
-		if ( mailbox_open(move_box) < 0 ) {
+		if (mailbox_open(move_box) < 0) {
 			i_fatal("Couldn't open mailbox '%s': %s",
-				move_mailbox, mailbox_get_last_error(move_box, &error));
+				move_mailbox,
+				mailbox_get_last_internal_error(move_box, &error));
 		}
 
-		if ( mailbox_backends_equal(src_box, move_box) ) {
+		if (mailbox_backends_equal(src_box, move_box))
 			i_fatal("Source mailbox and mailbox for move action are identical.");
-		}
 	}
 
 	/* Compose script environment */
-	memset(&scriptenv, 0, sizeof(scriptenv));
+	if (sieve_script_env_init(&scriptenv, mail_user, &errstr) < 0)
+		i_fatal("Failed to initialize script execution: %s", errstr);
 	scriptenv.mailbox_autocreate = FALSE;
 	scriptenv.default_mailbox = dst_mailbox;
-	scriptenv.user = mail_user;
-	scriptenv.postmaster_address = "postmaster@example.com";
+	scriptenv.result_amend_log_message = result_amend_log_message;
 
 	/* Compose filter context */
-	memset(&sfdata, 0, sizeof(sfdata));
+	i_zero(&sfdata);
 	sfdata.senv = &scriptenv;
 	sfdata.discard_action = discard_action;
 	sfdata.move_mailbox = move_box;
@@ -570,16 +591,20 @@ int main(int argc, char **argv)
 	sfdata.default_move = default_move;
 
 	/* Apply Sieve filter to all messages found */
-	(void) filter_mailbox(&sfdata, src_box);
+	(void)filter_mailbox(&sfdata, src_box);
 
 	/* Close the source mailbox */
-	if ( src_box != NULL )
+	if (src_box != NULL)
 		mailbox_free(&src_box);
 
 	/* Close the move mailbox */
-	if ( move_box != NULL )
+	if (move_box != NULL)
 		mailbox_free(&move_box);
 
+	/* Close the script binary */
+	if (main_sbin != NULL)
+		sieve_close(&main_sbin);
+
 	/* Cleanup error handler */
 	sieve_error_handler_unref(&ehandler);
 
