@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2016 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -32,7 +32,6 @@
  */
 
 #define DEFAULT_SENDMAIL_PATH "/usr/lib/sendmail"
-#define DEFAULT_ENVELOPE_SENDER "MAILER-DAEMON"
 
 /*
  * Print help
@@ -55,38 +54,47 @@ static void print_help(void)
  * Dummy SMTP session
  */
 
-static void *sieve_smtp_start
-(const struct sieve_script_env *senv ATTR_UNUSED,
-	const char *return_path)
+static void *
+sieve_smtp_start(const struct sieve_script_env *senv ATTR_UNUSED,
+		 const struct smtp_address *mail_from)
 {
 	struct ostream *output;
 
-	i_info("sending message from <%s>:",
-		( return_path == NULL ? "" : return_path ));
+	i_info("sending message from <%s>:", smtp_address_encode(mail_from));
 
-	output = o_stream_create_fd(STDOUT_FILENO, (size_t)-1, FALSE);
+	output = o_stream_create_fd(STDOUT_FILENO, (size_t)-1);
+	o_stream_set_no_error_handling(output, TRUE);
 	return (void*)output;
 }
 
-static void sieve_smtp_add_rcpt
-(const struct sieve_script_env *senv ATTR_UNUSED,
-	void *handle ATTR_UNUSED, const char *address)
+static void
+sieve_smtp_add_rcpt(const struct sieve_script_env *senv ATTR_UNUSED,
+		    void *handle ATTR_UNUSED,
+		    const struct smtp_address *rcpt_to)
 {
-	printf("\nRECIPIENT: %s\n", address);
+	printf("\nRECIPIENT: %s\n", smtp_address_encode(rcpt_to));
 }
 
-static struct ostream *sieve_smtp_send
-(const struct sieve_script_env *senv ATTR_UNUSED,
-	void *handle)
+static struct ostream *
+sieve_smtp_send(const struct sieve_script_env *senv ATTR_UNUSED, void *handle)
 {
 	printf("START MESSAGE:\n");
 
 	return (struct ostream *)handle;
 }
 
-static int sieve_smtp_finish
-(const struct sieve_script_env *senv ATTR_UNUSED,
-	void *handle, const char **error_r ATTR_UNUSED)
+static void
+sieve_smtp_abort(const struct sieve_script_env *senv ATTR_UNUSED, void *handle)
+{
+	struct ostream *output = (struct ostream *)handle;
+
+	printf("#### ABORT MESSAGE ####\n\n");
+	o_stream_unref(&output);
+}
+
+static int
+sieve_smtp_finish(const struct sieve_script_env *senv ATTR_UNUSED, void *handle,
+		  const char **error_r ATTR_UNUSED)
 {
 	struct ostream *output = (struct ostream *)handle;
 
@@ -99,21 +107,44 @@ static int sieve_smtp_finish
  * Dummy duplicate check implementation
  */
 
-static int duplicate_check
-(const struct sieve_script_env *senv, const void *id ATTR_UNUSED,
-	size_t id_size ATTR_UNUSED)
+static bool
+duplicate_check(const struct sieve_script_env *senv, const void *id ATTR_UNUSED,
+		size_t id_size ATTR_UNUSED)
 {
 	i_info("checked duplicate for user %s.\n", senv->user->username);
 	return 0;
 }
 
-static void duplicate_mark
-(const struct sieve_script_env *senv, const void *id ATTR_UNUSED,
-	size_t id_size ATTR_UNUSED, time_t time ATTR_UNUSED)
+static void
+duplicate_mark(const struct sieve_script_env *senv, const void *id ATTR_UNUSED,
+	       size_t id_size ATTR_UNUSED, time_t time ATTR_UNUSED)
 {
 	i_info("marked duplicate for user %s.\n", senv->user->username);
 }
 
+/*
+ * Result logging
+ */
+
+static const char *
+result_amend_log_message(const struct sieve_script_env *senv,
+			 enum log_type log_type, const char *message)
+{
+	const struct sieve_message_data *msgdata = senv->script_context;
+	string_t *str;
+
+	if (log_type == LOG_TYPE_DEBUG)
+		return message;
+
+	str = t_str_new(256);
+	str_printfa(str, "msgid=%s", (msgdata->id == NULL ?
+				      "unspecified" : msgdata->id));
+	str_append(str, ": ");
+	str_append(str, message);
+
+	return str_c(str);
+}
+
 /*
  * Tool implementation
  */
@@ -122,45 +153,58 @@ int main(int argc, char **argv)
 {
 	struct sieve_instance *svinst;
 	ARRAY_TYPE (const_string) scriptfiles;
-	const char *scriptfile, *recipient, *final_recipient, *sender, *mailbox,
-		*dumpfile, *tracefile, *mailfile, *mailloc;
+	const char *scriptfile, *mailbox, *dumpfile, *tracefile, *mailfile,
+		*mailloc, *errstr;
+	struct smtp_address *rcpt_to, *final_rcpt_to, *mail_from;
 	struct sieve_trace_config trace_config;
 	struct mail *mail;
 	struct sieve_binary *main_sbin, *sbin = NULL;
 	struct sieve_message_data msgdata;
 	struct sieve_script_env scriptenv;
 	struct sieve_exec_status estatus;
-	struct sieve_error_handler *ehandler, *action_ehandler;
+	enum sieve_execute_flags exflags = SIEVE_EXECUTE_FLAG_LOG_RESULT;
+	struct sieve_error_handler *ehandler;
 	struct ostream *teststream = NULL;
 	struct sieve_trace_log *trace_log = NULL;
 	bool force_compile = FALSE, execute = FALSE;
 	int exit_status = EXIT_SUCCESS;
 	int ret, c;
 
-	sieve_tool = sieve_tool_init
-		("sieve-test", &argc, &argv, "r:a:f:m:d:l:s:eCt:T:DP:x:u:", FALSE);
+	sieve_tool = sieve_tool_init("sieve-test", &argc, &argv,
+				     "r:a:f:m:d:l:s:eCt:T:DP:x:u:", FALSE);
 
-	ehandler = action_ehandler = NULL;
+	ehandler = NULL;
 	t_array_init(&scriptfiles, 16);
 
 	/* Parse arguments */
-	recipient = final_recipient = sender = mailbox = dumpfile =
-		tracefile = mailloc = NULL;
-	memset(&trace_config, 0, sizeof(trace_config));
+	mailbox = dumpfile = tracefile = mailloc = NULL;
+	mail_from = final_rcpt_to = rcpt_to = NULL;
+	i_zero(&trace_config);
 	trace_config.level = SIEVE_TRLVL_ACTIONS;
 	while ((c = sieve_tool_getopt(sieve_tool)) > 0) {
 		switch (c) {
 		case 'r':
 			/* final recipient address */
-			final_recipient = optarg;
+			if (smtp_address_parse_mailbox(
+				pool_datastack_create(), optarg,
+				SMTP_ADDRESS_PARSE_FLAG_ALLOW_LOCALPART,
+				&final_rcpt_to, &errstr) < 0)
+				i_fatal("Invalid -r parameter: %s", errstr);
 			break;
 		case 'a':
 			/* original recipient address */
-			recipient = optarg;
+			if (smtp_address_parse_mailbox(
+				pool_datastack_create(), optarg,
+				SMTP_ADDRESS_PARSE_FLAG_ALLOW_LOCALPART,
+				&rcpt_to, &errstr) < 0)
+				i_fatal("Invalid -a parameter: %s", errstr);
 			break;
 		case 'f':
 			/* envelope sender address */
-			sender = optarg;
+			if (smtp_address_parse_mailbox(
+				pool_datastack_create(), optarg,
+				0, &mail_from, &errstr) < 0)
+				i_fatal("Invalid -f parameter: %s", errstr);
 			break;
 		case 'm':
 			/* default mailbox (keep box) */
@@ -207,16 +251,16 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if ( optind < argc ) {
+	if (optind < argc)
 		scriptfile = argv[optind++];
-	} else {
+	else {
 		print_help();
 		i_fatal_status(EX_USAGE, "Missing <script-file> argument");
 	}
 
-	if ( optind < argc ) {
+	if (optind < argc)
 		mailfile = argv[optind++];
-	} else {
+	else {
 		print_help();
 		i_fatal_status(EX_USAGE, "Missing <mail-file> argument");
 	}
@@ -234,94 +278,96 @@ int main(int argc, char **argv)
 
 	/* Create error handler */
 	ehandler = sieve_stderr_ehandler_create(svinst, 0);
-	sieve_system_ehandler_set(ehandler);
 	sieve_error_handler_accept_infolog(ehandler, TRUE);
 	sieve_error_handler_accept_debuglog(ehandler, svinst->debug);
 
 	/* Compile main sieve script */
-	if ( force_compile ) {
+	if (force_compile) {
 		main_sbin = sieve_tool_script_compile(svinst, scriptfile, NULL);
-		if ( main_sbin != NULL )
-			(void) sieve_save(main_sbin, TRUE, NULL);
+		if (main_sbin != NULL)
+			(void)sieve_save(main_sbin, TRUE, NULL);
 	} else {
 		main_sbin = sieve_tool_script_open(svinst, scriptfile);
 	}
 
-	if ( main_sbin == NULL ) {
+	if (main_sbin == NULL) {
 		exit_status = EXIT_FAILURE;
 	} else {
 		/* Dump script */
 		sieve_tool_dump_binary_to(main_sbin, dumpfile, FALSE);
 
 		/* Obtain mail namespaces from -l argument */
-		if ( mailloc != NULL ) {
+		if (mailloc != NULL)
 			sieve_tool_init_mail_user(sieve_tool, mailloc);
-		}
 
 		/* Initialize raw mail object */
 		mail = sieve_tool_open_file_as_mail(sieve_tool, mailfile);
 
-		sieve_tool_get_envelope_data(mail, &recipient, &sender);
-
-		if ( mailbox == NULL )
+		if (mailbox == NULL)
 			mailbox = "INBOX";
 
 		/* Collect necessary message data */
-		memset(&msgdata, 0, sizeof(msgdata));
+		i_zero(&msgdata);
 		msgdata.mail = mail;
-		msgdata.return_path = sender;
-		msgdata.orig_envelope_to = recipient;
-		msgdata.final_envelope_to =
-			( final_recipient == NULL ? recipient : final_recipient );
 		msgdata.auth_user = sieve_tool_get_username(sieve_tool);
-		(void)mail_get_first_header(mail, "Message-ID", &msgdata.id);
+		(void)mail_get_message_id(mail, &msgdata.id);
+
+		sieve_tool_get_envelope_data(&msgdata, mail,
+					     mail_from, rcpt_to, final_rcpt_to);
 
 		/* Create streams for test and trace output */
 
-		if ( !execute ) {
-			action_ehandler = NULL;
-			teststream = o_stream_create_fd(1, 0, FALSE);
-		} else {
-			action_ehandler = sieve_prefix_ehandler_create
-				(ehandler, NULL, t_strdup_printf("msgid=%s",
-					( msgdata.id == NULL ? "unspecified" : msgdata.id )));
+		if (!execute) {
+			teststream = o_stream_create_fd(1, 0);
+			o_stream_set_no_error_handling(teststream, TRUE);
 		}
 
-		if ( tracefile != NULL ) {
-			(void)sieve_trace_log_create(svinst,
-				(strcmp(tracefile, "-") == 0 ? NULL : tracefile),
-				&trace_log);
+		if (tracefile != NULL) {
+			(void)sieve_trace_log_create(
+				svinst, (strcmp(tracefile, "-") == 0 ?
+					 NULL : tracefile), &trace_log);
 		}
 
 		/* Compose script environment */
-		memset(&scriptenv, 0, sizeof(scriptenv));
+		if (sieve_script_env_init(
+			&scriptenv, sieve_tool_get_mail_user(sieve_tool),
+			&errstr) < 0) {
+			i_fatal("Failed to initialize script execution: %s",
+				errstr);
+		}
+
 		scriptenv.default_mailbox = mailbox;
-		scriptenv.user = sieve_tool_get_mail_user(sieve_tool);
-		scriptenv.postmaster_address = "postmaster@example.com";
 		scriptenv.smtp_start = sieve_smtp_start;
 		scriptenv.smtp_add_rcpt = sieve_smtp_add_rcpt;
 		scriptenv.smtp_send = sieve_smtp_send;
+		scriptenv.smtp_abort = sieve_smtp_abort;
 		scriptenv.smtp_finish = sieve_smtp_finish;
 		scriptenv.duplicate_mark = duplicate_mark;
 		scriptenv.duplicate_check = duplicate_check;
+		scriptenv.result_amend_log_message = result_amend_log_message;
 		scriptenv.trace_log = trace_log;
 		scriptenv.trace_config = trace_config;
+		scriptenv.script_context = &msgdata;
+
+		i_zero(&estatus);
 		scriptenv.exec_status = &estatus;
 
 		/* Run the test */
 		ret = 1;
-		if ( array_count(&scriptfiles) == 0 ) {
+		if (array_count(&scriptfiles) == 0) {
 			/* Single script */
 			sbin = main_sbin;
 			main_sbin = NULL;
 
 			/* Execute/Test script */
-			if ( execute ) {
+			if (execute) {
 				ret = sieve_execute(sbin, &msgdata, &scriptenv,
-					ehandler, action_ehandler, 0, NULL);
+						    ehandler, ehandler, exflags,
+						    NULL);
 			} else {
 				ret = sieve_test(sbin, &msgdata, &scriptenv,
-					ehandler, teststream, 0, NULL);
+						 ehandler, teststream, exflags,
+						 NULL);
 			}
 		} else {
 			/* Multiple scripts */
@@ -331,76 +377,92 @@ int main(int argc, char **argv)
 			bool more = TRUE;
 			int result;
 
-			if ( execute )
-				mscript = sieve_multiscript_start_execute
-					(svinst, &msgdata, &scriptenv);
+			if (execute)
+				mscript = sieve_multiscript_start_execute(
+					svinst, &msgdata, &scriptenv);
 			else
-				mscript = sieve_multiscript_start_test
-					(svinst, &msgdata, &scriptenv, teststream);
+				mscript = sieve_multiscript_start_test(
+					svinst, &msgdata, &scriptenv,
+					teststream);
 
 			/* Execute scripts sequentially */
 			sfiles = array_get(&scriptfiles, &count);
-			for ( i = 0; i < count && more; i++ ) {
-				if ( teststream != NULL )
-					o_stream_send_str(teststream,
-						t_strdup_printf("\n## Executing script: %s\n", sfiles[i]));
+			for (i = 0; i < count && more; i++) {
+				if (teststream != NULL) {
+					o_stream_nsend_str(
+						teststream,
+						t_strdup_printf("\n## Executing script: %s\n",
+								sfiles[i]));
+				}
 
 				/* Close previous script */
-				if ( sbin != NULL )
+				if (sbin != NULL)
 					sieve_close(&sbin);
 
 				/* Compile sieve script */
-				if ( force_compile ) {
-					sbin = sieve_tool_script_compile(svinst, sfiles[i], sfiles[i]);
-					if ( sbin != NULL )
-						(void) sieve_save(sbin, FALSE, NULL);
+				if (force_compile) {
+					sbin = sieve_tool_script_compile(
+						svinst, sfiles[i], sfiles[i]);
+					if (sbin != NULL)
+						(void)sieve_save(sbin, FALSE, NULL);
 				} else {
 					sbin = sieve_tool_script_open(svinst, sfiles[i]);
 				}
 
-				if ( sbin == NULL ) {
+				if (sbin == NULL) {
 					ret = SIEVE_EXEC_FAILURE;
 					break;
 				}
 
 				/* Execute/Test script */
-				more = sieve_multiscript_run(mscript, sbin,
-					ehandler, action_ehandler, 0);
+				more = sieve_multiscript_run(
+					mscript, sbin, ehandler, ehandler,
+					exflags);
 			}
 
 			/* Execute/Test main script */
-			if ( more && ret > 0 ) {
-				if ( teststream != NULL )
-					o_stream_send_str(teststream,
-						t_strdup_printf("## Executing script: %s\n", scriptfile));
+			if (more && ret > 0) {
+				if (teststream != NULL) {
+					o_stream_nsend_str(
+						teststream,
+						t_strdup_printf("## Executing script: %s\n",
+								scriptfile));
+				}
 
 				/* Close previous script */
-				if ( sbin != NULL )
+				if (sbin != NULL)
 					sieve_close(&sbin);
 
 				sbin = main_sbin;
 				main_sbin = NULL;
 
-				(void)sieve_multiscript_run(mscript, sbin,
-					ehandler, ehandler, 0);
+				(void)sieve_multiscript_run(
+					mscript, sbin, ehandler, ehandler,
+					exflags);
 			}
 
-			result = sieve_multiscript_finish(&mscript, ehandler, 0, NULL);
+			result = sieve_multiscript_finish(
+				&mscript, ehandler, exflags, NULL);
 
-			ret = ret > 0 ? result : ret;
+			ret = (ret > 0 ? result : ret);
 		}
 
 		/* Run */
-		switch ( ret ) {
+		switch (ret) {
 		case SIEVE_EXEC_OK:
 			i_info("final result: success");
 			break;
+		case SIEVE_EXEC_RESOURCE_LIMIT:
+			i_info("resource limit exceeded");
+			exit_status = EXIT_FAILURE;
+			break;
 		case SIEVE_EXEC_BIN_CORRUPT:
 			i_info("corrupt binary deleted.");
-			(void) unlink(sieve_binary_path(sbin));
+			i_unlink_if_exists(sieve_binary_path(sbin));
 			/* fall through */
 		case SIEVE_EXEC_FAILURE:
-			i_info("final result: failed; resolved with successful implicit keep");
+			i_info("final result: failed; "
+			       "resolved with successful implicit keep");
 			exit_status = EXIT_FAILURE;
 			break;
 		case SIEVE_EXEC_TEMP_FAILURE:
@@ -413,21 +475,19 @@ int main(int argc, char **argv)
 			break;
 		}
 
-		if ( teststream != NULL )
+		if (teststream != NULL)
 			o_stream_destroy(&teststream);
-		if ( trace_log != NULL )
+		if (trace_log != NULL)
 			sieve_trace_log_free(&trace_log);
 
 		/* Cleanup remaining binaries */
-		if ( sbin != NULL )
+		if (sbin != NULL)
 			sieve_close(&sbin);
-		if ( main_sbin != NULL )
+		if (main_sbin != NULL)
 			sieve_close(&main_sbin);
 	}
 
 	/* Cleanup error handler */
-	if (action_ehandler != NULL)
-		sieve_error_handler_unref(&action_ehandler);
 	sieve_error_handler_unref(&ehandler);
 
 	sieve_tool_deinit(&sieve_tool);
