@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "sieve-common.h"
@@ -30,15 +30,14 @@ static bool cmd_test_binary_generate
  */
 
 const struct sieve_command_def cmd_test_binary_load = {
-	"test_binary_load",
-	SCT_COMMAND,
-	1, 0, FALSE, FALSE,
-	NULL,
-	NULL,
-	cmd_test_binary_validate,
-	NULL,
-	cmd_test_binary_generate,
-	NULL
+	.identifier = "test_binary_load",
+	.type = SCT_COMMAND,
+	.positional_args = 1,
+	.subtests = 0,
+	.block_allowed = FALSE,
+	.block_required = FALSE,
+	.validate = cmd_test_binary_validate,
+	.generate = cmd_test_binary_generate
 };
 
 /* Test_binary_save command
@@ -48,15 +47,14 @@ const struct sieve_command_def cmd_test_binary_load = {
  */
 
 const struct sieve_command_def cmd_test_binary_save = {
-	"test_binary_save",
-	SCT_COMMAND,
-	1, 0, FALSE, FALSE,
-	NULL,
-	NULL,
-	cmd_test_binary_validate,
-	NULL,
-	cmd_test_binary_generate,
-	NULL
+	.identifier = "test_binary_save",
+	.type = SCT_COMMAND,
+	.positional_args = 1,
+	.subtests = 0,
+	.block_allowed = FALSE,
+	.block_required = FALSE,
+	.validate = cmd_test_binary_validate,
+	.generate = cmd_test_binary_generate,
 };
 
 /*
@@ -71,21 +69,21 @@ static int cmd_test_binary_operation_execute
 /* test_binary_create operation */
 
 const struct sieve_operation_def test_binary_load_operation = {
-	"TEST_BINARY_LOAD",
-	&testsuite_extension,
-	TESTSUITE_OPERATION_TEST_BINARY_LOAD,
-	cmd_test_binary_operation_dump,
-	cmd_test_binary_operation_execute
+	.mnemonic = "TEST_BINARY_LOAD",
+	.ext_def = &testsuite_extension,
+	.code = TESTSUITE_OPERATION_TEST_BINARY_LOAD,
+	.dump = cmd_test_binary_operation_dump,
+	.execute = cmd_test_binary_operation_execute
 };
 
 /* test_binary_delete operation */
 
 const struct sieve_operation_def test_binary_save_operation = {
-	"TEST_BINARY_SAVE",
-	&testsuite_extension,
-	TESTSUITE_OPERATION_TEST_BINARY_SAVE,
-	cmd_test_binary_operation_dump,
-	cmd_test_binary_operation_execute
+	.mnemonic = "TEST_BINARY_SAVE",
+	.ext_def = &testsuite_extension,
+	.code = TESTSUITE_OPERATION_TEST_BINARY_SAVE,
+	.dump = cmd_test_binary_operation_dump,
+	.execute = cmd_test_binary_operation_execute
 };
 
 /*
@@ -180,7 +178,7 @@ static int cmd_test_binary_operation_execute
 
 			sieve_binary_unref(&sbin);
 		} else {
-			sieve_sys_error(testsuite_sieve_instance,
+			e_error(testsuite_sieve_instance->event,
 				"failed to load binary %s", str_c(binary_name));
 			return SIEVE_EXEC_FAILURE;
 		}
@@ -197,8 +195,9 @@ static int cmd_test_binary_operation_execute
 		if ( sbin != NULL )
 			testsuite_binary_save(sbin, str_c(binary_name));
 		else {
-			sieve_sys_error(testsuite_sieve_instance,
-				"no compiled binary to save as %s", str_c(binary_name));
+			e_error(testsuite_sieve_instance->event,
+				"no compiled binary to save as %s",
+				str_c(binary_name));
 			return SIEVE_EXEC_FAILURE;
 		}
 	} else {
