@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 #include "sieve-common.h"
@@ -25,14 +25,14 @@ static bool cmd_test_generate
 	(const struct sieve_codegen_env *cgenv, struct sieve_command *md);
 
 const struct sieve_command_def cmd_test = {
-	"test",
-	SCT_COMMAND,
-	1, 0, TRUE, TRUE,
-	NULL, NULL,
-	cmd_test_validate,
-	NULL,
-	cmd_test_generate,
-	NULL
+	.identifier = "test",
+	.type = SCT_COMMAND,
+	.positional_args = 1,
+	.subtests = 0,
+	.block_allowed = TRUE,
+	.block_required = TRUE,
+	.validate = cmd_test_validate,
+	.generate = cmd_test_generate,
 };
 
 /*
@@ -47,11 +47,11 @@ static int cmd_test_operation_execute
 	(const struct sieve_runtime_env *renv, sieve_size_t *address);
 
 const struct sieve_operation_def test_operation = {
-	"TEST",
-	&testsuite_extension,
-	TESTSUITE_OPERATION_TEST,
-	cmd_test_operation_dump,
-	cmd_test_operation_execute
+	.mnemonic = "TEST",
+	.ext_def = &testsuite_extension,
+	.code = TESTSUITE_OPERATION_TEST,
+	.dump = cmd_test_operation_dump,
+	.execute = cmd_test_operation_execute
 };
 
 /* Test_finish operation */
@@ -60,11 +60,10 @@ static int cmd_test_finish_operation_execute
 	(const struct sieve_runtime_env *renv, sieve_size_t *address);
 
 const struct sieve_operation_def test_finish_operation = {
-	"TEST-FINISH",
-	&testsuite_extension,
-	TESTSUITE_OPERATION_TEST_FINISH,
-	NULL,
-	cmd_test_finish_operation_execute
+	.mnemonic = "TEST-FINISH",
+	.ext_def = &testsuite_extension,
+	.code = TESTSUITE_OPERATION_TEST_FINISH,
+	.execute = cmd_test_finish_operation_execute
 };
 
 /*
