@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2016 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -29,11 +29,9 @@ static bool arg_testsuite_string_validate
 		struct sieve_command *context);
 
 const struct sieve_argument_def testsuite_string_argument = {
-	"@testsuite-string",
-	NULL,
-	arg_testsuite_string_validate,
-	NULL, NULL,
-	sieve_arg_catenated_string_generate,
+	.identifier = "@testsuite-string",
+	.validate = arg_testsuite_string_validate,
+	.generate = sieve_arg_catenated_string_generate,
 };
 
 static bool arg_testsuite_string_validate
