@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -109,7 +109,7 @@ bool testsuite_generator_context_initialize
 
 static void testsuite_interpreter_free
 (const struct sieve_extension *ext ATTR_UNUSED,
-        struct sieve_interpreter *interp ATTR_UNUSED, void *context)
+	struct sieve_interpreter *interp ATTR_UNUSED, void *context)
 {
 	struct testsuite_interpreter_context *ctx =
 		(struct testsuite_interpreter_context *)context;
@@ -118,10 +118,10 @@ static void testsuite_interpreter_free
 		sieve_binary_unref(&ctx->compiled_script);
 }
 
-const struct sieve_interpreter_extension testsuite_interpreter_ext = {
-        &testsuite_extension,
-	NULL,
-        testsuite_interpreter_free,
+const struct sieve_interpreter_extension
+testsuite_interpreter_ext = {
+	.ext_def = &testsuite_extension,
+	.free = testsuite_interpreter_free,
 };
 
 bool testsuite_interpreter_context_initialize
@@ -261,9 +261,11 @@ static void testsuite_tmp_dir_init(void)
 
 static void testsuite_tmp_dir_deinit(void)
 {
-	if ( unlink_directory(testsuite_tmp_dir, TRUE) < 0 )
-		i_warning("failed to remove temporary directory '%s': %m.",
-			testsuite_tmp_dir);
+	const char *error;
+
+	if ( unlink_directory(testsuite_tmp_dir, UNLINK_DIRECTORY_FLAG_RMDIR, &error) < 0 )
+		i_warning("failed to remove temporary directory '%s': %s.",
+			testsuite_tmp_dir, error);
 
 	i_free(testsuite_tmp_dir);
 }
