@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2016 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -76,7 +76,7 @@ bool testsuite_result_execute(const struct sieve_runtime_env *renv)
 
 	/* Execute the result */
 	ret=sieve_result_execute
-		(_testsuite_result, NULL, testsuite_log_ehandler);
+		(_testsuite_result, NULL, testsuite_log_ehandler, 0);
 
 	return ( ret > 0 );
 }
@@ -86,11 +86,12 @@ void testsuite_result_print
 {
 	struct ostream *out;
 
-	out = o_stream_create_fd(1, 0, FALSE);
+	out = o_stream_create_fd(1, 0);
+	o_stream_set_no_error_handling(out, TRUE);
 
-	o_stream_send_str(out, "\n--");
+	o_stream_nsend_str(out, "\n--");
 	sieve_result_print(_testsuite_result, renv->scriptenv, out, NULL);
-	o_stream_send_str(out, "--\n\n");
+	o_stream_nsend_str(out, "--\n\n");
 
 	o_stream_destroy(&out);
 }
