@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #ifndef __TESTSUITE_SMTP_H
@@ -14,10 +14,10 @@ void testsuite_smtp_reset(void);
 
 void *testsuite_smtp_start
 	(const struct sieve_script_env *senv ATTR_UNUSED,
-		const char *return_path);
+		const struct smtp_address *mail_from);
 void testsuite_smtp_add_rcpt
 	(const struct sieve_script_env *senv ATTR_UNUSED,
-		void *handle, const char *address);
+		void *handle, const struct smtp_address *rcpt_to);
 struct ostream *testsuite_smtp_send
 	(const struct sieve_script_env *senv ATTR_UNUSED,
 		void *handle);
