@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2015 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2018 Pigeonhole authors, see the included COPYING file
  */
 
 #include "lib.h"
@@ -7,7 +7,7 @@
 #include "env-util.h"
 #include "ostream.h"
 #include "hostpid.h"
-#include "abspath.h"
+#include "path-util.h"
 
 #include "sieve.h"
 #include "sieve-extensions.h"
@@ -39,7 +39,6 @@ const struct sieve_script_env *testsuite_scriptenv;
  */
 
 #define DEFAULT_SENDMAIL_PATH "/usr/lib/sendmail"
-#define DEFAULT_ENVELOPE_SENDER "MAILER-DAEMON"
 
 /*
  * Testsuite execution
@@ -48,31 +47,29 @@ const struct sieve_script_env *testsuite_scriptenv;
 static void print_help(void)
 {
 	printf(
-"Usage: testsuite [-D] [-E] [-d <dump-filename>]\n"
+"Usage: testsuite [-D] [-E] [-F] [-d <dump-filename>]\n"
 "                 [-t <trace-filename>] [-T <trace-option>]\n"
 "                 [-P <plugin>] [-x <extensions>]\n"
 "                 <scriptfile>\n"
 	);
 }
 
-static int testsuite_run
-(struct sieve_binary *sbin, const struct sieve_message_data *msgdata,
-	const struct sieve_script_env *senv, struct sieve_error_handler *ehandler)
+static int
+testsuite_run(struct sieve_binary *sbin, struct sieve_error_handler *ehandler)
 {
 	struct sieve_interpreter *interp;
 	struct sieve_result *result;
 	int ret = 0;
 
 	/* Create the interpreter */
-	if ( (interp=sieve_interpreter_create
-		(sbin, msgdata, senv, ehandler, 0)) == NULL )
+	interp = sieve_interpreter_create(sbin, NULL, &testsuite_execute_env,
+					  ehandler);
+	if (interp == NULL)
 		return SIEVE_EXEC_BIN_CORRUPT;
 
 	/* Run the interpreter */
 	result = testsuite_result_get();
-	sieve_result_ref(result);
 	ret = sieve_interpreter_run(interp, result);
-	sieve_result_unref(&result);
 
 	/* Free the interpreter */
 	sieve_interpreter_free(&interp);
@@ -84,19 +81,19 @@ int main(int argc, char **argv)
 {
 	struct sieve_instance *svinst;
 	const char *scriptfile, *dumpfile, *tracefile;
-	struct sieve_trace_config tr_config;
+	struct sieve_trace_config trace_config;
 	struct sieve_binary *sbin;
-	const char *sieve_dir;
-	bool log_stdout = FALSE;
+	const char *sieve_dir, *cwd, *error;
+	bool log_stdout = FALSE, expect_failure = FALSE;
 	int ret, c;
 
-	sieve_tool = sieve_tool_init
-		("testsuite", &argc, &argv, "d:t:T:EDP:", TRUE);
+	sieve_tool = sieve_tool_init("testsuite", &argc, &argv,
+				     "d:t:T:EFDP:", TRUE);
 
 	/* Parse arguments */
 	dumpfile = tracefile = NULL;
-	memset(&tr_config, 0, sizeof(tr_config));
-	tr_config.level = SIEVE_TRLVL_ACTIONS;
+	i_zero(&trace_config);
+	trace_config.level = SIEVE_TRLVL_ACTIONS;
 	while ((c = sieve_tool_getopt(sieve_tool)) > 0) {
 		switch (c) {
 		case 'd':
@@ -108,20 +105,22 @@ int main(int argc, char **argv)
 			tracefile = optarg;
 			break;
 		case 'T':
-			sieve_tool_parse_trace_option(&tr_config, optarg);
+			sieve_tool_parse_trace_option(&trace_config, optarg);
 			break;
 		case 'E':
 			log_stdout = TRUE;
 			break;
+		case 'F':
+			expect_failure = TRUE;
+			break;
 		default:
 			print_help();
-			i_fatal_status(EX_USAGE,
-				"Unknown argument: %c", c);
+			i_fatal_status(EX_USAGE, "Unknown argument: %c", c);
 			break;
 		}
 	}
 
-	if ( optind < argc ) {
+	if (optind < argc) {
 		scriptfile = t_strdup(argv[optind++]);
 	} else {
 		print_help();
@@ -133,24 +132,30 @@ int main(int argc, char **argv)
 		i_fatal_status(EX_USAGE, "Unknown argument: %s", argv[optind]);
 	}
 
+	// FIXME: very very ugly
+	master_service_parse_option(
+		master_service, 'o',
+		"postmaster_address=postmaster@example.com");
+
 	/* Initialize mail user */
-	sieve_tool_set_homedir(sieve_tool, t_abspath(""));
+	if (t_get_working_dir(&cwd, &error) < 0)
+		i_fatal("Failed to get working directory: %s", error);
+	sieve_tool_set_homedir(sieve_tool, cwd);
 
 	/* Initialize settings environment */
 	testsuite_settings_init();
 
 	/* Currently needed for include (FIXME) */
 	sieve_dir = strrchr(scriptfile, '/');
-	if ( sieve_dir == NULL )
-		sieve_dir= "./";
-	else {
+	if (sieve_dir == NULL)
+		sieve_dir = "./";
+	else
 		sieve_dir = t_strdup_until(scriptfile, sieve_dir+1);
-	}
 
-	testsuite_setting_set
-		("sieve_dir", t_strconcat(sieve_dir, "included", NULL));
-	testsuite_setting_set
-		("sieve_global_dir", t_strconcat(sieve_dir, "included-global", NULL));
+	testsuite_setting_set("sieve_dir",
+			      t_strconcat(sieve_dir, "included", NULL));
+	testsuite_setting_set("sieve_global_dir",
+			      t_strconcat(sieve_dir, "included-global", NULL));
 
 	/* Finish testsuite initialization */
 	svinst = sieve_tool_init_finish(sieve_tool, FALSE, FALSE);
@@ -159,62 +164,79 @@ int main(int argc, char **argv)
 	printf("Test case: %s:\n\n", scriptfile);
 
 	/* Compile sieve script */
-	if ( (sbin = sieve_compile
-		(svinst, scriptfile, NULL, testsuite_log_main_ehandler, 0, NULL))
-			!= NULL ) {
-		struct ostream *tracestream = NULL;
+	if ((sbin = sieve_compile(svinst, scriptfile, NULL,
+				  testsuite_log_main_ehandler,
+				  0, NULL)) != NULL) {
+		struct sieve_trace_log *trace_log = NULL;
+		struct sieve_exec_status exec_status;
 		struct sieve_script_env scriptenv;
 
 		/* Dump script */
 		sieve_tool_dump_binary_to(sbin, dumpfile, FALSE);
 
-		if ( tracefile != NULL )
-			tracestream = sieve_tool_open_output_stream(tracefile);
+		if (tracefile != NULL) {
+			(void)sieve_trace_log_create(
+				svinst, (strcmp(tracefile, "-") == 0 ?
+					 NULL : tracefile), &trace_log);
+		}
 
 		testsuite_mailstore_init();
 		testsuite_message_init();
 
-		memset(&scriptenv, 0, sizeof(scriptenv));
-		scriptenv.user = testsuite_mailstore_get_user();
+		if (sieve_script_env_init(&scriptenv,
+					  testsuite_mailstore_get_user(),
+					  &error) < 0) {
+			i_fatal("Failed to initialize script execution: %s",
+				error);
+		}
+
+		i_zero(&exec_status);
+
 		scriptenv.default_mailbox = "INBOX";
-		scriptenv.postmaster_address = "postmaster@example.com";
 		scriptenv.smtp_start = testsuite_smtp_start;
 		scriptenv.smtp_add_rcpt = testsuite_smtp_add_rcpt;
 		scriptenv.smtp_send = testsuite_smtp_send;
+		scriptenv.smtp_abort = testsuite_smtp_abort;
 		scriptenv.smtp_finish = testsuite_smtp_finish;
-		scriptenv.trace_stream = tracestream;
-		scriptenv.trace_config = tr_config;
+		scriptenv.trace_log = trace_log;
+		scriptenv.trace_config = trace_config;
+		scriptenv.exec_status = &exec_status;
 
 		testsuite_scriptenv = &scriptenv;
 
 		testsuite_result_init();
 
 		/* Run the test */
-		ret = testsuite_run
-			(sbin, &testsuite_msgdata, &scriptenv, testsuite_log_main_ehandler);
+		ret = testsuite_run(sbin, testsuite_log_main_ehandler);
 
-		switch ( ret ) {
+		switch (ret) {
 		case SIEVE_EXEC_OK:
 			break;
 		case SIEVE_EXEC_FAILURE:
 		case SIEVE_EXEC_KEEP_FAILED:
 		case SIEVE_EXEC_TEMP_FAILURE:
-			testsuite_testcase_fail("test script execution aborted due to error");
+			testsuite_testcase_fail(
+				"test script execution aborted due to error");
 			break;
 		case SIEVE_EXEC_BIN_CORRUPT:
-			testsuite_testcase_fail("compiled test script binary is corrupt");
+			testsuite_testcase_fail(
+				"compiled test script binary is corrupt");
+			break;
+		case SIEVE_EXEC_RESOURCE_LIMIT:
+			testsuite_testcase_fail(
+				"resource limit exceeded");
 			break;
 		}
 
 		sieve_close(&sbin);
 
 		/* De-initialize message environment */
+		testsuite_result_deinit();
 		testsuite_message_deinit();
 		testsuite_mailstore_deinit();
-		testsuite_result_deinit();
 
-		if ( tracestream != NULL )
-			o_stream_unref(&tracestream);
+		if (trace_log != NULL)
+			sieve_trace_log_free(&trace_log);
 
 		testsuite_scriptenv = NULL;
 	} else {
@@ -227,7 +249,7 @@ int main(int argc, char **argv)
 
 	sieve_tool_deinit(&sieve_tool);
 
-	if ( !testsuite_testcase_result() )
+	if (!testsuite_testcase_result(expect_failure))
 		return EXIT_FAILURE;
 
 	return EXIT_SUCCESS;
