@@ -1,4 +1,4 @@
-/* Copyright (c) 2002-2013 Pigeonhole authors, see the included COPYING file
+/* Copyright (c) 2002-2017 Pigeonhole authors, see the included COPYING file
  */
 
 #include "sieve-common.h"
@@ -30,13 +30,14 @@ static bool tst_test_script_run_generate
 	(const struct sieve_codegen_env *cgenv, struct sieve_command *cmd);
 
 const struct sieve_command_def tst_test_script_run = {
-	"test_script_run",
-	SCT_TEST,
-	0, 0, FALSE, FALSE,
-	tst_test_script_run_registered,
-	NULL, NULL, NULL,
-	tst_test_script_run_generate,
-	NULL
+	.identifier = "test_script_run",
+	.type = SCT_TEST,
+	.positional_args = 0,
+	.subtests = 0,
+	.block_allowed = FALSE,
+	.block_required = FALSE,
+	.registered = tst_test_script_run_registered,
+	.generate = tst_test_script_run_generate
 };
 
 /*
@@ -49,11 +50,11 @@ static int tst_test_script_run_operation_execute
 	(const struct sieve_runtime_env *renv, sieve_size_t *address);
 
 const struct sieve_operation_def test_script_run_operation = {
-	"test_script_run",
-	&testsuite_extension,
-	TESTSUITE_OPERATION_TEST_SCRIPT_RUN,
-	tst_test_script_run_operation_dump,
-	tst_test_script_run_operation_execute
+	.mnemonic = "TEST_SCRIPT_RUN",
+	.ext_def = &testsuite_extension,
+	.code = TESTSUITE_OPERATION_TEST_SCRIPT_RUN,
+	.dump = tst_test_script_run_operation_dump,
+	.execute = tst_test_script_run_operation_execute
 };
 
 /*
@@ -70,8 +71,7 @@ enum cmd_vacation_optional {
 /* Tags */
 
 static const struct sieve_argument_def append_result_tag = {
-	"append_result",
-	NULL, NULL, NULL, NULL, NULL
+	.identifier = "append_result"
 };
 
 static bool tst_test_script_run_registered
